struct _IO_FILE;
struct _IO_FILE *_coverage_fout  ;
enum YYCONDTYPE {
    yycST_IN_SCRIPTING = 0,
    yycST_LOOKING_FOR_PROPERTY = 1,
    yycST_BACKQUOTE = 2,
    yycST_DOUBLE_QUOTES = 3,
    yycST_HEREDOC = 4,
    yycST_LOOKING_FOR_VARNAME = 5,
    yycST_VAR_OFFSET = 6,
    yycINITIAL = 7,
    yycST_END_HEREDOC = 8,
    yycST_NOWDOC = 9
} ;
typedef unsigned int size_t;
typedef long wchar_t;
struct __anonstruct___wait_terminated_1 {
   unsigned int __w_termsig : 7 ;
   unsigned int __w_coredump : 1 ;
   unsigned int __w_retcode : 8 ;
   unsigned int  : 16 ;
};
struct __anonstruct___wait_stopped_2 {
   unsigned int __w_stopval : 8 ;
   unsigned int __w_stopsig : 8 ;
   unsigned int  : 16 ;
};
union wait {
   int w_status ;
   struct __anonstruct___wait_terminated_1 __wait_terminated ;
   struct __anonstruct___wait_stopped_2 __wait_stopped ;
};
union __anonunion___WAIT_STATUS_3 {
   union wait *__uptr ;
   int *__iptr ;
};
typedef union __anonunion___WAIT_STATUS_3  __attribute__((__transparent_union__)) __WAIT_STATUS;
struct __anonstruct_div_t_4 {
   int quot ;
   int rem ;
};
typedef struct __anonstruct_div_t_4 div_t;
struct __anonstruct_ldiv_t_5 {
   long quot ;
   long rem ;
};
typedef struct __anonstruct_ldiv_t_5 ldiv_t;
struct __anonstruct_lldiv_t_6 {
   long long quot ;
   long long rem ;
};
typedef struct __anonstruct_lldiv_t_6 lldiv_t;
typedef unsigned char __u_char;
typedef unsigned short __u_short;
typedef unsigned int __u_int;
typedef unsigned long __u_long;
typedef signed char __int8_t;
typedef unsigned char __uint8_t;
typedef short __int16_t;
typedef unsigned short __uint16_t;
typedef int __int32_t;
typedef unsigned int __uint32_t;
typedef long long __int64_t;
typedef unsigned long long __uint64_t;
typedef long long __quad_t;
typedef unsigned long long __u_quad_t;
typedef __u_quad_t __dev_t;
typedef unsigned int __uid_t;
typedef unsigned int __gid_t;
typedef unsigned long __ino_t;
typedef __u_quad_t __ino64_t;
typedef unsigned int __mode_t;
typedef unsigned int __nlink_t;
typedef long __off_t;
typedef __quad_t __off64_t;
typedef int __pid_t;
struct __anonstruct___fsid_t_7 {
   int __val[2] ;
};
typedef struct __anonstruct___fsid_t_7 __fsid_t;
typedef long __clock_t;
typedef unsigned long __rlim_t;
typedef __u_quad_t __rlim64_t;
typedef unsigned int __id_t;
typedef long __time_t;
typedef unsigned int __useconds_t;
typedef long __suseconds_t;
typedef int __daddr_t;
typedef long __swblk_t;
typedef int __key_t;
typedef int __clockid_t;
typedef void *__timer_t;
typedef long __blksize_t;
typedef long __blkcnt_t;
typedef __quad_t __blkcnt64_t;
typedef unsigned long __fsblkcnt_t;
typedef __u_quad_t __fsblkcnt64_t;
typedef unsigned long __fsfilcnt_t;
typedef __u_quad_t __fsfilcnt64_t;
typedef int __ssize_t;
typedef __off64_t __loff_t;
typedef __quad_t *__qaddr_t;
typedef char *__caddr_t;
typedef int __intptr_t;
typedef unsigned int __socklen_t;
typedef __u_char u_char;
typedef __u_short u_short;
typedef __u_int u_int;
typedef __u_long u_long;
typedef __quad_t quad_t;
typedef __u_quad_t u_quad_t;
typedef __fsid_t fsid_t;
typedef __loff_t loff_t;
typedef __ino_t ino_t;
typedef __dev_t dev_t;
typedef __gid_t gid_t;
typedef __mode_t mode_t;
typedef __nlink_t nlink_t;
typedef __uid_t uid_t;
typedef __off_t off_t;
typedef __pid_t pid_t;
typedef __id_t id_t;
typedef __ssize_t ssize_t;
typedef __daddr_t daddr_t;
typedef __caddr_t caddr_t;
typedef __key_t key_t;
typedef __clock_t clock_t;
typedef __time_t time_t;
typedef __clockid_t clockid_t;
typedef __timer_t timer_t;
typedef unsigned long ulong;
typedef unsigned short ushort;
typedef unsigned int uint;
typedef signed char int8_t;
typedef short int16_t;
typedef int int32_t;
typedef long long int64_t;
typedef unsigned char u_int8_t;
typedef unsigned short u_int16_t;
typedef unsigned int u_int32_t;
typedef unsigned long long u_int64_t;
typedef int register_t;
typedef int __sig_atomic_t;
struct __anonstruct___sigset_t_8 {
   unsigned long __val[1024U / (8U * sizeof(unsigned long ))] ;
};
typedef struct __anonstruct___sigset_t_8 __sigset_t;
typedef __sigset_t sigset_t;
struct timespec {
   __time_t tv_sec ;
   long tv_nsec ;
};
struct timeval {
   __time_t tv_sec ;
   __suseconds_t tv_usec ;
};
typedef __suseconds_t suseconds_t;
typedef long __fd_mask;
struct __anonstruct_fd_set_9 {
   __fd_mask __fds_bits[1024 / (8 * (int )sizeof(__fd_mask ))] ;
};
typedef struct __anonstruct_fd_set_9 fd_set;
typedef __fd_mask fd_mask;
typedef __blksize_t blksize_t;
typedef __blkcnt_t blkcnt_t;
typedef __fsblkcnt_t fsblkcnt_t;
typedef __fsfilcnt_t fsfilcnt_t;
typedef unsigned long pthread_t;
union __anonunion_pthread_attr_t_10 {
   char __size[36] ;
   long __align ;
};
typedef union __anonunion_pthread_attr_t_10 pthread_attr_t;
struct __pthread_internal_slist {
   struct __pthread_internal_slist *__next ;
};
typedef struct __pthread_internal_slist __pthread_slist_t;
union __anonunion____missing_field_name_12 {
   int __spins ;
   __pthread_slist_t __list ;
};
struct __pthread_mutex_s {
   int __lock ;
   unsigned int __count ;
   int __owner ;
   int __kind ;
   unsigned int __nusers ;
   union __anonunion____missing_field_name_12 __annonCompField1 ;
};
union __anonunion_pthread_mutex_t_11 {
   struct __pthread_mutex_s __data ;
   char __size[24] ;
   long __align ;
};
typedef union __anonunion_pthread_mutex_t_11 pthread_mutex_t;
union __anonunion_pthread_mutexattr_t_13 {
   char __size[4] ;
   long __align ;
};
typedef union __anonunion_pthread_mutexattr_t_13 pthread_mutexattr_t;
struct __anonstruct___data_15 {
   int __lock ;
   unsigned int __futex ;
   unsigned long long __total_seq ;
   unsigned long long __wakeup_seq ;
   unsigned long long __woken_seq ;
   void *__mutex ;
   unsigned int __nwaiters ;
   unsigned int __broadcast_seq ;
};
union __anonunion_pthread_cond_t_14 {
   struct __anonstruct___data_15 __data ;
   char __size[48] ;
   long long __align ;
};
typedef union __anonunion_pthread_cond_t_14 pthread_cond_t;
union __anonunion_pthread_condattr_t_16 {
   char __size[4] ;
   long __align ;
};
typedef union __anonunion_pthread_condattr_t_16 pthread_condattr_t;
typedef unsigned int pthread_key_t;
typedef int pthread_once_t;
struct __anonstruct___data_18 {
   int __lock ;
   unsigned int __nr_readers ;
   unsigned int __readers_wakeup ;
   unsigned int __writer_wakeup ;
   unsigned int __nr_readers_queued ;
   unsigned int __nr_writers_queued ;
   unsigned char __flags ;
   unsigned char __shared ;
   unsigned char __pad1 ;
   unsigned char __pad2 ;
   int __writer ;
};
union __anonunion_pthread_rwlock_t_17 {
   struct __anonstruct___data_18 __data ;
   char __size[32] ;
   long __align ;
};
typedef union __anonunion_pthread_rwlock_t_17 pthread_rwlock_t;
union __anonunion_pthread_rwlockattr_t_19 {
   char __size[8] ;
   long __align ;
};
typedef union __anonunion_pthread_rwlockattr_t_19 pthread_rwlockattr_t;
typedef int volatile   pthread_spinlock_t;
union __anonunion_pthread_barrier_t_20 {
   char __size[20] ;
   long __align ;
};
typedef union __anonunion_pthread_barrier_t_20 pthread_barrier_t;
union __anonunion_pthread_barrierattr_t_21 {
   char __size[4] ;
   int __align ;
};
typedef union __anonunion_pthread_barrierattr_t_21 pthread_barrierattr_t;
struct random_data {
   int32_t *fptr ;
   int32_t *rptr ;
   int32_t *state ;
   int rand_type ;
   int rand_deg ;
   int rand_sep ;
   int32_t *end_ptr ;
};
struct drand48_data {
   unsigned short __x[3] ;
   unsigned short __old_x[3] ;
   unsigned short __c ;
   unsigned short __init ;
   unsigned long long __a ;
};
typedef int (*__compar_fn_t)(void const   * , void const   * );
struct __locale_data;
struct __locale_struct {
   struct __locale_data *__locales[13] ;
   unsigned short const   *__ctype_b ;
   int const   *__ctype_tolower ;
   int const   *__ctype_toupper ;
   char const   *__names[13] ;
};
typedef struct __locale_struct *__locale_t;
typedef __locale_t locale_t;
typedef long double float_t;
typedef long double double_t;
enum __anonenum_22 {
    FP_NAN = 0,
    FP_INFINITE = 1,
    FP_ZERO = 2,
    FP_SUBNORMAL = 3,
    FP_NORMAL = 4
} ;
enum __anonenum__LIB_VERSION_TYPE_23 {
    _IEEE_ = -1,
    _SVID_ = 0,
    _XOPEN_ = 1,
    _POSIX_ = 2,
    _ISOC_ = 3
} ;
typedef enum __anonenum__LIB_VERSION_TYPE_23 _LIB_VERSION_TYPE;
struct exception {
   int type ;
   char *name ;
   double arg1 ;
   double arg2 ;
   double retval ;
};
union __anonunion___u_24 {
   float __f ;
   int __i ;
};
union __anonunion___u_25 {
   double __d ;
   int __i[2] ;
};
union __anonunion___u_26 {
   long double __l ;
   int __i[3] ;
};
union __anonunion___n_27 {
   long double __xld ;
   unsigned int __xi[3] ;
};
union __anonunion_28 {
   double __d ;
   int __i[2] ;
};
typedef struct _IO_FILE FILE;
typedef struct _IO_FILE __FILE;
union __anonunion___value_30 {
   unsigned int __wch ;
   char __wchb[4] ;
};
struct __anonstruct___mbstate_t_29 {
   int __count ;
   union __anonunion___value_30 __value ;
};
typedef struct __anonstruct___mbstate_t_29 __mbstate_t;
struct __anonstruct__G_fpos_t_31 {
   __off_t __pos ;
   __mbstate_t __state ;
};
typedef struct __anonstruct__G_fpos_t_31 _G_fpos_t;
struct __anonstruct__G_fpos64_t_32 {
   __off64_t __pos ;
   __mbstate_t __state ;
};
typedef struct __anonstruct__G_fpos64_t_32 _G_fpos64_t;
typedef short _G_int16_t;
typedef int _G_int32_t;
typedef unsigned short _G_uint16_t;
typedef unsigned int _G_uint32_t;
typedef __builtin_va_list __gnuc_va_list;
struct _IO_jump_t;
typedef void _IO_lock_t;
struct _IO_marker {
   struct _IO_marker *_next ;
   struct _IO_FILE *_sbuf ;
   int _pos ;
};
enum __codecvt_result {
    __codecvt_ok = 0,
    __codecvt_partial = 1,
    __codecvt_error = 2,
    __codecvt_noconv = 3
} ;
struct _IO_FILE {
   int _flags ;
   char *_IO_read_ptr ;
   char *_IO_read_end ;
   char *_IO_read_base ;
   char *_IO_write_base ;
   char *_IO_write_ptr ;
   char *_IO_write_end ;
   char *_IO_buf_base ;
   char *_IO_buf_end ;
   char *_IO_save_base ;
   char *_IO_backup_base ;
   char *_IO_save_end ;
   struct _IO_marker *_markers ;
   struct _IO_FILE *_chain ;
   int _fileno ;
   int _flags2 ;
   __off_t _old_offset ;
   unsigned short _cur_column ;
   signed char _vtable_offset ;
   char _shortbuf[1] ;
   _IO_lock_t *_lock ;
   __off64_t _offset ;
   void *__pad1 ;
   void *__pad2 ;
   void *__pad3 ;
   void *__pad4 ;
   size_t __pad5 ;
   int _mode ;
   char _unused2[(15U * sizeof(int ) - 4U * sizeof(void *)) - sizeof(size_t )] ;
};
typedef struct _IO_FILE _IO_FILE;
struct _IO_FILE_plus;
typedef __ssize_t __io_read_fn(void *__cookie , char *__buf , size_t __nbytes );
typedef __ssize_t __io_write_fn(void *__cookie , char const   *__buf ,
                                size_t __n );
typedef int __io_seek_fn(void *__cookie , __off64_t *__pos , int __w );
typedef int __io_close_fn(void *__cookie );
typedef __io_read_fn cookie_read_function_t;
typedef __io_write_fn cookie_write_function_t;
typedef __io_seek_fn cookie_seek_function_t;
typedef __io_close_fn cookie_close_function_t;
struct __anonstruct__IO_cookie_io_functions_t_33 {
   __io_read_fn *read ;
   __io_write_fn *write ;
   __io_seek_fn *seek ;
   __io_close_fn *close ;
};
typedef struct __anonstruct__IO_cookie_io_functions_t_33 _IO_cookie_io_functions_t;
typedef _IO_cookie_io_functions_t cookie_io_functions_t;
struct _IO_cookie_file;
typedef __gnuc_va_list va_list;
typedef _G_fpos_t fpos_t;
typedef long tsrm_intptr_t;
typedef unsigned long tsrm_uintptr_t;
struct _zend_leak_info {
   void *addr ;
   size_t size ;
   char const   *filename ;
   uint lineno ;
   char const   *orig_filename ;
   uint orig_lineno ;
};
typedef struct _zend_leak_info zend_leak_info;
struct _zend_mm_heap;
typedef struct _zend_mm_heap zend_mm_heap;
struct _zend_mm_storage;
typedef struct _zend_mm_storage zend_mm_storage;
struct _zend_mm_segment {
   size_t size ;
   struct _zend_mm_segment *next_segment ;
};
typedef struct _zend_mm_segment zend_mm_segment;
struct _zend_mm_mem_handlers {
   char const   *name ;
   zend_mm_storage *(*init)(void *params ) ;
   void (*dtor)(zend_mm_storage *storage ) ;
   void (*compact)(zend_mm_storage *storage ) ;
   zend_mm_segment *(*_alloc)(zend_mm_storage *storage , size_t size ) ;
   zend_mm_segment *(*_realloc)(zend_mm_storage *storage ,
                                zend_mm_segment *ptr , size_t size ) ;
   void (*_free)(zend_mm_storage *storage , zend_mm_segment *ptr ) ;
};
typedef struct _zend_mm_mem_handlers zend_mm_mem_handlers;
struct _zend_mm_storage {
   zend_mm_mem_handlers const   *handlers ;
   void *data ;
};
typedef unsigned char zend_bool;
typedef unsigned char zend_uchar;
typedef unsigned int zend_uint;
typedef unsigned long zend_ulong;
typedef unsigned short zend_ushort;
typedef long long zend_long64;
typedef unsigned long long zend_ulong64;
typedef long zend_intptr_t;
typedef unsigned long zend_uintptr_t;
typedef unsigned int zend_object_handle;
struct _zend_object_handlers;
typedef struct _zend_object_handlers zend_object_handlers;
struct _zend_object_value {
   zend_object_handle handle ;
   zend_object_handlers const   *handlers ;
};
typedef struct _zend_object_value zend_object_value;
typedef ulong (*hash_func_t)(char const   *arKey , uint nKeyLength );
typedef int (*compare_func_t)(void const   * , void const   * );
typedef void (*sort_func_t)(void * , size_t  , size_t  ,
                            int (*)(void const   * , void const   * ) );
typedef void (*dtor_func_t)(void *pDest );
typedef void (*copy_ctor_func_t)(void *pElement );
typedef void (*copy_ctor_param_func_t)(void *pElement , void *pParam );
struct _hashtable;
struct bucket {
   ulong h ;
   uint nKeyLength ;
   void *pData ;
   void *pDataPtr ;
   struct bucket *pListNext ;
   struct bucket *pListLast ;
   struct bucket *pNext ;
   struct bucket *pLast ;
   char const   *arKey ;
};
typedef struct bucket Bucket;
struct _hashtable {
   uint nTableSize ;
   uint nTableMask ;
   uint nNumOfElements ;
   ulong nNextFreeElement ;
   Bucket *pInternalPointer ;
   Bucket *pListHead ;
   Bucket *pListTail ;
   Bucket **arBuckets ;
   void (*pDestructor)(void *pDest ) ;
   zend_bool persistent ;
   unsigned char nApplyCount ;
   zend_bool bApplyProtection ;
};
typedef struct _hashtable HashTable;
struct _zend_hash_key {
   char const   *arKey ;
   uint nKeyLength ;
   ulong h ;
};
typedef struct _zend_hash_key zend_hash_key;
typedef zend_bool (*merge_checker_func_t)(HashTable *target_ht ,
                                          void *source_data ,
                                          zend_hash_key *hash_key ,
                                          void *pParam );
typedef Bucket *HashPosition;
typedef int (*apply_func_t)(void *pDest );
typedef int (*apply_func_arg_t)(void *pDest , void *argument );
typedef int (*apply_func_args_t)(void *pDest , int num_args , va_list args ,
                                 zend_hash_key *hash_key );
struct _HashPointer {
   HashPosition pos ;
   ulong h ;
};
typedef struct _HashPointer HashPointer;
struct _zend_ts_hashtable {
   HashTable hash ;
   zend_uint reader ;
};
typedef struct _zend_ts_hashtable TsHashTable;
struct _zend_llist_element {
   struct _zend_llist_element *next ;
   struct _zend_llist_element *prev ;
   char data[1] ;
};
typedef struct _zend_llist_element zend_llist_element;
typedef void (*llist_dtor_func_t)(void * );
typedef int (*llist_compare_func_t)(zend_llist_element const   ** ,
                                    zend_llist_element const   ** );
typedef void (*llist_apply_with_args_func_t)(void *data , int num_args ,
                                             va_list args );
typedef void (*llist_apply_with_arg_func_t)(void *data , void *arg );
typedef void (*llist_apply_func_t)(void * );
struct _zend_llist {
   zend_llist_element *head ;
   zend_llist_element *tail ;
   size_t count ;
   size_t size ;
   void (*dtor)(void * ) ;
   unsigned char persistent ;
   zend_llist_element *traverse_ptr ;
};
typedef struct _zend_llist zend_llist;
typedef zend_llist_element *zend_llist_position;
struct _zval_struct;
typedef struct _zval_struct zval;
struct _zend_class_entry;
typedef struct _zend_class_entry zend_class_entry;
struct _zend_guard {
   zend_bool in_get ;
   zend_bool in_set ;
   zend_bool in_unset ;
   zend_bool in_isset ;
   zend_bool dummy ;
};
typedef struct _zend_guard zend_guard;
struct _zend_object {
   zend_class_entry *ce ;
   HashTable *properties ;
   zval **properties_table ;
   HashTable *guards ;
};
typedef struct _zend_object zend_object;
union _zend_function;
struct _zend_property_info;
struct _zend_literal;
typedef zval *(*zend_object_read_property_t)(zval *object , zval *member ,
                                             int type ,
                                             struct _zend_literal  const  *key );
typedef zval *(*zend_object_read_dimension_t)(zval *object , zval *offset ,
                                              int type );
typedef void (*zend_object_write_property_t)(zval *object , zval *member ,
                                             zval *value ,
                                             struct _zend_literal  const  *key );
typedef void (*zend_object_write_dimension_t)(zval *object , zval *offset ,
                                              zval *value );
typedef zval **(*zend_object_get_property_ptr_ptr_t)(zval *object ,
                                                     zval *member ,
                                                     struct _zend_literal  const  *key );
typedef void (*zend_object_set_t)(zval **object , zval *value );
typedef zval *(*zend_object_get_t)(zval *object );
typedef int (*zend_object_has_property_t)(zval *object , zval *member ,
                                          int has_set_exists ,
                                          struct _zend_literal  const  *key );
typedef int (*zend_object_has_dimension_t)(zval *object , zval *member ,
                                           int check_empty );
typedef void (*zend_object_unset_property_t)(zval *object , zval *member ,
                                             struct _zend_literal  const  *key );
typedef void (*zend_object_unset_dimension_t)(zval *object , zval *offset );
typedef HashTable *(*zend_object_get_properties_t)(zval *object );
typedef HashTable *(*zend_object_get_debug_info_t)(zval *object , int *is_temp );
typedef int (*zend_object_call_method_t)(char const   *method , int ht ,
                                         zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used );
typedef union _zend_function *(*zend_object_get_method_t)(zval **object_ptr ,
                                                          char *method ,
                                                          int method_len ,
                                                          struct _zend_literal  const  *key );
typedef union _zend_function *(*zend_object_get_constructor_t)(zval *object );
typedef void (*zend_object_add_ref_t)(zval *object );
typedef void (*zend_object_del_ref_t)(zval *object );
typedef void (*zend_object_delete_obj_t)(zval *object );
typedef zend_object_value (*zend_object_clone_obj_t)(zval *object );
typedef zend_class_entry *(*zend_object_get_class_entry_t)(zval const   *object );
typedef int (*zend_object_get_class_name_t)(zval const   *object ,
                                            char const   **class_name ,
                                            zend_uint *class_name_len ,
                                            int parent );
typedef int (*zend_object_compare_t)(zval *object1 , zval *object2 );
typedef int (*zend_object_cast_t)(zval *readobj , zval *retval , int type );
typedef int (*zend_object_count_elements_t)(zval *object , long *count );
typedef int (*zend_object_get_closure_t)(zval *obj , zend_class_entry **ce_ptr ,
                                         union _zend_function **fptr_ptr ,
                                         zval **zobj_ptr );
typedef HashTable *(*zend_object_get_gc_t)(zval *object , zval ***table ,
                                           int *n );
struct _zend_object_handlers {
   void (*add_ref)(zval *object ) ;
   void (*del_ref)(zval *object ) ;
   zend_object_value (*clone_obj)(zval *object ) ;
   zval *(*read_property)(zval *object , zval *member , int type ,
                          struct _zend_literal  const  *key ) ;
   void (*write_property)(zval *object , zval *member , zval *value ,
                          struct _zend_literal  const  *key ) ;
   zval *(*read_dimension)(zval *object , zval *offset , int type ) ;
   void (*write_dimension)(zval *object , zval *offset , zval *value ) ;
   zval **(*get_property_ptr_ptr)(zval *object , zval *member ,
                                  struct _zend_literal  const  *key ) ;
   zval *(*get)(zval *object ) ;
   void (*set)(zval **object , zval *value ) ;
   int (*has_property)(zval *object , zval *member , int has_set_exists ,
                       struct _zend_literal  const  *key ) ;
   void (*unset_property)(zval *object , zval *member ,
                          struct _zend_literal  const  *key ) ;
   int (*has_dimension)(zval *object , zval *member , int check_empty ) ;
   void (*unset_dimension)(zval *object , zval *offset ) ;
   HashTable *(*get_properties)(zval *object ) ;
   union _zend_function *(*get_method)(zval **object_ptr , char *method ,
                                       int method_len ,
                                       struct _zend_literal  const  *key ) ;
   int (*call_method)(char const   *method , int ht , zval *return_value ,
                      zval **return_value_ptr , zval *this_ptr ,
                      int return_value_used ) ;
   union _zend_function *(*get_constructor)(zval *object ) ;
   zend_class_entry *(*get_class_entry)(zval const   *object ) ;
   int (*get_class_name)(zval const   *object , char const   **class_name ,
                         zend_uint *class_name_len , int parent ) ;
   int (*compare_objects)(zval *object1 , zval *object2 ) ;
   int (*cast_object)(zval *readobj , zval *retval , int type ) ;
   int (*count_elements)(zval *object , long *count ) ;
   HashTable *(*get_debug_info)(zval *object , int *is_temp ) ;
   int (*get_closure)(zval *obj , zend_class_entry **ce_ptr ,
                      union _zend_function **fptr_ptr , zval **zobj_ptr ) ;
   HashTable *(*get_gc)(zval *object , zval ***table , int *n ) ;
};
struct __anonstruct_str_34 {
   char *val ;
   int len ;
};
union _zvalue_value {
   long lval ;
   double dval ;
   struct __anonstruct_str_34 str ;
   HashTable *ht ;
   zend_object_value obj ;
};
typedef union _zvalue_value zvalue_value;
struct _zval_struct {
   zvalue_value value ;
   zend_uint refcount__gc ;
   zend_uchar type ;
   zend_uchar is_ref__gc ;
};
struct _zend_object_iterator;
typedef struct _zend_object_iterator zend_object_iterator;
struct _zend_object_iterator_funcs {
   void (*dtor)(zend_object_iterator *iter ) ;
   int (*valid)(zend_object_iterator *iter ) ;
   void (*get_current_data)(zend_object_iterator *iter , zval ***data ) ;
   int (*get_current_key)(zend_object_iterator *iter , char **str_key ,
                          uint *str_key_len , ulong *int_key ) ;
   void (*move_forward)(zend_object_iterator *iter ) ;
   void (*rewind)(zend_object_iterator *iter ) ;
   void (*invalidate_current)(zend_object_iterator *iter ) ;
};
typedef struct _zend_object_iterator_funcs zend_object_iterator_funcs;
struct _zend_object_iterator {
   void *data ;
   zend_object_iterator_funcs *funcs ;
   ulong index ;
};
struct _zend_class_iterator_funcs {
   zend_object_iterator_funcs *funcs ;
   union _zend_function *zf_new_iterator ;
   union _zend_function *zf_valid ;
   union _zend_function *zf_current ;
   union _zend_function *zf_key ;
   union _zend_function *zf_next ;
   union _zend_function *zf_rewind ;
};
typedef struct _zend_class_iterator_funcs zend_class_iterator_funcs;
enum zend_object_iterator_kind {
    ZEND_ITER_INVALID = 0,
    ZEND_ITER_PLAIN_ARRAY = 1,
    ZEND_ITER_PLAIN_OBJECT = 2,
    ZEND_ITER_OBJECT = 3
} ;
struct _zend_serialize_data;
struct _zend_unserialize_data;
typedef struct _zend_serialize_data zend_serialize_data;
typedef struct _zend_unserialize_data zend_unserialize_data;
struct _zend_trait_method_reference {
   char const   *method_name ;
   unsigned int mname_len ;
   zend_class_entry *ce ;
   char const   *class_name ;
   unsigned int cname_len ;
};
typedef struct _zend_trait_method_reference zend_trait_method_reference;
struct _zend_trait_precedence {
   zend_trait_method_reference *trait_method ;
   zend_class_entry **exclude_from_classes ;
   union _zend_function *function ;
};
typedef struct _zend_trait_precedence zend_trait_precedence;
struct _zend_trait_alias {
   zend_trait_method_reference *trait_method ;
   char const   *alias ;
   unsigned int alias_len ;
   zend_uint modifiers ;
   union _zend_function *function ;
};
typedef struct _zend_trait_alias zend_trait_alias;
struct __anonstruct_user_36 {
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
};
struct _zend_function_entry;
struct _zend_module_entry;
struct __anonstruct_internal_37 {
   struct _zend_function_entry  const  *builtin_functions ;
   struct _zend_module_entry *module ;
};
union __anonunion_info_35 {
   struct __anonstruct_user_36 user ;
   struct __anonstruct_internal_37 internal ;
};
struct _zend_class_entry {
   char type ;
   char const   *name ;
   zend_uint name_length ;
   struct _zend_class_entry *parent ;
   int refcount ;
   zend_uint ce_flags ;
   HashTable function_table ;
   HashTable properties_info ;
   zval **default_properties_table ;
   zval **default_static_members_table ;
   zval **static_members_table ;
   HashTable constants_table ;
   int default_properties_count ;
   int default_static_members_count ;
   union _zend_function *constructor ;
   union _zend_function *destructor ;
   union _zend_function *clone ;
   union _zend_function *__get ;
   union _zend_function *__set ;
   union _zend_function *__unset ;
   union _zend_function *__isset ;
   union _zend_function *__call ;
   union _zend_function *__callstatic ;
   union _zend_function *__tostring ;
   union _zend_function *serialize_func ;
   union _zend_function *unserialize_func ;
   zend_class_iterator_funcs iterator_funcs ;
   zend_object_value (*create_object)(zend_class_entry *class_type ) ;
   zend_object_iterator *(*get_iterator)(zend_class_entry *ce , zval *object ,
                                         int by_ref ) ;
   int (*interface_gets_implemented)(zend_class_entry *iface ,
                                     zend_class_entry *class_type ) ;
   union _zend_function *(*get_static_method)(zend_class_entry *ce ,
                                              char *method , int method_len ) ;
   int (*serialize)(zval *object , unsigned char **buffer , zend_uint *buf_len ,
                    zend_serialize_data *data ) ;
   int (*unserialize)(zval **object , zend_class_entry *ce ,
                      unsigned char const   *buf , zend_uint buf_len ,
                      zend_unserialize_data *data ) ;
   zend_class_entry **interfaces ;
   zend_uint num_interfaces ;
   zend_class_entry **traits ;
   zend_uint num_traits ;
   zend_trait_alias **trait_aliases ;
   zend_trait_precedence **trait_precedences ;
   union __anonunion_info_35 info ;
};
typedef size_t (*zend_stream_fsizer_t)(void *handle );
typedef size_t (*zend_stream_reader_t)(void *handle , char *buf , size_t len );
typedef void (*zend_stream_closer_t)(void *handle );
enum __anonenum_zend_stream_type_38 {
    ZEND_HANDLE_FILENAME = 0,
    ZEND_HANDLE_FD = 1,
    ZEND_HANDLE_FP = 2,
    ZEND_HANDLE_STREAM = 3,
    ZEND_HANDLE_MAPPED = 4
} ;
typedef enum __anonenum_zend_stream_type_38 zend_stream_type;
struct _zend_mmap {
   size_t len ;
   size_t pos ;
   void *map ;
   char *buf ;
   void *old_handle ;
   void (*old_closer)(void *handle ) ;
};
typedef struct _zend_mmap zend_mmap;
struct _zend_stream {
   void *handle ;
   int isatty ;
   zend_mmap mmap ;
   size_t (*reader)(void *handle , char *buf , size_t len ) ;
   size_t (*fsizer)(void *handle ) ;
   void (*closer)(void *handle ) ;
};
typedef struct _zend_stream zend_stream;
union __anonunion_handle_39 {
   int fd ;
   FILE *fp ;
   zend_stream stream ;
};
struct _zend_file_handle {
   zend_stream_type type ;
   char const   *filename ;
   char *opened_path ;
   union __anonunion_handle_39 handle ;
   zend_bool free_filename ;
};
typedef struct _zend_file_handle zend_file_handle;
struct _zend_utility_functions {
   void ( /* format attribute */  (*error_function))(int type ,
                                                     char const   *error_filename ,
                                                     uint error_lineno ,
                                                     char const   *format ,
                                                     va_list args ) ;
   int ( /* format attribute */  (*printf_function))(char const   *format  , ...) ;
   int (*write_function)(char const   *str , uint str_length ) ;
   FILE *(*fopen_function)(char const   *filename , char **opened_path ) ;
   void (*message_handler)(long message , void const   *data ) ;
   void (*block_interruptions)(void) ;
   void (*unblock_interruptions)(void) ;
   int (*get_configuration_directive)(char const   *name , uint name_length ,
                                      zval *contents ) ;
   void (*ticks_function)(int ticks ) ;
   void (*on_timeout)(int seconds ) ;
   int (*stream_open_function)(char const   *filename ,
                               zend_file_handle *handle ) ;
   int (*vspprintf_function)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
   char *(*getenv_function)(char *name , size_t name_len ) ;
   char *(*resolve_path_function)(char const   *filename , int filename_len ) ;
};
typedef struct _zend_utility_functions zend_utility_functions;
struct _zend_utility_values {
   char *import_use_extension ;
   uint import_use_extension_length ;
   zend_bool html_errors ;
};
typedef struct _zend_utility_values zend_utility_values;
typedef int (*zend_write_func_t)(char const   *str , uint str_length );
union __anonunion_u_40 {
   zval *pz ;
   zend_object_handlers const   *handlers ;
};
struct _gc_root_buffer {
   struct _gc_root_buffer *prev ;
   struct _gc_root_buffer *next ;
   zend_object_handle handle ;
   union __anonunion_u_40 u ;
};
typedef struct _gc_root_buffer gc_root_buffer;
struct _zval_gc_info;
union __anonunion_u_41 {
   gc_root_buffer *buffered ;
   struct _zval_gc_info *next ;
};
struct _zval_gc_info {
   zval z ;
   union __anonunion_u_41 u ;
};
typedef struct _zval_gc_info zval_gc_info;
struct _zend_gc_globals {
   zend_bool gc_enabled ;
   zend_bool gc_active ;
   gc_root_buffer *buf ;
   gc_root_buffer roots ;
   gc_root_buffer *unused ;
   gc_root_buffer *first_unused ;
   gc_root_buffer *last_unused ;
   zval_gc_info *zval_to_free ;
   zval_gc_info *free_list ;
   zval_gc_info *next_to_free ;
   zend_uint gc_runs ;
   zend_uint collected ;
};
typedef struct _zend_gc_globals zend_gc_globals;
enum __anonenum_zend_error_handling_t_42 {
    EH_NORMAL = 0,
    EH_SUPPRESS = 1,
    EH_THROW = 2
} ;
typedef enum __anonenum_zend_error_handling_t_42 zend_error_handling_t;
struct __anonstruct_zend_error_handling_43 {
   zend_error_handling_t handling ;
   zend_class_entry *exception ;
   zval *user_handler ;
};
typedef struct __anonstruct_zend_error_handling_43 zend_error_handling;
enum yytokentype {
    END = 0,
    T_REQUIRE_ONCE = 258,
    T_REQUIRE = 259,
    T_EVAL = 260,
    T_INCLUDE_ONCE = 261,
    T_INCLUDE = 262,
    T_LOGICAL_OR = 263,
    T_LOGICAL_XOR = 264,
    T_LOGICAL_AND = 265,
    T_PRINT = 266,
    T_SR_EQUAL = 267,
    T_SL_EQUAL = 268,
    T_XOR_EQUAL = 269,
    T_OR_EQUAL = 270,
    T_AND_EQUAL = 271,
    T_MOD_EQUAL = 272,
    T_CONCAT_EQUAL = 273,
    T_DIV_EQUAL = 274,
    T_MUL_EQUAL = 275,
    T_MINUS_EQUAL = 276,
    T_PLUS_EQUAL = 277,
    T_BOOLEAN_OR = 278,
    T_BOOLEAN_AND = 279,
    T_IS_NOT_IDENTICAL = 280,
    T_IS_IDENTICAL = 281,
    T_IS_NOT_EQUAL = 282,
    T_IS_EQUAL = 283,
    T_IS_GREATER_OR_EQUAL = 284,
    T_IS_SMALLER_OR_EQUAL = 285,
    T_SR = 286,
    T_SL = 287,
    T_INSTANCEOF = 288,
    T_UNSET_CAST = 289,
    T_BOOL_CAST = 290,
    T_OBJECT_CAST = 291,
    T_ARRAY_CAST = 292,
    T_STRING_CAST = 293,
    T_DOUBLE_CAST = 294,
    T_INT_CAST = 295,
    T_DEC = 296,
    T_INC = 297,
    T_CLONE = 298,
    T_NEW = 299,
    T_EXIT = 300,
    T_IF = 301,
    T_ELSEIF = 302,
    T_ELSE = 303,
    T_ENDIF = 304,
    T_LNUMBER = 305,
    T_DNUMBER = 306,
    T_STRING = 307,
    T_STRING_VARNAME = 308,
    T_VARIABLE = 309,
    T_NUM_STRING = 310,
    T_INLINE_HTML = 311,
    T_CHARACTER = 312,
    T_BAD_CHARACTER = 313,
    T_ENCAPSED_AND_WHITESPACE = 314,
    T_CONSTANT_ENCAPSED_STRING = 315,
    T_ECHO = 316,
    T_DO = 317,
    T_WHILE = 318,
    T_ENDWHILE = 319,
    T_FOR = 320,
    T_ENDFOR = 321,
    T_FOREACH = 322,
    T_ENDFOREACH = 323,
    T_DECLARE = 324,
    T_ENDDECLARE = 325,
    T_AS = 326,
    T_SWITCH = 327,
    T_ENDSWITCH = 328,
    T_CASE = 329,
    T_DEFAULT = 330,
    T_BREAK = 331,
    T_CONTINUE = 332,
    T_GOTO = 333,
    T_FUNCTION = 334,
    T_CONST = 335,
    T_RETURN = 336,
    T_TRY = 337,
    T_CATCH = 338,
    T_THROW = 339,
    T_USE = 340,
    T_INSTEADOF = 341,
    T_GLOBAL = 342,
    T_PUBLIC = 343,
    T_PROTECTED = 344,
    T_PRIVATE = 345,
    T_FINAL = 346,
    T_ABSTRACT = 347,
    T_STATIC = 348,
    T_VAR = 349,
    T_UNSET = 350,
    T_ISSET = 351,
    T_EMPTY = 352,
    T_HALT_COMPILER = 353,
    T_CLASS = 354,
    T_TRAIT = 355,
    T_INTERFACE = 356,
    T_EXTENDS = 357,
    T_IMPLEMENTS = 358,
    T_OBJECT_OPERATOR = 359,
    T_DOUBLE_ARROW = 360,
    T_LIST = 361,
    T_ARRAY = 362,
    T_CALLABLE = 363,
    T_CLASS_C = 364,
    T_TRAIT_C = 365,
    T_METHOD_C = 366,
    T_FUNC_C = 367,
    T_LINE = 368,
    T_FILE = 369,
    T_COMMENT = 370,
    T_DOC_COMMENT = 371,
    T_OPEN_TAG = 372,
    T_OPEN_TAG_WITH_ECHO = 373,
    T_CLOSE_TAG = 374,
    T_WHITESPACE = 375,
    T_START_HEREDOC = 376,
    T_END_HEREDOC = 377,
    T_DOLLAR_OPEN_CURLY_BRACES = 378,
    T_CURLY_OPEN = 379,
    T_PAAMAYIM_NEKUDOTAYIM = 380,
    T_NAMESPACE = 381,
    T_NS_C = 382,
    T_DIR = 383,
    T_NS_SEPARATOR = 384
} ;
typedef int YYSTYPE;
struct _zend_op_array;
typedef struct _zend_op_array zend_op_array;
struct _zend_op;
typedef struct _zend_op zend_op;
struct _zend_compiler_context {
   zend_uint opcodes_size ;
   int vars_size ;
   int literals_size ;
   int current_brk_cont ;
   int backpatch_count ;
   HashTable *labels ;
};
typedef struct _zend_compiler_context zend_compiler_context;
struct _zend_literal {
   zval constant ;
   zend_ulong hash_value ;
   zend_uint cache_slot ;
};
typedef struct _zend_literal zend_literal;
union _znode_op {
   zend_uint constant ;
   zend_uint var ;
   zend_uint num ;
   zend_ulong hash ;
   zend_uint opline_num ;
   zend_op *jmp_addr ;
   zval *zv ;
   zend_literal *literal ;
   void *ptr ;
};
typedef union _znode_op znode_op;
union __anonunion_u_44 {
   znode_op op ;
   zval constant ;
   zend_op_array *op_array ;
};
struct _znode {
   int op_type ;
   union __anonunion_u_44 u ;
   zend_uint EA ;
};
typedef struct _znode znode;
struct _zend_execute_data;
typedef struct _zend_execute_data zend_execute_data;
typedef int (*user_opcode_handler_t)(zend_execute_data *execute_data );
typedef int ( __attribute__((__fastcall__)) (*opcode_handler_t))(zend_execute_data *execute_data );
struct _zend_op {
   int ( __attribute__((__fastcall__)) (*handler))(zend_execute_data *execute_data ) ;
   znode_op op1 ;
   znode_op op2 ;
   znode_op result ;
   ulong extended_value ;
   uint lineno ;
   zend_uchar opcode ;
   zend_uchar op1_type ;
   zend_uchar op2_type ;
   zend_uchar result_type ;
};
struct _zend_brk_cont_element {
   int start ;
   int cont ;
   int brk ;
   int parent ;
};
typedef struct _zend_brk_cont_element zend_brk_cont_element;
struct _zend_label {
   int brk_cont ;
   zend_uint opline_num ;
};
typedef struct _zend_label zend_label;
struct _zend_try_catch_element {
   zend_uint try_op ;
   zend_uint catch_op ;
};
typedef struct _zend_try_catch_element zend_try_catch_element;
struct _zend_property_info {
   zend_uint flags ;
   char const   *name ;
   int name_length ;
   ulong h ;
   int offset ;
   char const   *doc_comment ;
   int doc_comment_len ;
   zend_class_entry *ce ;
};
typedef struct _zend_property_info zend_property_info;
struct _zend_arg_info {
   char const   *name ;
   zend_uint name_len ;
   char const   *class_name ;
   zend_uint class_name_len ;
   zend_uchar type_hint ;
   zend_bool allow_null ;
   zend_bool pass_by_reference ;
};
typedef struct _zend_arg_info zend_arg_info;
struct _zend_internal_function_info {
   char const   *_name ;
   zend_uint _name_len ;
   char const   *_class_name ;
   zend_uint required_num_args ;
   zend_uchar _type_hint ;
   zend_bool return_reference ;
   zend_bool pass_rest_by_reference ;
};
typedef struct _zend_internal_function_info zend_internal_function_info;
struct _zend_compiled_variable {
   char const   *name ;
   int name_len ;
   ulong hash_value ;
};
typedef struct _zend_compiled_variable zend_compiled_variable;
struct _zend_op_array {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   zend_uint *refcount ;
   zend_op *opcodes ;
   zend_uint last ;
   zend_compiled_variable *vars ;
   int last_var ;
   zend_uint T ;
   zend_brk_cont_element *brk_cont_array ;
   int last_brk_cont ;
   zend_try_catch_element *try_catch_array ;
   int last_try_catch ;
   HashTable *static_variables ;
   zend_uint this_var ;
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint early_binding ;
   zend_literal *literals ;
   int last_literal ;
   void **run_time_cache ;
   int last_cache_slot ;
   void *reserved[4] ;
};
struct _zend_internal_function {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_module_entry *module ;
};
typedef struct _zend_internal_function zend_internal_function;
struct __anonstruct_common_45 {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
};
union _zend_function {
   zend_uchar type ;
   struct __anonstruct_common_45 common ;
   zend_op_array op_array ;
   zend_internal_function internal_function ;
};
typedef union _zend_function zend_function;
struct _zend_function_state {
   zend_function *function ;
   void **arguments ;
};
typedef struct _zend_function_state zend_function_state;
struct _zend_switch_entry {
   znode cond ;
   int default_case ;
   int control_var ;
};
typedef struct _zend_switch_entry zend_switch_entry;
struct _list_llist_element {
   znode var ;
   zend_llist dimensions ;
   znode value ;
};
typedef struct _list_llist_element list_llist_element;
union _temp_variable;
struct _zend_execute_data {
   struct _zend_op *opline ;
   zend_function_state function_state ;
   zend_function *fbc ;
   zend_class_entry *called_scope ;
   zend_op_array *op_array ;
   zval *object ;
   union _temp_variable *Ts ;
   zval ***CVs ;
   HashTable *symbol_table ;
   struct _zend_execute_data *prev_execute_data ;
   zval *old_error_reporting ;
   zend_bool nested ;
   zval **original_return_value ;
   zend_class_entry *current_scope ;
   zend_class_entry *current_called_scope ;
   zval *current_this ;
   zval *current_object ;
};
typedef int __jmp_buf[6];
struct __jmp_buf_tag {
   __jmp_buf __jmpbuf ;
   int __mask_was_saved ;
   __sigset_t __saved_mask ;
};
typedef struct __jmp_buf_tag jmp_buf[1];
typedef struct __jmp_buf_tag sigjmp_buf[1];
struct _zend_compiler_globals;
typedef struct _zend_compiler_globals zend_compiler_globals;
struct _zend_executor_globals;
typedef struct _zend_executor_globals zend_executor_globals;
struct _zend_php_scanner_globals;
typedef struct _zend_php_scanner_globals zend_php_scanner_globals;
struct _zend_ini_scanner_globals;
typedef struct _zend_ini_scanner_globals zend_ini_scanner_globals;
struct _zend_stack {
   int top ;
   int max ;
   void **elements ;
};
typedef struct _zend_stack zend_stack;
struct _zend_ptr_stack {
   int top ;
   int max ;
   void **elements ;
   void **top_element ;
   zend_bool persistent ;
};
typedef struct _zend_ptr_stack zend_ptr_stack;
typedef void (*zend_objects_store_dtor_t)(void *object ,
                                          zend_object_handle handle );
typedef void (*zend_objects_free_object_storage_t)(void *object );
typedef void (*zend_objects_store_clone_t)(void *object , void **object_clone );
struct _store_object {
   void *object ;
   void (*dtor)(void *object , zend_object_handle handle ) ;
   void (*free_storage)(void *object ) ;
   void (*clone)(void *object , void **object_clone ) ;
   zend_object_handlers const   *handlers ;
   zend_uint refcount ;
   gc_root_buffer *buffered ;
};
struct __anonstruct_free_list_46 {
   int next ;
};
union _store_bucket {
   struct _store_object obj ;
   struct __anonstruct_free_list_46 free_list ;
};
struct _zend_object_store_bucket {
   zend_bool destructor_called ;
   zend_bool valid ;
   union _store_bucket bucket ;
};
typedef struct _zend_object_store_bucket zend_object_store_bucket;
struct _zend_objects_store {
   zend_object_store_bucket *object_buckets ;
   zend_uint top ;
   zend_uint size ;
   int free_list_head ;
};
typedef struct _zend_objects_store zend_objects_store;
struct _zend_ini_entry;
typedef struct _zend_module_entry zend_module_entry;
struct _zend_module_dep;
typedef struct _zend_module_dep zend_module_dep;
struct _zend_module_entry {
   unsigned short size ;
   unsigned int zend_api ;
   unsigned char zend_debug ;
   unsigned char zts ;
   struct _zend_ini_entry  const  *ini_entry ;
   struct _zend_module_dep  const  *deps ;
   char const   *name ;
   struct _zend_function_entry  const  *functions ;
   int (*module_startup_func)(int type , int module_number ) ;
   int (*module_shutdown_func)(int type , int module_number ) ;
   int (*request_startup_func)(int type , int module_number ) ;
   int (*request_shutdown_func)(int type , int module_number ) ;
   void (*info_func)(zend_module_entry *zend_module ) ;
   char const   *version ;
   size_t globals_size ;
   void *globals_ptr ;
   void (*globals_ctor)(void *global ) ;
   void (*globals_dtor)(void *global ) ;
   int (*post_deactivate_func)(void) ;
   int module_started ;
   unsigned char type ;
   void *handle ;
   int module_number ;
   char const   *build_id ;
};
struct _zend_module_dep {
   char const   *name ;
   char const   *rel ;
   char const   *version ;
   unsigned char type ;
};
typedef unsigned short fpu_control_t;
struct _zend_encoding;
typedef struct _zend_encoding zend_encoding;
typedef size_t (*zend_encoding_filter)(unsigned char **str ,
                                       size_t *str_length ,
                                       unsigned char const   *buf ,
                                       size_t length );
typedef zend_encoding const   *(*zend_encoding_fetcher)(char const   *encoding_name );
typedef char const   *(*zend_encoding_name_getter)(zend_encoding const   *encoding );
typedef int (*zend_encoding_lexer_compatibility_checker)(zend_encoding const   *encoding );
typedef zend_encoding const   *(*zend_encoding_detector)(unsigned char const   *string ,
                                                         size_t length ,
                                                         zend_encoding const   **list ,
                                                         size_t list_size );
typedef size_t (*zend_encoding_converter)(unsigned char **to ,
                                          size_t *to_length ,
                                          unsigned char const   *from ,
                                          size_t from_length ,
                                          zend_encoding const   *encoding_to ,
                                          zend_encoding const   *encoding_from );
typedef int (*zend_encoding_list_parser)(char const   *encoding_list ,
                                         size_t encoding_list_len ,
                                         zend_encoding const   ***return_list ,
                                         size_t *return_size , int persistent );
typedef zend_encoding const   *(*zend_encoding_internal_encoding_getter)(void);
typedef int (*zend_encoding_internal_encoding_setter)(zend_encoding const   *encoding );
struct _zend_multibyte_functions {
   char const   *provider_name ;
   zend_encoding const   *(*encoding_fetcher)(char const   *encoding_name ) ;
   char const   *(*encoding_name_getter)(zend_encoding const   *encoding ) ;
   int (*lexer_compatibility_checker)(zend_encoding const   *encoding ) ;
   zend_encoding const   *(*encoding_detector)(unsigned char const   *string ,
                                               size_t length ,
                                               zend_encoding const   **list ,
                                               size_t list_size ) ;
   size_t (*encoding_converter)(unsigned char **to , size_t *to_length ,
                                unsigned char const   *from ,
                                size_t from_length ,
                                zend_encoding const   *encoding_to ,
                                zend_encoding const   *encoding_from ) ;
   int (*encoding_list_parser)(char const   *encoding_list ,
                               size_t encoding_list_len ,
                               zend_encoding const   ***return_list ,
                               size_t *return_size , int persistent ) ;
   zend_encoding const   *(*internal_encoding_getter)(void) ;
   int (*internal_encoding_setter)(zend_encoding const   *encoding ) ;
};
typedef struct _zend_multibyte_functions zend_multibyte_functions;
struct _zend_declarables {
   zval ticks ;
};
typedef struct _zend_declarables zend_declarables;
struct _zend_vm_stack;
typedef struct _zend_vm_stack *zend_vm_stack;
typedef struct _zend_ini_entry zend_ini_entry;
struct _zend_ini_parser_param;
struct _zend_compiler_globals {
   zend_stack bp_stack ;
   zend_stack switch_cond_stack ;
   zend_stack foreach_copy_stack ;
   zend_stack object_stack ;
   zend_stack declare_stack ;
   zend_class_entry *active_class_entry ;
   zend_llist list_llist ;
   zend_llist dimension_llist ;
   zend_stack list_stack ;
   zend_stack function_call_stack ;
   char *compiled_filename ;
   int zend_lineno ;
   char *heredoc ;
   int heredoc_len ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable filenames_table ;
   HashTable *auto_globals ;
   zend_bool parse_error ;
   zend_bool in_compilation ;
   zend_bool short_tags ;
   zend_bool asp_tags ;
   zend_declarables declarables ;
   zend_bool unclean_shutdown ;
   zend_bool ini_parser_unbuffered_errors ;
   zend_llist open_files ;
   long catch_begin ;
   struct _zend_ini_parser_param *ini_parser_param ;
   int interactive ;
   zend_uint start_lineno ;
   zend_bool increment_lineno ;
   znode implementing_class ;
   zend_uint access_type ;
   char *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint compiler_options ;
   zval *current_namespace ;
   HashTable *current_import ;
   zend_bool in_namespace ;
   zend_bool has_bracketed_namespaces ;
   zend_compiler_context context ;
   zend_stack context_stack ;
   char *interned_strings_start ;
   char *interned_strings_end ;
   char *interned_strings_top ;
   char *interned_strings_snapshot_top ;
   HashTable interned_strings ;
   zend_encoding const   **script_encoding_list ;
   size_t script_encoding_list_size ;
   zend_bool multibyte ;
   zend_bool detect_unicode ;
   zend_bool encoding_declared ;
};
struct _zend_executor_globals {
   zval **return_value_ptr_ptr ;
   zval uninitialized_zval ;
   zval *uninitialized_zval_ptr ;
   zval error_zval ;
   zval *error_zval_ptr ;
   zend_ptr_stack arg_types_stack ;
   HashTable *symtable_cache[32] ;
   HashTable **symtable_cache_limit ;
   HashTable **symtable_cache_ptr ;
   zend_op **opline_ptr ;
   HashTable *active_symbol_table ;
   HashTable symbol_table ;
   HashTable included_files ;
   jmp_buf *bailout ;
   int error_reporting ;
   int orig_error_reporting ;
   int exit_status ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable *zend_constants ;
   zend_class_entry *scope ;
   zend_class_entry *called_scope ;
   zval *This ;
   long precision ;
   int ticks_count ;
   zend_bool in_execution ;
   HashTable *in_autoload ;
   zend_function *autoload_func ;
   zend_bool full_tables_cleanup ;
   zend_bool no_extensions ;
   HashTable regular_list ;
   HashTable persistent_list ;
   zend_vm_stack argument_stack ;
   int user_error_handler_error_reporting ;
   zval *user_error_handler ;
   zval *user_exception_handler ;
   zend_stack user_error_handlers_error_reporting ;
   zend_ptr_stack user_error_handlers ;
   zend_ptr_stack user_exception_handlers ;
   zend_error_handling_t error_handling ;
   zend_class_entry *exception_class ;
   int timeout_seconds ;
   int lambda_count ;
   HashTable *ini_directives ;
   HashTable *modified_ini_directives ;
   zend_ini_entry *error_reporting_ini_entry ;
   zend_objects_store objects_store ;
   zval *exception ;
   zval *prev_exception ;
   zend_op *opline_before_exception ;
   zend_op exception_op[3] ;
   struct _zend_execute_data *current_execute_data ;
   struct _zend_module_entry *current_module ;
   zend_property_info std_property_info ;
   zend_bool active ;
   zend_op *start_op ;
   void *saved_fpu_cw_ptr ;
   fpu_control_t saved_fpu_cw ;
   void *reserved[4] ;
};
struct _zend_ini_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   char *filename ;
   int lineno ;
   int scanner_mode ;
};
struct _zend_php_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   unsigned char *script_org ;
   size_t script_org_size ;
   unsigned char *script_filtered ;
   size_t script_filtered_size ;
   size_t (*input_filter)(unsigned char **str , size_t *str_length ,
                          unsigned char const   *buf , size_t length ) ;
   size_t (*output_filter)(unsigned char **str , size_t *str_length ,
                           unsigned char const   *buf , size_t length ) ;
   zend_encoding const   *script_encoding ;
};
typedef int (*unary_op_type)(zval * , zval * );
typedef int (*binary_op_type)(zval * , zval * , zval * );
typedef zend_bool (*zend_auto_global_callback)(char const   *name ,
                                               uint name_len );
struct _zend_auto_global {
   char const   *name ;
   uint name_len ;
   zend_bool (*auto_global_callback)(char const   *name , uint name_len ) ;
   zend_bool jit ;
   zend_bool armed ;
};
typedef struct _zend_auto_global zend_auto_global;
struct _zend_lex_state {
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   zend_file_handle *in ;
   uint lineno ;
   char *filename ;
   unsigned char *script_org ;
   size_t script_org_size ;
   unsigned char *script_filtered ;
   size_t script_filtered_size ;
   size_t (*input_filter)(unsigned char **str , size_t *str_length ,
                          unsigned char const   *buf , size_t length ) ;
   size_t (*output_filter)(unsigned char **str , size_t *str_length ,
                           unsigned char const   *buf , size_t length ) ;
   zend_encoding const   *script_encoding ;
};
typedef struct _zend_lex_state zend_lex_state;
struct _zend_syntax_highlighter_ini {
   char *highlight_html ;
   char *highlight_comment ;
   char *highlight_default ;
   char *highlight_string ;
   char *highlight_keyword ;
};
typedef struct _zend_syntax_highlighter_ini zend_syntax_highlighter_ini;
struct _zend_constant {
   zval value ;
   int flags ;
   char *name ;
   uint name_len ;
   int module_number ;
};
typedef struct _zend_constant zend_constant;
struct _zend_rsrc_list_entry {
   void *ptr ;
   int type ;
   int refcount ;
};
typedef struct _zend_rsrc_list_entry zend_rsrc_list_entry;
typedef void (*rsrc_dtor_func_t)(zend_rsrc_list_entry *rsrc );
struct _zend_rsrc_list_dtors_entry {
   void (*list_dtor)(void * ) ;
   void (*plist_dtor)(void * ) ;
   void (*list_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   void (*plist_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   char const   *type_name ;
   int module_number ;
   int resource_id ;
   unsigned char type ;
};
typedef struct _zend_rsrc_list_dtors_entry zend_rsrc_list_dtors_entry;
struct __anonstruct_var_47 {
   zval **ptr_ptr ;
   zval *ptr ;
   zend_bool fcall_returned_reference ;
};
struct __anonstruct_str_offset_48 {
   zval **ptr_ptr ;
   zval *str ;
   zend_uint offset ;
};
struct __anonstruct_fe_49 {
   zval **ptr_ptr ;
   zval *ptr ;
   HashPointer fe_pos ;
};
union _temp_variable {
   zval tmp_var ;
   struct __anonstruct_var_47 var ;
   struct __anonstruct_str_offset_48 str_offset ;
   struct __anonstruct_fe_49 fe ;
   zend_class_entry *class_entry ;
};
typedef union _temp_variable temp_variable;
struct _zend_vm_stack {
   void **top ;
   void **end ;
   zend_vm_stack prev ;
};
struct _zend_free_op {
   zval *var ;
};
typedef struct _zend_free_op zend_free_op;
struct _zend_function_entry {
   char const   *fname ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_arg_info  const  *arg_info ;
   zend_uint num_args ;
   zend_uint flags ;
};
typedef struct _zend_function_entry zend_function_entry;
struct _zend_fcall_info {
   size_t size ;
   HashTable *function_table ;
   zval *function_name ;
   HashTable *symbol_table ;
   zval **retval_ptr_ptr ;
   zend_uint param_count ;
   zval ***params ;
   zval *object_ptr ;
   zend_bool no_separation ;
};
typedef struct _zend_fcall_info zend_fcall_info;
struct _zend_fcall_info_cache {
   zend_bool initialized ;
   zend_function *function_handler ;
   zend_class_entry *calling_scope ;
   zend_class_entry *called_scope ;
   zval *object_ptr ;
};
typedef struct _zend_fcall_info_cache zend_fcall_info_cache;
typedef __useconds_t useconds_t;
typedef __intptr_t intptr_t;
typedef __socklen_t socklen_t;
enum __anonenum_50 {
    _PC_LINK_MAX = 0,
    _PC_MAX_CANON = 1,
    _PC_MAX_INPUT = 2,
    _PC_NAME_MAX = 3,
    _PC_PATH_MAX = 4,
    _PC_PIPE_BUF = 5,
    _PC_CHOWN_RESTRICTED = 6,
    _PC_NO_TRUNC = 7,
    _PC_VDISABLE = 8,
    _PC_SYNC_IO = 9,
    _PC_ASYNC_IO = 10,
    _PC_PRIO_IO = 11,
    _PC_SOCK_MAXBUF = 12,
    _PC_FILESIZEBITS = 13,
    _PC_REC_INCR_XFER_SIZE = 14,
    _PC_REC_MAX_XFER_SIZE = 15,
    _PC_REC_MIN_XFER_SIZE = 16,
    _PC_REC_XFER_ALIGN = 17,
    _PC_ALLOC_SIZE_MIN = 18,
    _PC_SYMLINK_MAX = 19,
    _PC_2_SYMLINKS = 20
} ;
enum __anonenum_51 {
    _SC_ARG_MAX = 0,
    _SC_CHILD_MAX = 1,
    _SC_CLK_TCK = 2,
    _SC_NGROUPS_MAX = 3,
    _SC_OPEN_MAX = 4,
    _SC_STREAM_MAX = 5,
    _SC_TZNAME_MAX = 6,
    _SC_JOB_CONTROL = 7,
    _SC_SAVED_IDS = 8,
    _SC_REALTIME_SIGNALS = 9,
    _SC_PRIORITY_SCHEDULING = 10,
    _SC_TIMERS = 11,
    _SC_ASYNCHRONOUS_IO = 12,
    _SC_PRIORITIZED_IO = 13,
    _SC_SYNCHRONIZED_IO = 14,
    _SC_FSYNC = 15,
    _SC_MAPPED_FILES = 16,
    _SC_MEMLOCK = 17,
    _SC_MEMLOCK_RANGE = 18,
    _SC_MEMORY_PROTECTION = 19,
    _SC_MESSAGE_PASSING = 20,
    _SC_SEMAPHORES = 21,
    _SC_SHARED_MEMORY_OBJECTS = 22,
    _SC_AIO_LISTIO_MAX = 23,
    _SC_AIO_MAX = 24,
    _SC_AIO_PRIO_DELTA_MAX = 25,
    _SC_DELAYTIMER_MAX = 26,
    _SC_MQ_OPEN_MAX = 27,
    _SC_MQ_PRIO_MAX = 28,
    _SC_VERSION = 29,
    _SC_PAGESIZE = 30,
    _SC_RTSIG_MAX = 31,
    _SC_SEM_NSEMS_MAX = 32,
    _SC_SEM_VALUE_MAX = 33,
    _SC_SIGQUEUE_MAX = 34,
    _SC_TIMER_MAX = 35,
    _SC_BC_BASE_MAX = 36,
    _SC_BC_DIM_MAX = 37,
    _SC_BC_SCALE_MAX = 38,
    _SC_BC_STRING_MAX = 39,
    _SC_COLL_WEIGHTS_MAX = 40,
    _SC_EQUIV_CLASS_MAX = 41,
    _SC_EXPR_NEST_MAX = 42,
    _SC_LINE_MAX = 43,
    _SC_RE_DUP_MAX = 44,
    _SC_CHARCLASS_NAME_MAX = 45,
    _SC_2_VERSION = 46,
    _SC_2_C_BIND = 47,
    _SC_2_C_DEV = 48,
    _SC_2_FORT_DEV = 49,
    _SC_2_FORT_RUN = 50,
    _SC_2_SW_DEV = 51,
    _SC_2_LOCALEDEF = 52,
    _SC_PII = 53,
    _SC_PII_XTI = 54,
    _SC_PII_SOCKET = 55,
    _SC_PII_INTERNET = 56,
    _SC_PII_OSI = 57,
    _SC_POLL = 58,
    _SC_SELECT = 59,
    _SC_UIO_MAXIOV = 60,
    _SC_IOV_MAX = 60,
    _SC_PII_INTERNET_STREAM = 61,
    _SC_PII_INTERNET_DGRAM = 62,
    _SC_PII_OSI_COTS = 63,
    _SC_PII_OSI_CLTS = 64,
    _SC_PII_OSI_M = 65,
    _SC_T_IOV_MAX = 66,
    _SC_THREADS = 67,
    _SC_THREAD_SAFE_FUNCTIONS = 68,
    _SC_GETGR_R_SIZE_MAX = 69,
    _SC_GETPW_R_SIZE_MAX = 70,
    _SC_LOGIN_NAME_MAX = 71,
    _SC_TTY_NAME_MAX = 72,
    _SC_THREAD_DESTRUCTOR_ITERATIONS = 73,
    _SC_THREAD_KEYS_MAX = 74,
    _SC_THREAD_STACK_MIN = 75,
    _SC_THREAD_THREADS_MAX = 76,
    _SC_THREAD_ATTR_STACKADDR = 77,
    _SC_THREAD_ATTR_STACKSIZE = 78,
    _SC_THREAD_PRIORITY_SCHEDULING = 79,
    _SC_THREAD_PRIO_INHERIT = 80,
    _SC_THREAD_PRIO_PROTECT = 81,
    _SC_THREAD_PROCESS_SHARED = 82,
    _SC_NPROCESSORS_CONF = 83,
    _SC_NPROCESSORS_ONLN = 84,
    _SC_PHYS_PAGES = 85,
    _SC_AVPHYS_PAGES = 86,
    _SC_ATEXIT_MAX = 87,
    _SC_PASS_MAX = 88,
    _SC_XOPEN_VERSION = 89,
    _SC_XOPEN_XCU_VERSION = 90,
    _SC_XOPEN_UNIX = 91,
    _SC_XOPEN_CRYPT = 92,
    _SC_XOPEN_ENH_I18N = 93,
    _SC_XOPEN_SHM = 94,
    _SC_2_CHAR_TERM = 95,
    _SC_2_C_VERSION = 96,
    _SC_2_UPE = 97,
    _SC_XOPEN_XPG2 = 98,
    _SC_XOPEN_XPG3 = 99,
    _SC_XOPEN_XPG4 = 100,
    _SC_CHAR_BIT = 101,
    _SC_CHAR_MAX = 102,
    _SC_CHAR_MIN = 103,
    _SC_INT_MAX = 104,
    _SC_INT_MIN = 105,
    _SC_LONG_BIT = 106,
    _SC_WORD_BIT = 107,
    _SC_MB_LEN_MAX = 108,
    _SC_NZERO = 109,
    _SC_SSIZE_MAX = 110,
    _SC_SCHAR_MAX = 111,
    _SC_SCHAR_MIN = 112,
    _SC_SHRT_MAX = 113,
    _SC_SHRT_MIN = 114,
    _SC_UCHAR_MAX = 115,
    _SC_UINT_MAX = 116,
    _SC_ULONG_MAX = 117,
    _SC_USHRT_MAX = 118,
    _SC_NL_ARGMAX = 119,
    _SC_NL_LANGMAX = 120,
    _SC_NL_MSGMAX = 121,
    _SC_NL_NMAX = 122,
    _SC_NL_SETMAX = 123,
    _SC_NL_TEXTMAX = 124,
    _SC_XBS5_ILP32_OFF32 = 125,
    _SC_XBS5_ILP32_OFFBIG = 126,
    _SC_XBS5_LP64_OFF64 = 127,
    _SC_XBS5_LPBIG_OFFBIG = 128,
    _SC_XOPEN_LEGACY = 129,
    _SC_XOPEN_REALTIME = 130,
    _SC_XOPEN_REALTIME_THREADS = 131,
    _SC_ADVISORY_INFO = 132,
    _SC_BARRIERS = 133,
    _SC_BASE = 134,
    _SC_C_LANG_SUPPORT = 135,
    _SC_C_LANG_SUPPORT_R = 136,
    _SC_CLOCK_SELECTION = 137,
    _SC_CPUTIME = 138,
    _SC_THREAD_CPUTIME = 139,
    _SC_DEVICE_IO = 140,
    _SC_DEVICE_SPECIFIC = 141,
    _SC_DEVICE_SPECIFIC_R = 142,
    _SC_FD_MGMT = 143,
    _SC_FIFO = 144,
    _SC_PIPE = 145,
    _SC_FILE_ATTRIBUTES = 146,
    _SC_FILE_LOCKING = 147,
    _SC_FILE_SYSTEM = 148,
    _SC_MONOTONIC_CLOCK = 149,
    _SC_MULTI_PROCESS = 150,
    _SC_SINGLE_PROCESS = 151,
    _SC_NETWORKING = 152,
    _SC_READER_WRITER_LOCKS = 153,
    _SC_SPIN_LOCKS = 154,
    _SC_REGEXP = 155,
    _SC_REGEX_VERSION = 156,
    _SC_SHELL = 157,
    _SC_SIGNALS = 158,
    _SC_SPAWN = 159,
    _SC_SPORADIC_SERVER = 160,
    _SC_THREAD_SPORADIC_SERVER = 161,
    _SC_SYSTEM_DATABASE = 162,
    _SC_SYSTEM_DATABASE_R = 163,
    _SC_TIMEOUTS = 164,
    _SC_TYPED_MEMORY_OBJECTS = 165,
    _SC_USER_GROUPS = 166,
    _SC_USER_GROUPS_R = 167,
    _SC_2_PBS = 168,
    _SC_2_PBS_ACCOUNTING = 169,
    _SC_2_PBS_LOCATE = 170,
    _SC_2_PBS_MESSAGE = 171,
    _SC_2_PBS_TRACK = 172,
    _SC_SYMLOOP_MAX = 173,
    _SC_STREAMS = 174,
    _SC_2_PBS_CHECKPOINT = 175,
    _SC_V6_ILP32_OFF32 = 176,
    _SC_V6_ILP32_OFFBIG = 177,
    _SC_V6_LP64_OFF64 = 178,
    _SC_V6_LPBIG_OFFBIG = 179,
    _SC_HOST_NAME_MAX = 180,
    _SC_TRACE = 181,
    _SC_TRACE_EVENT_FILTER = 182,
    _SC_TRACE_INHERIT = 183,
    _SC_TRACE_LOG = 184,
    _SC_LEVEL1_ICACHE_SIZE = 185,
    _SC_LEVEL1_ICACHE_ASSOC = 186,
    _SC_LEVEL1_ICACHE_LINESIZE = 187,
    _SC_LEVEL1_DCACHE_SIZE = 188,
    _SC_LEVEL1_DCACHE_ASSOC = 189,
    _SC_LEVEL1_DCACHE_LINESIZE = 190,
    _SC_LEVEL2_CACHE_SIZE = 191,
    _SC_LEVEL2_CACHE_ASSOC = 192,
    _SC_LEVEL2_CACHE_LINESIZE = 193,
    _SC_LEVEL3_CACHE_SIZE = 194,
    _SC_LEVEL3_CACHE_ASSOC = 195,
    _SC_LEVEL3_CACHE_LINESIZE = 196,
    _SC_LEVEL4_CACHE_SIZE = 197,
    _SC_LEVEL4_CACHE_ASSOC = 198,
    _SC_LEVEL4_CACHE_LINESIZE = 199,
    _SC_IPV6 = 235,
    _SC_RAW_SOCKETS = 236,
    _SC_V7_ILP32_OFF32 = 237,
    _SC_V7_ILP32_OFFBIG = 238,
    _SC_V7_LP64_OFF64 = 239,
    _SC_V7_LPBIG_OFFBIG = 240,
    _SC_SS_REPL_MAX = 241,
    _SC_TRACE_EVENT_NAME_MAX = 242,
    _SC_TRACE_NAME_MAX = 243,
    _SC_TRACE_SYS_MAX = 244,
    _SC_TRACE_USER_EVENT_MAX = 245,
    _SC_XOPEN_STREAMS = 246,
    _SC_THREAD_ROBUST_PRIO_INHERIT = 247,
    _SC_THREAD_ROBUST_PRIO_PROTECT = 248
} ;
enum __anonenum_52 {
    _CS_PATH = 0,
    _CS_V6_WIDTH_RESTRICTED_ENVS = 1,
    _CS_GNU_LIBC_VERSION = 2,
    _CS_GNU_LIBPTHREAD_VERSION = 3,
    _CS_V5_WIDTH_RESTRICTED_ENVS = 4,
    _CS_V7_WIDTH_RESTRICTED_ENVS = 5,
    _CS_LFS_CFLAGS = 1000,
    _CS_LFS_LDFLAGS = 1001,
    _CS_LFS_LIBS = 1002,
    _CS_LFS_LINTFLAGS = 1003,
    _CS_LFS64_CFLAGS = 1004,
    _CS_LFS64_LDFLAGS = 1005,
    _CS_LFS64_LIBS = 1006,
    _CS_LFS64_LINTFLAGS = 1007,
    _CS_XBS5_ILP32_OFF32_CFLAGS = 1100,
    _CS_XBS5_ILP32_OFF32_LDFLAGS = 1101,
    _CS_XBS5_ILP32_OFF32_LIBS = 1102,
    _CS_XBS5_ILP32_OFF32_LINTFLAGS = 1103,
    _CS_XBS5_ILP32_OFFBIG_CFLAGS = 1104,
    _CS_XBS5_ILP32_OFFBIG_LDFLAGS = 1105,
    _CS_XBS5_ILP32_OFFBIG_LIBS = 1106,
    _CS_XBS5_ILP32_OFFBIG_LINTFLAGS = 1107,
    _CS_XBS5_LP64_OFF64_CFLAGS = 1108,
    _CS_XBS5_LP64_OFF64_LDFLAGS = 1109,
    _CS_XBS5_LP64_OFF64_LIBS = 1110,
    _CS_XBS5_LP64_OFF64_LINTFLAGS = 1111,
    _CS_XBS5_LPBIG_OFFBIG_CFLAGS = 1112,
    _CS_XBS5_LPBIG_OFFBIG_LDFLAGS = 1113,
    _CS_XBS5_LPBIG_OFFBIG_LIBS = 1114,
    _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS = 1115,
    _CS_POSIX_V6_ILP32_OFF32_CFLAGS = 1116,
    _CS_POSIX_V6_ILP32_OFF32_LDFLAGS = 1117,
    _CS_POSIX_V6_ILP32_OFF32_LIBS = 1118,
    _CS_POSIX_V6_ILP32_OFF32_LINTFLAGS = 1119,
    _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS = 1120,
    _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS = 1121,
    _CS_POSIX_V6_ILP32_OFFBIG_LIBS = 1122,
    _CS_POSIX_V6_ILP32_OFFBIG_LINTFLAGS = 1123,
    _CS_POSIX_V6_LP64_OFF64_CFLAGS = 1124,
    _CS_POSIX_V6_LP64_OFF64_LDFLAGS = 1125,
    _CS_POSIX_V6_LP64_OFF64_LIBS = 1126,
    _CS_POSIX_V6_LP64_OFF64_LINTFLAGS = 1127,
    _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS = 1128,
    _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS = 1129,
    _CS_POSIX_V6_LPBIG_OFFBIG_LIBS = 1130,
    _CS_POSIX_V6_LPBIG_OFFBIG_LINTFLAGS = 1131,
    _CS_POSIX_V7_ILP32_OFF32_CFLAGS = 1132,
    _CS_POSIX_V7_ILP32_OFF32_LDFLAGS = 1133,
    _CS_POSIX_V7_ILP32_OFF32_LIBS = 1134,
    _CS_POSIX_V7_ILP32_OFF32_LINTFLAGS = 1135,
    _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS = 1136,
    _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS = 1137,
    _CS_POSIX_V7_ILP32_OFFBIG_LIBS = 1138,
    _CS_POSIX_V7_ILP32_OFFBIG_LINTFLAGS = 1139,
    _CS_POSIX_V7_LP64_OFF64_CFLAGS = 1140,
    _CS_POSIX_V7_LP64_OFF64_LDFLAGS = 1141,
    _CS_POSIX_V7_LP64_OFF64_LIBS = 1142,
    _CS_POSIX_V7_LP64_OFF64_LINTFLAGS = 1143,
    _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS = 1144,
    _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS = 1145,
    _CS_POSIX_V7_LPBIG_OFFBIG_LIBS = 1146,
    _CS_POSIX_V7_LPBIG_OFFBIG_LINTFLAGS = 1147,
    _CS_V6_ENV = 1148,
    _CS_V7_ENV = 1149
} ;
struct stat {
   __dev_t st_dev ;
   unsigned short __pad1 ;
   __ino_t st_ino ;
   __mode_t st_mode ;
   __nlink_t st_nlink ;
   __uid_t st_uid ;
   __gid_t st_gid ;
   __dev_t st_rdev ;
   unsigned short __pad2 ;
   __off_t st_size ;
   __blksize_t st_blksize ;
   __blkcnt_t st_blocks ;
   struct timespec st_atim ;
   struct timespec st_mtim ;
   struct timespec st_ctim ;
   unsigned long __unused4 ;
   unsigned long __unused5 ;
};
enum __anonenum_53 {
    _ISupper = 256,
    _ISlower = 512,
    _ISalpha = 1024,
    _ISdigit = 2048,
    _ISxdigit = 4096,
    _ISspace = 8192,
    _ISprint = 16384,
    _ISgraph = 32768,
    _ISblank = 1,
    _IScntrl = 2,
    _ISpunct = 4,
    _ISalnum = 8
} ;
struct utimbuf {
   __time_t actime ;
   __time_t modtime ;
};
struct dirent {
   __ino_t d_ino ;
   __off_t d_off ;
   unsigned short d_reclen ;
   unsigned char d_type ;
   char d_name[256] ;
};
enum __anonenum_54 {
    DT_UNKNOWN = 0,
    DT_FIFO = 1,
    DT_CHR = 2,
    DT_DIR = 4,
    DT_BLK = 6,
    DT_REG = 8,
    DT_LNK = 10,
    DT_SOCK = 12,
    DT_WHT = 14
} ;
struct __dirstream;
typedef struct __dirstream DIR;
struct _cwd_state {
   char *cwd ;
   int cwd_length ;
};
typedef struct _cwd_state cwd_state;
typedef int (*verify_path_func)(cwd_state const   * );
struct _realpath_cache_bucket {
   unsigned long key ;
   char *path ;
   int path_len ;
   char *realpath ;
   int realpath_len ;
   int is_dir ;
   time_t expires ;
   struct _realpath_cache_bucket *next ;
};
typedef struct _realpath_cache_bucket realpath_cache_bucket;
struct _virtual_cwd_globals {
   cwd_state cwd ;
   long realpath_cache_size ;
   long realpath_cache_size_limit ;
   long realpath_cache_ttl ;
   realpath_cache_bucket *realpath_cache[1024] ;
};
typedef struct _virtual_cwd_globals virtual_cwd_globals;
extern  __attribute__((__nothrow__)) int *__errno_location(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) size_t __ctype_get_mb_cur_max(void) ;
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) double strtod(char const   * __restrict  __nptr ,
                                                   char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) float strtof(char const   * __restrict  __nptr ,
                                                  char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long double strtold(char const   * __restrict  __nptr ,
                                                         char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long strtol(char const   * __restrict  __nptr ,
                                                 char ** __restrict  __endptr ,
                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long strtoul(char const   * __restrict  __nptr ,
                                                           char ** __restrict  __endptr ,
                                                           int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long long strtoq(char const   * __restrict  __nptr ,
                                                      char ** __restrict  __endptr ,
                                                      int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtouq(char const   * __restrict  __nptr ,
                                                                char ** __restrict  __endptr ,
                                                                int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long long strtoll(char const   * __restrict  __nptr ,
                                                       char ** __restrict  __endptr ,
                                                       int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtoull(char const   * __restrict  __nptr ,
                                                                 char ** __restrict  __endptr ,
                                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern FILE *fopen(char const   * __restrict  __filename ,
                   char const   * __restrict  __modes ) ;
extern int fprintf(FILE * __restrict  __stream ,
                   char const   * __restrict  __format  , ...) ;
extern int fflush(FILE *__stream ) ;
extern int fclose(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern double atof(char const   *__nptr ) 
{ double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7387\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtod((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7388\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern int atoi(char const   *__nptr ) 
{ long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7389\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long atol(char const   *__nptr ) 
{ long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long long atoll(char const   *__nptr ) 
{ long long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtoll((char const   */* __restrict  */)__nptr,
                (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) char *l64a(long __n ) ;
extern  __attribute__((__nothrow__)) long a64l(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
extern int select(int __nfds , fd_set * __restrict  __readfds ,
                  fd_set * __restrict  __writefds ,
                  fd_set * __restrict  __exceptfds ,
                  struct timeval * __restrict  __timeout ) ;
extern int pselect(int __nfds , fd_set * __restrict  __readfds ,
                   fd_set * __restrict  __writefds ,
                   fd_set * __restrict  __exceptfds ,
                   struct timespec  const  * __restrict  __timeout ,
                   __sigset_t const   * __restrict  __sigmask ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_major(unsigned long long __dev ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )(((__dev >> 8) & 4095ULL) | (unsigned long long )((unsigned int )(__dev >> 32) & 4294963200U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_minor(unsigned long long __dev ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )((__dev & 255ULL) | (unsigned long long )((unsigned int )(__dev >> 12) & 4294967040U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                   unsigned int __minor ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (((unsigned long long )((__minor & 255U) | ((__major & 4095U) << 8)) | ((unsigned long long )(__minor & 4294967040U) << 12)) | ((unsigned long long )(__major & 4294963200U) << 32));
}
}
extern  __attribute__((__nothrow__)) long random(void) ;
extern  __attribute__((__nothrow__)) void srandom(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) char *initstate(unsigned int __seed ,
                                                     char *__statebuf ,
                                                     size_t __statelen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *setstate(char *__statebuf )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int random_r(struct random_data * __restrict  __buf ,
                                                  int32_t * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int srandom_r(unsigned int __seed ,
                                                   struct random_data *__buf )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int initstate_r(unsigned int __seed ,
                                                     char * __restrict  __statebuf ,
                                                     size_t __statelen ,
                                                     struct random_data * __restrict  __buf )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int setstate_r(char * __restrict  __statebuf ,
                                                    struct random_data * __restrict  __buf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int rand(void) ;
extern  __attribute__((__nothrow__)) void srand(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) int rand_r(unsigned int *__seed ) ;
extern  __attribute__((__nothrow__)) double drand48(void) ;
extern  __attribute__((__nothrow__)) double erand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long lrand48(void) ;
extern  __attribute__((__nothrow__)) long nrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long mrand48(void) ;
extern  __attribute__((__nothrow__)) long jrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void srand48(long __seedval ) ;
extern  __attribute__((__nothrow__)) unsigned short *seed48(unsigned short *__seed16v )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void lcong48(unsigned short *__param )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int drand48_r(struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int erand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int lrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int nrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int mrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int jrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int srand48_r(long __seedval ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int seed48_r(unsigned short *__seed16v ,
                                                  struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int lcong48_r(unsigned short *__param ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *malloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *calloc(size_t __nmemb ,
                                                  size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *realloc(void *__ptr , size_t __size )  __attribute__((__warn_unused_result__)) ;
extern  __attribute__((__nothrow__)) void free(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void cfree(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void *alloca(size_t __size ) ;
extern  __attribute__((__nothrow__)) void *valloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) int posix_memalign(void **__memptr ,
                                                        size_t __alignment ,
                                                        size_t __size )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void abort(void) ;
extern  __attribute__((__nothrow__)) int atexit(void (*__func)(void) )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int on_exit(void (*__func)(int __status ,
                                                                void *__arg ) ,
                                                 void *__arg )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void exit(int __status ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void _Exit(int __status ) ;
extern  __attribute__((__nothrow__)) char *getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *__secure_getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int putenv(char *__string )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int setenv(char const   *__name ,
                                                char const   *__value ,
                                                int __replace )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int unsetenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int clearenv(void) ;
extern  __attribute__((__nothrow__)) char *mktemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemps(char *__template , int __suffixlen )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *mkdtemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern int system(char const   *__command ) ;
extern  __attribute__((__nothrow__)) char *realpath(char const   * __restrict  __name ,
                                                    char * __restrict  __resolved ) ;
extern void *bsearch(void const   *__key , void const   *__base ,
                     size_t __nmemb , size_t __size ,
                     int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,2,5))) ;
extern void qsort(void *__base , size_t __nmemb , size_t __size ,
                  int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) int abs(int __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long labs(long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long long llabs(long long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) div_t div(int __numer , int __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) ldiv_t ldiv(long __numer , long __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) lldiv_t lldiv(long long __numer ,
                                                   long long __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) char *ecvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *fcvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *gcvt(double __value , int __ndigit ,
                                                char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) char *qecvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *qfcvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *qgcvt(long double __value ,
                                                 int __ndigit , char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) int ecvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int fcvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int qecvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int qfcvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int mblen(char const   *__s , size_t __n ) ;
extern  __attribute__((__nothrow__)) int mbtowc(wchar_t * __restrict  __pwc ,
                                                char const   * __restrict  __s ,
                                                size_t __n ) ;
extern  __attribute__((__nothrow__)) int wctomb(char *__s , wchar_t __wchar ) ;
extern  __attribute__((__nothrow__)) size_t mbstowcs(wchar_t * __restrict  __pwcs ,
                                                     char const   * __restrict  __s ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) size_t wcstombs(char * __restrict  __s ,
                                                     wchar_t const   * __restrict  __pwcs ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int rpmatch(char const   *__response )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getsubopt(char ** __restrict  __optionp ,
                                                   char * const  * __restrict  __tokens ,
                                                   char ** __restrict  __valuep )  __attribute__((__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) int getloadavg(double *__loadavg ,
                                                    int __nelem )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *memcpy(void * __restrict  __dest ,
                                                  void const   * __restrict  __src ,
                                                  size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memmove(void *__dest ,
                                                   void const   *__src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memccpy(void * __restrict  __dest ,
                                                   void const   * __restrict  __src ,
                                                   int __c , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memset(void *__s , int __c ,
                                                  size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int memcmp(void const   *__s1 ,
                                                void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memchr(void const   *__s , int __c ,
                                                  size_t __n )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strcat(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncat(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strcmp(char const   *__s1 ,
                                                char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strncmp(char const   *__s1 ,
                                                 char const   *__s2 ,
                                                 size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strcoll(char const   *__s1 ,
                                                 char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strxfrm(char * __restrict  __dest ,
                                                    char const   * __restrict  __src ,
                                                    size_t __n )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int strcoll_l(char const   *__s1 ,
                                                   char const   *__s2 ,
                                                   __locale_t __l )  __attribute__((__pure__,
__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) size_t strxfrm_l(char *__dest ,
                                                      char const   *__src ,
                                                      size_t __n ,
                                                      __locale_t __l )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) char *strdup(char const   *__s )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) char *strndup(char const   *__string ,
                                                   size_t __n )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) char *strchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strrchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) size_t strcspn(char const   *__s ,
                                                    char const   *__reject )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strspn(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strpbrk(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strstr(char const   *__haystack ,
                                                  char const   *__needle )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strtok(char * __restrict  __s ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *__strtok_r(char * __restrict  __s ,
                                                      char const   * __restrict  __delim ,
                                                      char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) char *strtok_r(char * __restrict  __s ,
                                                    char const   * __restrict  __delim ,
                                                    char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) size_t strlen(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) size_t strnlen(char const   *__string ,
                                                    size_t __maxlen )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strerror(int __errnum ) ;
extern  __attribute__((__nothrow__)) int strerror_r(int __errnum , char *__buf ,
                                                    size_t __buflen )  __asm__("__xpg_strerror_r") __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *strerror_l(int __errnum ,
                                                      __locale_t __l ) ;
extern  __attribute__((__nothrow__)) void __bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void bcopy(void const   *__src ,
                                                void *__dest , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int bcmp(void const   *__s1 ,
                                              void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *index(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *rindex(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int ffs(int __i )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int strcasecmp(char const   *__s1 ,
                                                    char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strncasecmp(char const   *__s1 ,
                                                     char const   *__s2 ,
                                                     size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strsep(char ** __restrict  __stringp ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strsignal(int __sig ) ;
extern  __attribute__((__nothrow__)) char *__stpcpy(char * __restrict  __dest ,
                                                    char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *__stpncpy(char * __restrict  __dest ,
                                                     char const   * __restrict  __src ,
                                                     size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern void *__rawmemchr(void const   *__s , int __c ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7400\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7399\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7398\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7401\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) ;
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7410\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7411\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7408\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7407\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7406\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7405\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) ;
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7417\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7416\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7415\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7414\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*(__s + __result) != (int const   )__reject3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7413\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7418\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) ;
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7423\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7422\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7424\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7431\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7428\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7430\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7429\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7441\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7436\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7440\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7437\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7439\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) == (int const   )__accept3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7438\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7442\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7449\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7447\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7446\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7450\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7451\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7452\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7460\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7459\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7458\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7457\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*__s != (int const   )__accept3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7456\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7461\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7462\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7463\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) ;
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) 
{ char *__result ;
  char *tmp ;
  char *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__s == (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7467\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = *__nextp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7468\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7470\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*__s == (int )__sep) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7469\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7471\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*__s != 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7479\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __s;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7482\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7475\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__s != 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7472\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7476\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7477\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7478\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*tmp___0 == (int )__sep) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7473\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(__s + -1) = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7474\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7483\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *__nextp = __s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
extern char *__strsep_g(char **__stringp , char const   *__delim ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) 
{ register char *__retval ;
  char *tmp ;
  char *tmp___0 ;
  void *tmp___1 ;
  char *tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7494\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = __builtin_strchr(__retval, (int )__reject);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7495\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7496\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7497\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )tmp___0 != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7490\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = *__s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7491\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (*__s) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7492\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp = (char )'\000';
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7493\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7498\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) ;
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7519\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7520\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __cp = __retval;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7512\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7502\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7503\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7513\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7504\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __cp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7505\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7506\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7511\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7507\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = __cp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7508\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __cp ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7509\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7510\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7514\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __cp ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7521\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) ;
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7539\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __cp = __retval;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7540\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7536\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7522\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7523\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7537\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7524\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __cp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7525\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7526\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7535\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7527\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = __cp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7528\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __cp ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7529\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7534\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*__cp == (int )__reject3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7530\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = __cp;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7531\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __cp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7532\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *tmp = (char )'\000';
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7533\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7538\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __cp ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
extern  __attribute__((__nothrow__)) char *__strdup(char const   *__string )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *__strndup(char const   *__string ,
                                                     size_t __n )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) double acos(double __x ) ;
extern  __attribute__((__nothrow__)) double __acos(double __x ) ;
extern  __attribute__((__nothrow__)) double asin(double __x ) ;
extern  __attribute__((__nothrow__)) double __asin(double __x ) ;
extern  __attribute__((__nothrow__)) double atan(double __x ) ;
extern  __attribute__((__nothrow__)) double __atan(double __x ) ;
extern  __attribute__((__nothrow__)) double atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double __atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double cos(double __x ) ;
extern  __attribute__((__nothrow__)) double __cos(double __x ) ;
extern  __attribute__((__nothrow__)) double sin(double __x ) ;
extern  __attribute__((__nothrow__)) double __sin(double __x ) ;
extern  __attribute__((__nothrow__)) double tan(double __x ) ;
extern  __attribute__((__nothrow__)) double __tan(double __x ) ;
extern  __attribute__((__nothrow__)) double cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double exp(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp(double __x ) ;
extern  __attribute__((__nothrow__)) double frexp(double __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) double __frexp(double __x ,
                                                    int *__exponent ) ;
extern  __attribute__((__nothrow__)) double ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double __ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double log(double __x ) ;
extern  __attribute__((__nothrow__)) double __log(double __x ) ;
extern  __attribute__((__nothrow__)) double log10(double __x ) ;
extern  __attribute__((__nothrow__)) double __log10(double __x ) ;
extern  __attribute__((__nothrow__)) double modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double __modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double __expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double __log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double logb(double __x ) ;
extern  __attribute__((__nothrow__)) double __logb(double __x ) ;
extern  __attribute__((__nothrow__)) double exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double log2(double __x ) ;
extern  __attribute__((__nothrow__)) double __log2(double __x ) ;
extern  __attribute__((__nothrow__)) double pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __cbrt(double __x ) ;
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __ceil(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fabs(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int __isinf(double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finite(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double significand(double __x ) ;
extern  __attribute__((__nothrow__)) double __significand(double __x ) ;
extern  __attribute__((__nothrow__)) double copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double j0(double  ) ;
extern  __attribute__((__nothrow__)) double __j0(double  ) ;
extern  __attribute__((__nothrow__)) double j1(double  ) ;
extern  __attribute__((__nothrow__)) double __j1(double  ) ;
extern  __attribute__((__nothrow__)) double jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double y0(double  ) ;
extern  __attribute__((__nothrow__)) double __y0(double  ) ;
extern  __attribute__((__nothrow__)) double y1(double  ) ;
extern  __attribute__((__nothrow__)) double __y1(double  ) ;
extern  __attribute__((__nothrow__)) double yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double erf(double  ) ;
extern  __attribute__((__nothrow__)) double __erf(double  ) ;
extern  __attribute__((__nothrow__)) double erfc(double  ) ;
extern  __attribute__((__nothrow__)) double __erfc(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double gamma(double  ) ;
extern  __attribute__((__nothrow__)) double __gamma(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma_r(double  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double __lgamma_r(double  ,
                                                       int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double rint(double __x ) ;
extern  __attribute__((__nothrow__)) double __rint(double __x ) ;
extern  __attribute__((__nothrow__)) double nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nexttoward(double __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nexttoward(double __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double __scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) int ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) int __ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) double scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double __scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double __nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double remquo(double __x , double __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) double __remquo(double __x , double __y ,
                                                     int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
extern  __attribute__((__nothrow__)) long __lrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long lround(double __x ) ;
extern  __attribute__((__nothrow__)) long __lround(double __x ) ;
extern  __attribute__((__nothrow__)) long long llround(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llround(double __x ) ;
extern  __attribute__((__nothrow__)) double fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassify(double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double fma(double __x , double __y ,
                                                double __z ) ;
extern  __attribute__((__nothrow__)) double __fma(double __x , double __y ,
                                                  double __z ) ;
extern  __attribute__((__nothrow__)) double scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double __scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) float acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float __atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float expf(float __x ) ;
extern  __attribute__((__nothrow__)) float __expf(float __x ) ;
extern  __attribute__((__nothrow__)) float frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float __frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float __ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float logf(float __x ) ;
extern  __attribute__((__nothrow__)) float __logf(float __x ) ;
extern  __attribute__((__nothrow__)) float log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float __modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float __expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float __log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cbrtf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __ceilf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fabsf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int __isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float __significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float j0f(float  ) ;
extern  __attribute__((__nothrow__)) float __j0f(float  ) ;
extern  __attribute__((__nothrow__)) float j1f(float  ) ;
extern  __attribute__((__nothrow__)) float __j1f(float  ) ;
extern  __attribute__((__nothrow__)) float jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float y0f(float  ) ;
extern  __attribute__((__nothrow__)) float __y0f(float  ) ;
extern  __attribute__((__nothrow__)) float y1f(float  ) ;
extern  __attribute__((__nothrow__)) float __y1f(float  ) ;
extern  __attribute__((__nothrow__)) float ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float erff(float  ) ;
extern  __attribute__((__nothrow__)) float __erff(float  ) ;
extern  __attribute__((__nothrow__)) float erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float __erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float __lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nexttowardf(float __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nexttowardf(float __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float __scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) int ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) int __ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) float scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float __scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float remquof(float __x , float __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) float __remquof(float __x , float __y ,
                                                     int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) float fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyf(float __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float fmaf(float __x , float __y ,
                                                float __z ) ;
extern  __attribute__((__nothrow__)) float __fmaf(float __x , float __y ,
                                                  float __z ) ;
extern  __attribute__((__nothrow__)) float scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float __scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) long double acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atan2l(long double __y ,
                                                        long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
extern  __attribute__((__nothrow__)) long double cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double frexpl(long double __x ,
                                                        int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double __frexpl(long double __x ,
                                                          int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double ldexpl(long double __x ,
                                                        int __exponent ) ;
extern  __attribute__((__nothrow__)) long double __ldexpl(long double __x ,
                                                          int __exponent ) ;
extern  __attribute__((__nothrow__)) long double logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double modfl(long double __x ,
                                                       long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double __modfl(long double __x ,
                                                         long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double powl(long double __x ,
                                                      long double __y ) ;
extern  __attribute__((__nothrow__)) long double __powl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double hypotl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double __hypotl(long double __x ,
                                                          long double __y ) ;
extern  __attribute__((__nothrow__)) long double cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cbrtl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __ceill(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double fmodl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmodl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) int __isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double dreml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __dreml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double copysignl(long double __x ,
                                                           long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __copysignl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal_r(long double  ,
                                                           int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double __lgammal_r(long double  ,
                                                             int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double nextafterl(long double __x ,
                                                            long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nextafterl(long double __x ,
                                                              long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nexttowardl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nexttowardl(long double __x ,
                                                               long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double remainderl(long double __x ,
                                                            long double __y ) ;
extern  __attribute__((__nothrow__)) long double __remainderl(long double __x ,
                                                              long double __y ) ;
extern  __attribute__((__nothrow__)) long double scalbnl(long double __x ,
                                                         int __n ) ;
extern  __attribute__((__nothrow__)) long double __scalbnl(long double __x ,
                                                           int __n ) ;
extern  __attribute__((__nothrow__)) int ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) int __ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double scalblnl(long double __x ,
                                                          long __n ) ;
extern  __attribute__((__nothrow__)) long double __scalblnl(long double __x ,
                                                            long __n ) ;
extern  __attribute__((__nothrow__)) long double nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double remquol(long double __x ,
                                                         long double __y ,
                                                         int *__quo ) ;
extern  __attribute__((__nothrow__)) long double __remquol(long double __x ,
                                                           long double __y ,
                                                           int *__quo ) ;
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double fdiml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fdiml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmaxl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmaxl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fminl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fminl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyl(long double __value )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double fmal(long double __x ,
                                                      long double __y ,
                                                      long double __z ) ;
extern  __attribute__((__nothrow__)) long double __fmal(long double __x ,
                                                        long double __y ,
                                                        long double __z ) ;
extern  __attribute__((__nothrow__)) long double scalbl(long double __x ,
                                                        long double __n ) ;
extern  __attribute__((__nothrow__)) long double __scalbl(long double __x ,
                                                          long double __n ) ;
extern int signgam ;
extern _LIB_VERSION_TYPE _LIB_VERSION ;
extern int matherr(struct exception *__exc ) ;
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
__inline extern int __signbitf(float __x ) 
{ union __anonunion___u_24 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__f = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__u.__i < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
__inline extern int __signbit(double __x ) 
{ union __anonunion___u_25 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__d = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__u.__i[1] < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
__inline extern int __signbitl(long double __x ) 
{ union __anonunion___u_26 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__l = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__u.__i[2] & 0x8000) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern double __sgn(double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__x == 0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__x > 0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7553\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7554\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern float __sgnf(float __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((double )__x == 0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7559\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((double )__x > 0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7560\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7561\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern long double __sgnl(long double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__x == (long double )0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7566\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__x > (long double )0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7567\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7568\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7570\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((long double )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern long double __atan2l(long double __y , long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_atan2l(__y, __x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern double fabs(double __x ) 
{ double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabs(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern float fabsf(float __x ) 
{ float tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsf(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double __fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern long double __sgn1l(long double __x ) 
{ union __anonunion___n_27 __n ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xld = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[2] = (__n.__xi[2] & 32768U) | 16383U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[1] = 0x80000000;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[0] = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__n.__xld);
}
}
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
__inline extern double floor(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern float floorf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7590\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7591\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern long double floorl(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7592\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7593\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern double ceil(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7594\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7595\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern float ceilf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern long double ceill(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern long lrintf(float __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern long lrint(double __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern long lrintl(long double __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern long long llrintf(float __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern long long llrint(double __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7608\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7609\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern long long llrintl(long double __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7610\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern int __finite(double __x ) 
{ union __anonunion_28 __constr_expr_0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __constr_expr_0.__d = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )((((unsigned int )__constr_expr_0.__i[1] | 0x800fffffu) + 1U) >> 31));
}
}
extern struct _IO_FILE_plus _IO_2_1_stdin_ ;
extern struct _IO_FILE_plus _IO_2_1_stdout_ ;
extern struct _IO_FILE_plus _IO_2_1_stderr_ ;
extern void _IO_cookie_init(struct _IO_cookie_file *__cfile , int __read_write ,
                            void *__cookie , _IO_cookie_io_functions_t __fns ) ;
extern int __underflow(_IO_FILE * ) ;
extern int __uflow(_IO_FILE * ) ;
extern int __overflow(_IO_FILE * , int  ) ;
extern int _IO_getc(_IO_FILE *__fp ) ;
extern int _IO_putc(int __c , _IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_feof(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_ferror(_IO_FILE *__fp ) ;
extern int _IO_peekc_locked(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) void _IO_flockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_funlockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) int _IO_ftrylockfile(_IO_FILE * ) ;
extern int _IO_vfscanf(_IO_FILE * __restrict   , char const   * __restrict   ,
                       __gnuc_va_list  , int * __restrict   ) ;
extern int _IO_vfprintf(_IO_FILE * __restrict   , char const   * __restrict   ,
                        __gnuc_va_list  ) ;
extern __ssize_t _IO_padn(_IO_FILE * , int  , __ssize_t  ) ;
extern size_t _IO_sgetn(_IO_FILE * , void * , size_t  ) ;
extern __off64_t _IO_seekoff(_IO_FILE * , __off64_t  , int  , int  ) ;
extern __off64_t _IO_seekpos(_IO_FILE * , __off64_t  , int  ) ;
extern  __attribute__((__nothrow__)) void _IO_free_backup_area(_IO_FILE * ) ;
extern struct _IO_FILE *stdin ;
extern struct _IO_FILE *stdout ;
extern struct _IO_FILE *stderr ;
extern  __attribute__((__nothrow__)) int remove(char const   *__filename ) ;
extern  __attribute__((__nothrow__)) int rename(char const   *__old ,
                                                char const   *__new ) ;
extern  __attribute__((__nothrow__)) int renameat(int __oldfd ,
                                                  char const   *__old ,
                                                  int __newfd ,
                                                  char const   *__new ) ;
extern FILE *tmpfile(void) ;
extern  __attribute__((__nothrow__)) char *tmpnam(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tmpnam_r(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tempnam(char const   *__dir ,
                                                   char const   *__pfx )  __attribute__((__malloc__)) ;
extern int fflush_unlocked(FILE *__stream ) ;
extern FILE *freopen(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ,
                     FILE * __restrict  __stream ) ;
extern  __attribute__((__nothrow__)) FILE *fdopen(int __fd ,
                                                  char const   *__modes ) ;
extern  __attribute__((__nothrow__)) FILE *fmemopen(void *__s , size_t __len ,
                                                    char const   *__modes ) ;
extern  __attribute__((__nothrow__)) FILE *open_memstream(char **__bufloc ,
                                                          size_t *__sizeloc ) ;
extern  __attribute__((__nothrow__)) void setbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) int setvbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ,
                                                 int __modes , size_t __n ) ;
extern  __attribute__((__nothrow__)) void setbuffer(FILE * __restrict  __stream ,
                                                    char * __restrict  __buf ,
                                                    size_t __size ) ;
extern  __attribute__((__nothrow__)) void setlinebuf(FILE *__stream ) ;
extern int printf(char const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) int sprintf(char * __restrict  __s ,
                                                 char const   * __restrict  __format 
                                                 , ...) ;
extern int vfprintf(FILE * __restrict  __s ,
                    char const   * __restrict  __format , __gnuc_va_list __arg ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int vsprintf(char * __restrict  __s ,
                                                  char const   * __restrict  __format ,
                                                  __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  snprintf)(char * __restrict  __s ,
                                                                             size_t __maxlen ,
                                                                             char const   * __restrict  __format 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsnprintf)(char * __restrict  __s ,
                                                                              size_t __maxlen ,
                                                                              char const   * __restrict  __format ,
                                                                              __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vdprintf)(int __fd ,
                                               char const   * __restrict  __fmt ,
                                               __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  dprintf)(int __fd ,
                                              char const   * __restrict  __fmt 
                                              , ...) ;
extern int fscanf(FILE * __restrict  __stream ,
                  char const   * __restrict  __format  , ...)  __asm__("__isoc99_fscanf")  ;
extern int scanf(char const   * __restrict  __format  , ...)  __asm__("__isoc99_scanf")  ;
extern  __attribute__((__nothrow__)) int sscanf(char const   * __restrict  __s ,
                                                char const   * __restrict  __format 
                                                , ...)  __asm__("__isoc99_sscanf")  ;
extern int ( /* format attribute */  vfscanf)(FILE * __restrict  __s ,
                                              char const   * __restrict  __format ,
                                              __gnuc_va_list __arg )  __asm__("__isoc99_vfscanf")  ;
extern int ( /* format attribute */  vscanf)(char const   * __restrict  __format ,
                                             __gnuc_va_list __arg )  __asm__("__isoc99_vscanf")  ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsscanf)(char const   * __restrict  __s ,
                                                                            char const   * __restrict  __format ,
                                                                            __gnuc_va_list __arg )  __asm__("__isoc99_vsscanf")  ;
extern int fgetc(FILE *__stream ) ;
extern int getc(FILE *__stream ) ;
__inline extern int getchar(void) ;
__inline extern int getc_unlocked(FILE *__fp ) ;
__inline extern int getchar_unlocked(void) ;
__inline extern int fgetc_unlocked(FILE *__fp ) ;
extern int fputc(int __c , FILE *__stream ) ;
extern int putc(int __c , FILE *__stream ) ;
__inline extern int putchar(int __c ) ;
__inline extern int fputc_unlocked(int __c , FILE *__stream ) ;
__inline extern int putc_unlocked(int __c , FILE *__stream ) ;
__inline extern int putchar_unlocked(int __c ) ;
extern int getw(FILE *__stream ) ;
extern int putw(int __w , FILE *__stream ) ;
extern char *fgets(char * __restrict  __s , int __n ,
                   FILE * __restrict  __stream ) ;
extern char *gets(char *__s ) ;
extern __ssize_t __getdelim(char ** __restrict  __lineptr ,
                            size_t * __restrict  __n , int __delimiter ,
                            FILE * __restrict  __stream ) ;
extern __ssize_t getdelim(char ** __restrict  __lineptr ,
                          size_t * __restrict  __n , int __delimiter ,
                          FILE * __restrict  __stream ) ;
extern __ssize_t getline(char ** __restrict  __lineptr ,
                         size_t * __restrict  __n , FILE * __restrict  __stream ) ;
extern int fputs(char const   * __restrict  __s , FILE * __restrict  __stream ) ;
extern int puts(char const   *__s ) ;
extern int ungetc(int __c , FILE *__stream ) ;
extern size_t fread(void * __restrict  __ptr , size_t __size , size_t __n ,
                    FILE * __restrict  __stream ) ;
extern size_t fwrite(void const   * __restrict  __ptr , size_t __size ,
                     size_t __n , FILE * __restrict  __s ) ;
extern size_t fread_unlocked(void * __restrict  __ptr , size_t __size ,
                             size_t __n , FILE * __restrict  __stream ) ;
extern size_t fwrite_unlocked(void const   * __restrict  __ptr , size_t __size ,
                              size_t __n , FILE * __restrict  __stream ) ;
extern int fseek(FILE *__stream , long __off , int __whence ) ;
extern long ftell(FILE *__stream ) ;
extern void rewind(FILE *__stream ) ;
extern int fseeko(FILE *__stream , __off_t __off , int __whence ) ;
extern __off_t ftello(FILE *__stream ) ;
extern int fgetpos(FILE * __restrict  __stream , fpos_t * __restrict  __pos ) ;
extern int fsetpos(FILE *__stream , fpos_t const   *__pos ) ;
extern  __attribute__((__nothrow__)) void clearerr(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int feof(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ferror(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void clearerr_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
extern void perror(char const   *__s ) ;
extern int sys_nerr ;
extern char const   * const  sys_errlist[] ;
extern  __attribute__((__nothrow__)) int fileno(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int fileno_unlocked(FILE *__stream ) ;
extern FILE *popen(char const   *__command , char const   *__modes ) ;
extern int pclose(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) char *ctermid(char *__s ) ;
extern  __attribute__((__nothrow__)) void flockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ftrylockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void funlockfile(FILE *__stream ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = vfprintf((FILE */* __restrict  */)stdout, __fmt, __arg);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int getchar(void) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _IO_getc(stdin);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int fgetc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7621\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7622\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7627\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7629\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getchar_unlocked(void) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7639\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )stdin->_IO_read_ptr >= (unsigned int )stdin->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7640\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7634\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(stdin);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7635\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7636\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = stdin->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (stdin->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7641\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int putchar(int __c ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _IO_putc(__c, stdout);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7643\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int fputc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7646\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7647\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7648\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7649\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7650\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7654\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7655\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7656\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7657\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7658\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7659\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7660\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putchar_unlocked(int __c ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7671\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )stdout->_IO_write_ptr >= (unsigned int )stdout->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(stdout, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7665\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7666\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = stdout->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7667\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (stdout->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7668\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7670\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern int feof_unlocked(FILE *__stream ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7674\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__stream->_flags & 0x10) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern int ferror_unlocked(FILE *__stream ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7675\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__stream->_flags & 0x20) != 0);
}
}
extern  __attribute__((__nothrow__)) void *dlopen(char const   *__file ,
                                                  int __mode ) ;
extern  __attribute__((__nothrow__)) int dlclose(void *__handle )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *dlsym(void * __restrict  __handle ,
                                                 char const   * __restrict  __name )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *dlerror(void) ;
extern char __attribute__((__visibility__("default")))  *zend_strndup(char const   *s ,
                                                                      unsigned int length )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  *_emalloc(size_t size )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  *_safe_emalloc(size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  *_safe_malloc(size_t nmemb ,
                                                                      size_t size ,
                                                                      size_t offset )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  _efree(void *ptr ) ;
extern void __attribute__((__visibility__("default")))  *_ecalloc(size_t nmemb ,
                                                                  size_t size )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  *_erealloc(void *ptr ,
                                                                   size_t size ,
                                                                   int allow_failure ) ;
extern void __attribute__((__visibility__("default")))  *_safe_erealloc(void *ptr ,
                                                                        size_t nmemb ,
                                                                        size_t size ,
                                                                        size_t offset ) ;
extern void __attribute__((__visibility__("default")))  *_safe_realloc(void *ptr ,
                                                                       size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset ) ;
extern char __attribute__((__visibility__("default")))  *_estrdup(char const   *s )  __attribute__((__malloc__)) ;
extern char __attribute__((__visibility__("default")))  *_estrndup(char const   *s ,
                                                                   unsigned int length )  __attribute__((__malloc__)) ;
extern size_t __attribute__((__visibility__("default")))  _zend_mem_block_size(void *ptr ) ;
__inline static void *__zend_malloc(size_t len ) 
{ void *tmp ;
  void *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7678\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = malloc(len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7679\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7676\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7677\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  exit(1);
}
}
__inline static void *__zend_calloc(size_t nmemb , size_t len ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _safe_malloc(nmemb, len, 0U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (void *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset(tmp, 0, nmemb * len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static void *__zend_realloc(void *p , size_t len ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = realloc(p, len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (p) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7688\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  exit(1);
}
}
extern int __attribute__((__visibility__("default")))  zend_set_memory_limit(size_t memory_limit ) ;
extern void __attribute__((__visibility__("default")))  start_memory_manager(void) ;
extern void __attribute__((__visibility__("default")))  shutdown_memory_manager(int silent ,
                                                                                int full_shutdown ) ;
extern int __attribute__((__visibility__("default")))  is_zend_mm(void) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_usage(int real_usage ) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_peak_usage(int real_usage ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup(void) ;
extern void __attribute__((__visibility__("default")))  zend_mm_shutdown(zend_mm_heap *heap ,
                                                                         int full_shutdown ,
                                                                         int silent ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_alloc(zend_mm_heap *heap ,
                                                                        size_t size )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  _zend_mm_free(zend_mm_heap *heap ,
                                                                      void *p ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_realloc(zend_mm_heap *heap ,
                                                                          void *p ,
                                                                          size_t size ) ;
extern size_t __attribute__((__visibility__("default")))  _zend_mm_block_size(zend_mm_heap *heap ,
                                                                              void *p ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup_ex(zend_mm_mem_handlers const   *handlers ,
                                                                                    size_t block_size ,
                                                                                    size_t reserve_size ,
                                                                                    int internal ,
                                                                                    void *params ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_set_heap(zend_mm_heap *new_heap ) ;
extern zend_mm_storage __attribute__((__visibility__("default")))  *zend_mm_get_storage(zend_mm_heap *heap ) ;
extern void __attribute__((__visibility__("default")))  zend_mm_set_custom_handlers(zend_mm_heap *heap ,
                                                                                    void *(*_malloc)(size_t  ) ,
                                                                                    void (*_free)(void * ) ,
                                                                                    void *(*_realloc)(void * ,
                                                                                                      size_t  ) ) ;
extern char const __attribute__((__visibility__("default")))  *(*zend_new_interned_string)(char const   *str ,
                                                                                           int len ,
                                                                                           int free_src ) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_snapshot)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_restore)(void) ;
extern void zend_interned_strings_init(void) ;
extern void zend_interned_strings_dtor(void) ;
static char const   long_min_digits[11]  = 
  {      (char const   )'2',      (char const   )'1',      (char const   )'4',      (char const   )'7', 
        (char const   )'4',      (char const   )'8',      (char const   )'3',      (char const   )'6', 
        (char const   )'4',      (char const   )'8',      (char const   )'\000'};
extern int __attribute__((__visibility__("default")))  _zend_hash_init(HashTable *ht ,
                                                                       uint nSize ,
                                                                       ulong (*pHashFunction)(char const   *arKey ,
                                                                                              uint nKeyLength ) ,
                                                                       void (*pDestructor)(void *pDest ) ,
                                                                       zend_bool persistent ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_init_ex(HashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ,
                                                                          zend_bool bApplyProtection ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_destroy(HashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_clean(HashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_add_or_update(HashTable *ht ,
                                                                                char const   *arKey ,
                                                                                uint nKeyLength ,
                                                                                void *pData ,
                                                                                uint nDataSize ,
                                                                                void **pDest ,
                                                                                int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_quick_add_or_update(HashTable *ht ,
                                                                                      char const   *arKey ,
                                                                                      uint nKeyLength ,
                                                                                      ulong h ,
                                                                                      void *pData ,
                                                                                      uint nDataSize ,
                                                                                      void **pDest ,
                                                                                      int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_index_update_or_next_insert(HashTable *ht ,
                                                                                              ulong h ,
                                                                                              void *pData ,
                                                                                              uint nDataSize ,
                                                                                              void **pDest ,
                                                                                              int flag ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_add_empty_element(HashTable *ht ,
                                                                                   char const   *arKey ,
                                                                                   uint nKeyLength ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_destroy(HashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_reverse_destroy(HashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply(HashTable *ht ,
                                                                        int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_argument(HashTable *ht ,
                                                                                      int (*apply_func)(void *pDest ,
                                                                                                        void *argument ) ,
                                                                                      void * ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_arguments(HashTable *ht ,
                                                                                       int (*apply_func)(void *pDest ,
                                                                                                         int num_args ,
                                                                                                         va_list args ,
                                                                                                         zend_hash_key *hash_key ) ,
                                                                                       int  
                                                                                       , ...) ;
extern void __attribute__((__visibility__("default")))  zend_hash_reverse_apply(HashTable *ht ,
                                                                                int (*apply_func)(void *pDest ) ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_del_key_or_index(HashTable *ht ,
                                                                                  char const   *arKey ,
                                                                                  uint nKeyLength ,
                                                                                  ulong h ,
                                                                                  int flag ) ;
extern ulong __attribute__((__visibility__("default")))  zend_get_hash_value(char const   *arKey ,
                                                                             uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_find(HashTable const   *ht ,
                                                                      char const   *arKey ,
                                                                      uint nKeyLength ,
                                                                      void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_find(HashTable const   *ht ,
                                                                            char const   *arKey ,
                                                                            uint nKeyLength ,
                                                                            ulong h ,
                                                                            void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_index_find(HashTable const   *ht ,
                                                                            ulong h ,
                                                                            void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_exists(HashTable const   *ht ,
                                                                        char const   *arKey ,
                                                                        uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_exists(HashTable const   *ht ,
                                                                              char const   *arKey ,
                                                                              uint nKeyLength ,
                                                                              ulong h ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_index_exists(HashTable const   *ht ,
                                                                              ulong h ) ;
extern ulong __attribute__((__visibility__("default")))  zend_hash_next_free_element(HashTable const   *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_move_forward_ex(HashTable *ht ,
                                                                                 HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_move_backwards_ex(HashTable *ht ,
                                                                                   HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_ex(HashTable const   *ht ,
                                                                                    char **str_index ,
                                                                                    uint *str_length ,
                                                                                    ulong *num_index ,
                                                                                    zend_bool duplicate ,
                                                                                    HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_type_ex(HashTable *ht ,
                                                                                         HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_data_ex(HashTable *ht ,
                                                                                     void **pData ,
                                                                                     HashPosition *pos ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_reset_ex(HashTable *ht ,
                                                                                            HashPosition *pos ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_end_ex(HashTable *ht ,
                                                                                          HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_update_current_key_ex(HashTable *ht ,
                                                                                       int key_type ,
                                                                                       char const   *str_index ,
                                                                                       uint str_length ,
                                                                                       ulong num_index ,
                                                                                       int mode ,
                                                                                       HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_pointer(HashTable const   *ht ,
                                                                             HashPointer *ptr ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_set_pointer(HashTable *ht ,
                                                                             HashPointer const   *ptr ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_copy(HashTable *target ,
                                                                       HashTable *source ,
                                                                       void (*pCopyConstructor)(void *pElement ) ,
                                                                       void *tmp ,
                                                                       uint size ) ;
extern void __attribute__((__visibility__("default")))  _zend_hash_merge(HashTable *target ,
                                                                         HashTable *source ,
                                                                         void (*pCopyConstructor)(void *pElement ) ,
                                                                         void *tmp ,
                                                                         uint size ,
                                                                         int overwrite ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_merge_ex(HashTable *target ,
                                                                           HashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           uint size ,
                                                                           zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                     void *source_data ,
                                                                                                     zend_hash_key *hash_key ,
                                                                                                     void *pParam ) ,
                                                                           void *pParam ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_sort(HashTable *ht ,
                                                                      void (*sort_func)(void * ,
                                                                                        size_t  ,
                                                                                        size_t  ,
                                                                                        int (*)(void const   * ,
                                                                                                void const   * ) ) ,
                                                                      int (*compare_func)(void const   * ,
                                                                                          void const   * ) ,
                                                                      int renumber ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_compare(HashTable *ht1 ,
                                                                         HashTable *ht2 ,
                                                                         int (*compar)(void const   * ,
                                                                                       void const   * ) ,
                                                                         zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_minmax(HashTable const   *ht ,
                                                                        int (*compar)(void const   * ,
                                                                                      void const   * ) ,
                                                                        int flag ,
                                                                        void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_num_elements(HashTable const   *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_rehash(HashTable *ht ) ;
__inline static ulong zend_inline_hash_func(char const   *arKey ,
                                            uint nKeyLength ) 
{ register ulong hash ;
  char const   *tmp ;
  char const   *tmp___0 ;
  char const   *tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  char const   *tmp___4 ;
  char const   *tmp___5 ;
  char const   *tmp___6 ;
  char const   *tmp___7 ;
  char const   *tmp___8 ;
  char const   *tmp___9 ;
  char const   *tmp___10 ;
  char const   *tmp___11 ;
  char const   *tmp___12 ;
  char const   *tmp___13 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = (ulong )5381;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7694\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (nKeyLength >= 8U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7693\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7695\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7696\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7698\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7699\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7700\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7706\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7707\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7710\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7711\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7712\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___4;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7713\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7714\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___5;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___6 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7717\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7719\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    nKeyLength -= 8U;
  }
  switch ((int )nKeyLength) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7720\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 7: 
  tmp___7 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7721\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7722\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7723\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 6: 
  tmp___8 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___8;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 5: 
  tmp___9 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___9;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 4: 
  tmp___10 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 3: 
  tmp___11 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7734\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___11;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7735\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 2: 
  tmp___12 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7736\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7737\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___12;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  tmp___13 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___13;
  break;
  case 0: 
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (hash);
}
}
extern ulong __attribute__((__visibility__("default")))  zend_hash_func(char const   *arKey ,
                                                                        uint nKeyLength ) ;
__inline static int zend_symtable_update(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength , void *pData ,
                                         uint nDataSize , void **pDest ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7779\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7776\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7777\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7744\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7745\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7778\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7774\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7768\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7769\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7752\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7746\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7751\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7750\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7748\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7747\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7749\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7770\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7771\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7756\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7757\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7755\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7754\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7753\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7758\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7772\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7764\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7760\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7759\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7761\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7763\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7762\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7765\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = _zend_hash_index_update_or_next_insert(ht, idx, pData,
                                                             nDataSize, pDest, 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7766\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7767\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7773\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7775\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _zend_hash_add_or_update(ht, arKey, nKeyLength, pData, nDataSize,
                                     pDest, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7782\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_del(HashTable *ht , char const   *arKey ,
                                      uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7818\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7815\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7816\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7783\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7784\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7817\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7813\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7807\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7808\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7791\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7785\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7790\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7789\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7787\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7786\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7788\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7809\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7810\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7795\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7796\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7794\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7793\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7792\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7797\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7811\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7803\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7799\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7798\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7800\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7802\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7801\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7804\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_del_key_or_index(ht,
                                                 (char const   *)((void *)0),
                                                 0U, idx, 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7805\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7806\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7812\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7814\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_del_key_or_index(ht, arKey, nKeyLength, 0UL, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_find(HashTable *ht , char const   *arKey ,
                                       uint nKeyLength , void **pData ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7858\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7854\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7822\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7823\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7856\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7852\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7846\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7847\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7830\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7824\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7829\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7828\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7826\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7825\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7827\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7848\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7849\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7834\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7835\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7833\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7832\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7831\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7836\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7850\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7842\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7838\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7837\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7839\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7841\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7840\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7843\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_index_find((HashTable const   *)ht, idx, pData);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7844\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7845\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7851\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7853\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7859\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_find((HashTable const   *)ht, arKey, nKeyLength, pData);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_exists(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7896\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7893\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7894\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7861\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7862\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7895\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7891\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7885\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7886\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7869\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7863\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7868\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7867\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7865\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7864\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7866\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7887\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7888\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7873\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7874\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7872\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7871\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7870\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7875\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7889\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7881\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7877\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7876\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7878\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7880\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7879\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7882\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_index_exists((HashTable const   *)ht, idx);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7883\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7884\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7890\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7892\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_exists((HashTable const   *)ht, arKey, nKeyLength);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7899\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_update_current_key_ex(HashTable *ht ,
                                                        char const   *arKey ,
                                                        uint nKeyLength ,
                                                        int mode ,
                                                        HashPosition *pos ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7936\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7935\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7932\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7933\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7900\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7901\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7934\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7930\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7924\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7925\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7908\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7902\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7907\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7906\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7904\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7903\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7905\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7926\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7927\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7912\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7913\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7911\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7910\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7909\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7914\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7928\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7920\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7916\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7915\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7917\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7919\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7918\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7921\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_update_current_key_ex(ht, 2,
                                                      (char const   *)((void *)0),
                                                      0U, idx, mode, pos);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7922\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7923\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7929\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7931\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_update_current_key_ex(ht, 1, arKey, nKeyLength, 0UL, mode,
                                            pos);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init(TsHashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init_ex(TsHashTable *ht ,
                                                                             uint nSize ,
                                                                             ulong (*pHashFunction)(char const   *arKey ,
                                                                                                    uint nKeyLength ) ,
                                                                             void (*pDestructor)(void *pDest ) ,
                                                                             zend_bool persistent ,
                                                                             zend_bool bApplyProtection ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_destroy(TsHashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_clean(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_add_or_update(TsHashTable *ht ,
                                                                                   char *arKey ,
                                                                                   uint nKeyLength ,
                                                                                   void *pData ,
                                                                                   uint nDataSize ,
                                                                                   void **pDest ,
                                                                                   int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_quick_add_or_update(TsHashTable *ht ,
                                                                                         char *arKey ,
                                                                                         uint nKeyLength ,
                                                                                         ulong h ,
                                                                                         void *pData ,
                                                                                         uint nDataSize ,
                                                                                         void **pDest ,
                                                                                         int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_index_update_or_next_insert(TsHashTable *ht ,
                                                                                                 ulong h ,
                                                                                                 void *pData ,
                                                                                                 uint nDataSize ,
                                                                                                 void **pDest ,
                                                                                                 int flag ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_add_empty_element(TsHashTable *ht ,
                                                                                      char *arKey ,
                                                                                      uint nKeyLength ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_graceful_destroy(TsHashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply(TsHashTable *ht ,
                                                                           int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_argument(TsHashTable *ht ,
                                                                                         int (*apply_func)(void *pDest ,
                                                                                                           void *argument ) ,
                                                                                         void * ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_arguments(TsHashTable *ht ,
                                                                                          int (*apply_func)(void *pDest ,
                                                                                                            int num_args ,
                                                                                                            va_list args ,
                                                                                                            zend_hash_key *hash_key ) ,
                                                                                          int  
                                                                                          , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_reverse_apply(TsHashTable *ht ,
                                                                                   int (*apply_func)(void *pDest ) ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_del_key_or_index(TsHashTable *ht ,
                                                                                     char *arKey ,
                                                                                     uint nKeyLength ,
                                                                                     ulong h ,
                                                                                     int flag ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_get_hash_value(TsHashTable *ht ,
                                                                                char *arKey ,
                                                                                uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_find(TsHashTable *ht ,
                                                                         char *arKey ,
                                                                         uint nKeyLength ,
                                                                         void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_quick_find(TsHashTable *ht ,
                                                                               char *arKey ,
                                                                               uint nKeyLength ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_find(TsHashTable *ht ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_exists(TsHashTable *ht ,
                                                                           char *arKey ,
                                                                           uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_exists(TsHashTable *ht ,
                                                                                 ulong h ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy(TsHashTable *target ,
                                                                          TsHashTable *source ,
                                                                          void (*pCopyConstructor)(void *pElement ) ,
                                                                          void *tmp ,
                                                                          uint size ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy_to_hash(HashTable *target ,
                                                                                  TsHashTable *source ,
                                                                                  void (*pCopyConstructor)(void *pElement ) ,
                                                                                  void *tmp ,
                                                                                  uint size ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge(TsHashTable *target ,
                                                                           TsHashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           void *tmp ,
                                                                           uint size ,
                                                                           int overwrite ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge_ex(TsHashTable *target ,
                                                                              TsHashTable *source ,
                                                                              void (*pCopyConstructor)(void *pElement ) ,
                                                                              uint size ,
                                                                              zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                        void *source_data ,
                                                                                                        zend_hash_key *hash_key ,
                                                                                                        void *pParam ) ,
                                                                              void *pParam ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_sort(TsHashTable *ht ,
                                                                         void (*sort_func)(void * ,
                                                                                           size_t  ,
                                                                                           size_t  ,
                                                                                           int (*)(void const   * ,
                                                                                                   void const   * ) ) ,
                                                                         int (*compare_func)(void const   * ,
                                                                                             void const   * ) ,
                                                                         int renumber ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_compare(TsHashTable *ht1 ,
                                                                            TsHashTable *ht2 ,
                                                                            int (*compar)(void const   * ,
                                                                                          void const   * ) ,
                                                                            zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_minmax(TsHashTable *ht ,
                                                                           int (*compar)(void const   * ,
                                                                                         void const   * ) ,
                                                                           int flag ,
                                                                           void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_num_elements(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_rehash(TsHashTable *ht ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_hash_func(char *arKey ,
                                                                           uint nKeyLength ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_init(zend_llist *l ,
                                                                        size_t size ,
                                                                        void (*dtor)(void * ) ,
                                                                        unsigned char persistent ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_add_element(zend_llist *l ,
                                                                               void *element ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_prepend_element(zend_llist *l ,
                                                                                   void *element ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_del_element(zend_llist *l ,
                                                                               void *element ,
                                                                               int (*compare)(void *element1 ,
                                                                                              void *element2 ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_destroy(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_clean(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_remove_tail(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_copy(zend_llist *dst ,
                                                                        zend_llist *src ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply(zend_llist *l ,
                                                                         void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_del(zend_llist *l ,
                                                                                  int (*func)(void *data ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_argument(zend_llist *l ,
                                                                                       void (*func)(void *data ,
                                                                                                    void *arg ) ,
                                                                                       void *arg ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_arguments(zend_llist *l ,
                                                                                        void (*func)(void *data ,
                                                                                                     int num_args ,
                                                                                                     va_list args ) ,
                                                                                        int num_args 
                                                                                        , ...) ;
extern int __attribute__((__visibility__("default")))  zend_llist_count(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_sort(zend_llist *l ,
                                                                        int (*comp_func)(zend_llist_element const   ** ,
                                                                                         zend_llist_element const   ** ) ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_first_ex(zend_llist *l ,
                                                                                 zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_last_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_next_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_prev_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern  __attribute__((__noreturn__)) void zend_error_noreturn(int type ,
                                                               char const   *format 
                                                               , ...) ;
extern zend_object_handlers __attribute__((__visibility__("default")))  std_object_handlers ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_static_method(zend_class_entry *ce ,
                                                                                                    char const   *function_name_strval ,
                                                                                                    int function_name_strlen ,
                                                                                                    struct _zend_literal  const  *key ) ;
extern zval __attribute__((__visibility__("default")))  **zend_std_get_static_property(zend_class_entry *ce ,
                                                                                       char const   *property_name ,
                                                                                       int property_name_len ,
                                                                                       zend_bool silent ,
                                                                                       struct _zend_literal  const  *key ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_std_unset_static_property(zend_class_entry *ce ,
                                                                                            char const   *property_name ,
                                                                                            int property_name_len ,
                                                                                            struct _zend_literal  const  *key ) ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_constructor(zval *object ) ;
extern struct _zend_property_info  __attribute__((__visibility__("default"))) *zend_get_property_info(zend_class_entry *ce ,
                                                                                                      zval *member ,
                                                                                                      int silent ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_properties(zval *object ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_debug_info(zval *object ,
                                                                                      int *is_temp ) ;
extern int __attribute__((__visibility__("default")))  zend_std_cast_object_tostring(zval *readobj ,
                                                                                     zval *writeobj ,
                                                                                     int type ) ;
extern void __attribute__((__visibility__("default")))  zend_std_write_property(zval *object ,
                                                                                zval *member ,
                                                                                zval *value ,
                                                                                struct _zend_literal  const  *key ) ;
extern void __attribute__((__visibility__("default")))  rebuild_object_properties(zend_object *zobj ) ;
extern int __attribute__((__visibility__("default")))  zend_check_private(union _zend_function *fbc ,
                                                                          zend_class_entry *ce ,
                                                                          char *function_name_strval ,
                                                                          int function_name_strlen ) ;
extern int __attribute__((__visibility__("default")))  zend_check_protected(zend_class_entry *ce ,
                                                                            zend_class_entry *scope ) ;
extern int __attribute__((__visibility__("default")))  zend_check_property_access(zend_object *zobj ,
                                                                                  char const   *prop_info_name ,
                                                                                  int prop_info_name_len ) ;
extern void __attribute__((__visibility__("default")))  zend_std_call_user_call(int ht ,
                                                                                zval *return_value ,
                                                                                zval **return_value_ptr ,
                                                                                zval *this_ptr ,
                                                                                int return_value_used ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_refcount_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_set_refcount_p)(zval *pz ,
                                                                                    zend_uint rc ) 
{ zend_uint tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = rc;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7941\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->refcount__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7942\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_addref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7943\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (pz->refcount__gc) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7944\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_delref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7945\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (pz->refcount__gc) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7946\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_isref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7947\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->is_ref__gc);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7948\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7949\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7950\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_unset_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7951\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7952\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7953\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_to_p)(zval *pz ,
                                                                                    zend_bool isref ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7954\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = isref;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7955\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7956\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
extern enum zend_object_iterator_kind  __attribute__((__visibility__("default"))) zend_iterator_unwrap(zval *array_ptr ,
                                                                                                       zend_object_iterator **iter ) ;
extern zval __attribute__((__visibility__("default")))  *zend_iterator_wrap(zend_object_iterator *iter ) ;
extern void __attribute__((__visibility__("default")))  zend_register_iterator_wrapper(void) ;
extern int __attribute__((__visibility__("default")))  zend_stream_open(char const   *filename ,
                                                                        zend_file_handle *handle ) ;
extern int __attribute__((__visibility__("default")))  zend_stream_fixup(zend_file_handle *file_handle ,
                                                                         char **buf ,
                                                                         size_t *len ) ;
extern void __attribute__((__visibility__("default")))  zend_file_handle_dtor(zend_file_handle *fh ) ;
extern int __attribute__((__visibility__("default")))  zend_compare_file_handles(zend_file_handle *fh1 ,
                                                                                 zend_file_handle *fh2 ) ;
extern int zend_startup(zend_utility_functions *utility_functions ,
                        char **extensions ) ;
extern void zend_shutdown(void) ;
extern void zend_register_standard_ini_entries(void) ;
extern void zend_post_startup(void) ;
extern void zend_set_utility_values(zend_utility_values *utility_values ) ;
extern void __attribute__((__visibility__("default")))  _zend_bailout(char *filename ,
                                                                      uint lineno ) ;
extern char __attribute__((__visibility__("default")))  *get_zend_version(void) ;
extern void __attribute__((__visibility__("default")))  zend_make_printable_zval(zval *expr ,
                                                                                 zval *expr_copy ,
                                                                                 int *use_copy ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval(zval *expr ,
                                                                       int indent ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval_ex(int (*write_func)(char const   *str ,
                                                                                            uint str_length ) ,
                                                                          zval *expr ,
                                                                          int indent ) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r(zval *expr ,
                                                                          int indent ) ;
extern void __attribute__((__visibility__("default")))  zend_print_flat_zval_r(zval *expr ) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r_ex(int (*write_func)(char const   *str ,
                                                                                               uint str_length ) ,
                                                                             zval *expr ,
                                                                             int indent ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_output_debug_string)(zend_bool trigger_break ,
                                                                                                            char const   *format 
                                                                                                            , ...) ;
extern void zend_activate(void) ;
extern void zend_deactivate(void) ;
extern void zend_call_destructors(void) ;
extern void zend_activate_modules(void) ;
extern void zend_deactivate_modules(void) ;
extern void zend_post_deactivate_modules(void) ;
extern void __attribute__((__visibility__("default")))  free_estring(char **str_p ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_printf))(char const   *format 
                                                                                                 , ...) ;
extern int (* __attribute__((__visibility__("default"))) zend_write)(char const   *str ,
                                                                     uint str_length ) ;
extern FILE __attribute__((__visibility__("default")))  *(*zend_fopen)(char const   *filename ,
                                                                       char **opened_path ) ;
extern void __attribute__((__visibility__("default")))  (*zend_block_interruptions)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_unblock_interruptions)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_ticks_function)(int ticks ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_error_cb))(int type ,
                                                                                                    char const   *error_filename ,
                                                                                                    uint error_lineno ,
                                                                                                    char const   *format ,
                                                                                                    va_list args ) ;
extern void __attribute__((__visibility__("default")))  (*zend_on_timeout)(int seconds ) ;
extern int __attribute__((__visibility__("default")))  (*zend_stream_open_function)(char const   *filename ,
                                                                                    zend_file_handle *handle ) ;
extern int (*zend_vspprintf)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_getenv)(char *name ,
                                                                        size_t name_len ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_resolve_path)(char const   *filename ,
                                                                              int filename_len ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_error)(int type ,
                                                                                              char const   *format 
                                                                                              , ...) ;
extern void zenderror(char const   *error ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_standard_class_def ;
extern zend_utility_values __attribute__((__visibility__("default")))  zend_uv ;
extern zval __attribute__((__visibility__("default")))  zval_used_for_init ;
extern void __attribute__((__visibility__("default")))  zend_message_dispatcher(long message ,
                                                                                void const   *data ) ;
extern int __attribute__((__visibility__("default")))  zend_get_configuration_directive(char const   *name ,
                                                                                        uint name_length ,
                                                                                        zval *contents ) ;
extern zend_gc_globals __attribute__((__visibility__("default")))  gc_globals ;
extern int __attribute__((__visibility__("default")))  gc_collect_cycles(void) ;
extern void __attribute__((__visibility__("default")))  gc_zval_possible_root(zval *zv ) ;
extern void __attribute__((__visibility__("default")))  gc_zobj_possible_root(zval *zv ) ;
extern void __attribute__((__visibility__("default")))  gc_remove_zval_from_buffer(zval *zv ) ;
extern void __attribute__((__visibility__("default")))  gc_globals_ctor(void) ;
extern void __attribute__((__visibility__("default")))  gc_globals_dtor(void) ;
extern void __attribute__((__visibility__("default")))  gc_init(void) ;
extern void __attribute__((__visibility__("default")))  gc_reset(void) ;
__inline static void ( __attribute__((__always_inline__)) gc_zval_check_possible_root)(zval *z ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7961\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )z->type == 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    gc_zval_possible_root(z);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7960\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )z->type == 5) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7958\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      gc_zval_possible_root(z);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7959\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7962\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) gc_remove_from_buffer)(gc_root_buffer *root ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7963\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (root->next)->prev = root->prev;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7964\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (root->prev)->next = root->next;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7965\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  root->prev = gc_globals.unused;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7966\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  gc_globals.unused = root;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7967\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_fail(char const   *__assertion ,
                                  char const   *__file , unsigned int __line ,
                                  char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_perror_fail(int __errnum , char const   *__file ,
                                         unsigned int __line ,
                                         char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert(char const   *__assertion , char const   *__file ,
                             int __line ) ;
extern void __attribute__((__visibility__("default")))  zend_freedtoa(char *s ) ;
extern char __attribute__((__visibility__("default")))  *zend_dtoa(double _d ,
                                                                   int mode ,
                                                                   int ndigits ,
                                                                   int *decpt ,
                                                                   int *sign ,
                                                                   char **rve ) ;
extern double __attribute__((__visibility__("default")))  zend_strtod(char const   *s00 ,
                                                                      char const   **se ) ;
extern double __attribute__((__visibility__("default")))  zend_hex_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern double __attribute__((__visibility__("default")))  zend_oct_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern double __attribute__((__visibility__("default")))  zend_bin_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_strtod(void) ;
extern int __attribute__((__visibility__("default")))  zend_shutdown_strtod(void) ;
extern int __attribute__((__visibility__("default")))  add_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  sub_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mul_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  div_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mod_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  boolean_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  boolean_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_or_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_and_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_left_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_right_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  concat_function(zval *result ,
                                                                       zval *op1 ,
                                                                       zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_equal_function(zval *result ,
                                                                         zval *op1 ,
                                                                         zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_identical_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_identical_function(zval *result ,
                                                                                 zval *op1 ,
                                                                                 zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_equal_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_or_equal_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function_ex(zend_class_entry const   *instance_ce ,
                                                                                    zend_class_entry const   *ce ,
                                                                                    zend_bool interfaces_only ) ;
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function(zend_class_entry const   *instance_ce ,
                                                                                 zend_class_entry const   *ce ) ;
__inline static long ( __attribute__((__always_inline__)) zend_dval_to_lval)(double d ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7972\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (d > (double )2147483647L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7968\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((long )((unsigned long )((long long )d)));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7971\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (d < (double )(-0x7FFFFFFF-1)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7969\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((long )((unsigned long )((long long )d)));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7970\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7973\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((long )d);
}
}
__inline static zend_uchar is_numeric_string(char const   *str , int length ,
                                             long *lval , double *dval ,
                                             int allow_errors ) 
{ char const   *ptr ;
  int base ;
  int digits ;
  int dp_or_e ;
  double local_dval ;
  zend_uchar type ;
  char const   *e ;
  char const   *tmp ;
  double __attribute__((__visibility__("default")))  tmp___0 ;
  double __attribute__((__visibility__("default")))  tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  int cmp ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___7 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  double __attribute__((__visibility__("default")))  tmp___11 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  base = 10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  digits = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dp_or_e = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! length) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7987\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*str == 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7976\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*str == 9) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7977\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*str == 10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7978\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7984\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*str == 13) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7979\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7983\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*str == 11) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7980\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7982\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*str == 12) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7981\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                break;
              }
            }
          }
        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    str ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = str;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*ptr == 45) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7990\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ptr ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*ptr == 43) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7991\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ptr ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7992\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*ptr >= 48) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8062\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*ptr <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8057\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (length > 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8002\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*str == 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8000\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*(str + 1) == 120) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7994\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            base = 16;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7995\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ptr += 2;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7999\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*(str + 1) == 88) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7996\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              base = 16;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7997\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ptr += 2;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7998\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8001\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8003\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8058\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8005\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*ptr == 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8004\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8006\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ptr ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8059\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8060\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8040\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits >= 11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8009\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (dval) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8008\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (allow_errors == 1) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8007\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8010\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8041\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        check_digits: 
        if ((int const   )*ptr >= 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8011\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*ptr <= 57) {
            goto __Cont;
          } else {
            goto _L___3;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8039\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___3: /* CIL Label */ 
          if (base == 16) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8015\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*ptr >= 65) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8012\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*ptr <= 70) {
                goto __Cont;
              } else {
                goto _L___2;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8014\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___2: /* CIL Label */ 
              if ((int const   )*ptr >= 97) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8013\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*ptr <= 102) {
                  goto __Cont;
                } else {
                  goto _L___1;
                }
              } else {
                goto _L___1;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8038\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___1: /* CIL Label */ 
            if (base == 10) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8036\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*ptr == 46) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8016\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (dp_or_e < 1) {
                  goto process_double;
                } else {
                  goto _L___0;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8035\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((int const   )*ptr == 101) {
                  goto _L;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8034\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*ptr == 69) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "8032\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L: /* CIL Label */ 
                    if (dp_or_e < 2) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "8028\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      e = ptr + 1;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "8029\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*e == 45) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8017\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp = e;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8018\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        e ++;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8019\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        ptr = tmp;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8024\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((int const   )*e == 43) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "8020\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp = e;
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "8021\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          e ++;
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "8022\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          ptr = tmp;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "8023\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "8030\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*e >= 48) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8026\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((int const   )*e <= 57) {
                          goto process_double;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "8025\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "8027\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "8031\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "8033\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8037\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8042\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __Cont: /* CIL Label */ 
        digits ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8043\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ptr ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8061\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (base == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8046\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits >= 11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8044\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          dp_or_e = -1;
          goto process_double;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8045\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8056\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits < 8) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8047\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8055\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (digits == 8) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8049\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*(ptr + - digits) <= 55) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8048\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              goto _L___4;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8053\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___4: /* CIL Label */ 
            if (dval) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8050\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___0 = zend_hex_strtod(str, & ptr);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8051\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              local_dval = (double )tmp___0;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8052\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8054\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            type = (unsigned char)2;
          }
        }
      }
    } else {
      goto _L___5;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8081\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___5: /* CIL Label */ 
    if ((int const   )*ptr == 46) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8079\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(ptr + 1) >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8077\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(ptr + 1) <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8074\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          process_double: 
          type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8075\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (dval) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8063\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = zend_strtod(str, & ptr);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8064\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            local_dval = (double )tmp___1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8073\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (allow_errors != 1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8071\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (dp_or_e != -1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8067\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___3 = ptr;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8068\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ptr ++;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8069\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp___3 == 46) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8065\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  dp_or_e = 1;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8066\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  dp_or_e = 2;
                }
                goto check_digits;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8070\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8072\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8076\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)0);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8078\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)0);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8080\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )ptr != (unsigned int )(str + length)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8086\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! allow_errors) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8082\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8083\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8087\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (allow_errors == -1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8084\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 3L, "A non well formed numeric value encountered");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8088\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8121\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (digits == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8115\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8100\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s1_len = strlen(ptr + - digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8101\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s2_len = strlen(long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8102\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! ((unsigned int )((void const   *)((ptr + - digits) + 1)) - (unsigned int )((void const   *)(ptr + - digits)) == 1U)) {
          goto _L___7;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8095\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s1_len >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8093\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___7: /* CIL Label */ 
            if (! ((unsigned int )((void const   *)(long_min_digits + 1)) - (unsigned int )((void const   *)(long_min_digits)) == 1U)) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8089\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___10 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8092\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (__s2_len >= 4U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8090\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___10 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8091\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___10 = 0;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8094\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___10 = 0;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8103\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8096\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8097\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = tmp___5;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8098\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8099\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = tmp___8;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8104\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8105\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___9 = tmp___8;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8116\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      cmp = tmp___9;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8117\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (cmp < 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8106\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8114\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (cmp == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8108\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*str == 45) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8107\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            goto _L___8;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8112\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___8: /* CIL Label */ 
          if (dval) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8109\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___11 = zend_strtod(str, (char const   **)((void *)0));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8110\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *dval = (double )tmp___11;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8111\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8113\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)2);
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8118\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8122\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (lval) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8119\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *lval = strtol((char const   */* __restrict  */)str,
                     (char **/* __restrict  */)((void *)0), base);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8120\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8123\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8126\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (dval) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8124\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *dval = local_dval;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8125\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8127\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)2);
  }
}
}
__inline static char *zend_memnstr(char *haystack , char *needle ,
                                   int needle_len , char *end ) 
{ char *p ;
  char ne ;
  void *tmp ;
  int tmp___0 ;
  void *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = haystack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ne = *(needle + (needle_len - 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (needle_len == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8138\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = memchr((void const   *)p, (int )*needle, (unsigned int )(end - p));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8139\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8140\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (needle_len > end - haystack) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8141\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8142\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end -= needle_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8153\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )p <= (unsigned int )end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8143\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8154\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = memchr((void const   *)p, (int )*needle,
                     (unsigned int )((end - p) + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8155\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = (char *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8156\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (p) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8149\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ne == (int )*(p + (needle_len - 1))) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8146\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = memcmp((void const   *)needle, (void const   *)p,
                         (unsigned int )(needle_len - 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8147\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8144\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8145\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (p);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8148\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8150\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8157\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )p == (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8151\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((char *)((void *)0));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8152\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8158\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char *)((void *)0));
}
}
__inline static void const   *zend_memrchr(void const   *s , int c , size_t n ) 
{ register unsigned char const   *e ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (n <= 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8166\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((void const   *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8167\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  e = ((unsigned char const   *)s + n) - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8171\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )e >= (unsigned int )((unsigned char const   *)s)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8168\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8172\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*e == (int const   )((unsigned char const   )c)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8169\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((void const   *)e);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8170\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8173\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    e --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((void const   *)((void *)0));
}
}
extern int __attribute__((__visibility__("default")))  increment_function(zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  decrement_function(zval *op2 ) ;
extern void __attribute__((__visibility__("default")))  convert_scalar_to_number(zval *op ) ;
extern void __attribute__((__visibility__("default")))  _convert_to_string(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_long(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_double(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_long_base(zval *op ,
                                                                             int base ) ;
extern void __attribute__((__visibility__("default")))  convert_to_null(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_boolean(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_array(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_object(zval *op ) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_long_ex(int argc 
                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_double_ex(int argc 
                                                                                   , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_string_ex(int argc 
                                                                                   , ...) ;
extern int __attribute__((__visibility__("default")))  add_char_to_string(zval *result ,
                                                                          zval const   *op1 ,
                                                                          zval const   *op2 ) ;
extern int __attribute__((__visibility__("default")))  add_string_to_string(zval *result ,
                                                                            zval const   *op1 ,
                                                                            zval const   *op2 ) ;
extern double __attribute__((__visibility__("default")))  zend_string_to_double(char const   *number ,
                                                                                zend_uint length ) ;
extern int __attribute__((__visibility__("default")))  zval_is_true(zval *op ) ;
extern int __attribute__((__visibility__("default")))  compare_function(zval *result ,
                                                                        zval *op1 ,
                                                                        zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  numeric_compare_function(zval *result ,
                                                                                zval *op1 ,
                                                                                zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function_ex(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ,
                                                                                  zend_bool case_insensitive ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function(zval *result ,
                                                                               zval *op1 ,
                                                                               zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_case_compare_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_locale_compare_function(zval *result ,
                                                                                      zval *op1 ,
                                                                                      zval *op2 ) ;
extern void __attribute__((__visibility__("default")))  zend_str_tolower(char *str ,
                                                                         unsigned int length ) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_copy(char *dest ,
                                                                               char const   *source ,
                                                                               unsigned int length ) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_dup(char const   *source ,
                                                                              unsigned int length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcmp(zval *s1 ,
                                                                               zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncmp(zval *s1 ,
                                                                                zval *s2 ,
                                                                                zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcasecmp(zval *s1 ,
                                                                                   zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncasecmp(zval *s1 ,
                                                                                    zval *s2 ,
                                                                                    zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcmp(char const   *s1 ,
                                                                          uint len1 ,
                                                                          char const   *s2 ,
                                                                          uint len2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncmp(char const   *s1 ,
                                                                           uint len1 ,
                                                                           char const   *s2 ,
                                                                           uint len2 ,
                                                                           uint length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcasecmp(char const   *s1 ,
                                                                              uint len1 ,
                                                                              char const   *s2 ,
                                                                              uint len2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncasecmp(char const   *s1 ,
                                                                               uint len1 ,
                                                                               char const   *s2 ,
                                                                               uint len2 ,
                                                                               uint length ) ;
extern void __attribute__((__visibility__("default")))  zendi_smart_strcmp(zval *result ,
                                                                           zval *s1 ,
                                                                           zval *s2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_symbol_tables(zval *result ,
                                                                                   HashTable *ht1 ,
                                                                                   HashTable *ht2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_arrays(zval *result ,
                                                                            zval *a1 ,
                                                                            zval *a2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_objects(zval *result ,
                                                                             zval *o1 ,
                                                                             zval *o2 ) ;
extern int __attribute__((__visibility__("default")))  zend_atoi(char const   *str ,
                                                                 int str_len ) ;
extern long __attribute__((__visibility__("default")))  zend_atol(char const   *str ,
                                                                  int str_len ) ;
extern void __attribute__((__visibility__("default")))  zend_locale_sprintf_double(zval *op ) ;
__inline static int ( __attribute__((__always_inline__)) fast_increment_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8178\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __asm__  ("incl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x0, (%0)\n\t"
              "movl $0x41e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8179\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8180\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = increment_function(op1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_decrement_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8185\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __asm__  ("decl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x00200000, (%0)\n\t"
              "movl $0xc1e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8186\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8187\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = decrement_function(op1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_add_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8200\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8192\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "addl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%1)\n\t"
                "fildl\t(%2)\n\t"
                "faddp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8193\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8198\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8199\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8194\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval + op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8195\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8196\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8197\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8211\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8212\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8202\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval + op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8203\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8204\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8209\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8210\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8205\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval + (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8206\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8207\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8208\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8213\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = add_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_sub_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8229\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8220\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "subl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%2)\n\t"
                "fildl\t(%1)\n\t"
                "fsubp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8221\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8226\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8227\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8222\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval - op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8223\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8224\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8225\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8243\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8239\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8240\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8230\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval - op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8231\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8232\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8237\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8238\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8233\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval - (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8234\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8235\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8236\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8241\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = sub_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mul_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long overflow ;
  long __tmpvar ;
  long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8263\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8264\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8254\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8250\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __asm__  ("imul %3,%0\n"
                  "adc $0,%1": "=r" (__tmpvar),
                  "=r" (overflow): "0" (op1->value.lval), "r" (op2->value.lval),
                  "1" (0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8251\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (overflow) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8248\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = (double )op1->value.lval * (double )op2->value.lval;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8249\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.lval = __tmpvar;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8255\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (overflow) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8252\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8253\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)1;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8256\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8261\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8262\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8257\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval * op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8258\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8259\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8260\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8277\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8278\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8274\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8275\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8265\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval * op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8266\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8267\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8272\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8273\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8268\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval * (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8269\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8270\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8271\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8276\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = mul_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_div_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  int tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  long tmp___5 ;
  long tmp___6 ;
  long tmp___7 ;
  long tmp___8 ;
  long tmp___9 ;
  long tmp___10 ;
  long tmp___11 ;
  int __attribute__((__visibility__("default")))  tmp___12 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___11 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___11) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8285\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L: /* CIL Label */ 
    tmp___10 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8286\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8283\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8284\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___12 = div_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___12);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mod_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8310\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8311\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8307\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8308\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8301\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )(op2->value.lval == 0L), 0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8302\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8291\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 1L, "Division by zero");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8292\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.lval = 0L;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8293\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)3;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8294\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (-1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8299\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __builtin_expect((long )(op2->value.lval == -1L), 0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8300\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8295\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.lval = 0L;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8296\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8297\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8298\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8303\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->value.lval = op1->value.lval % op2->value.lval;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8304\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8305\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8306\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8309\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = mod_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___3);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_equal_function)(zval *result ,
                                                                              zval *op1 ,
                                                                              zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8331\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8319\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8320\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8314\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval == op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8317\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8318\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8315\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval == op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8316\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8329\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8330\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8326\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8327\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8321\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval == op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8324\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8325\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8322\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval == (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8323\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8328\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval == 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_not_equal_function)(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8340\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8341\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8335\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval != op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8338\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8339\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8336\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval != op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8337\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8350\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8351\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8347\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8348\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8342\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval != op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8345\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8346\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8343\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval != (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8344\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8349\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval != 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_function)(zval *result ,
                                                                                   zval *op1 ,
                                                                                   zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8373\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8374\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8361\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8362\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8356\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval < op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8359\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8360\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8357\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval < op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8358\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8371\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8372\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8368\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8369\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8363\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval < op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8366\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8367\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8364\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval < (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8365\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8370\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8375\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8376\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval < 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_or_equal_function)(zval *result ,
                                                                                            zval *op1 ,
                                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8382\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8383\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8377\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval <= op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8380\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8381\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8378\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval <= op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8379\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8392\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8393\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8389\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8390\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8384\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval <= op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8387\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8388\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8385\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval <= (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8386\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8391\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval <= 0L);
}
}
extern void __attribute__((__visibility__("default")))  _zval_dtor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_dtor)(zval *zvalue ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8398\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8399\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor_func(zvalue);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  _zval_copy_ctor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_copy_ctor)(zval *zvalue ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8403\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8404\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_copy_ctor_func(zvalue);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zval_copy_static_var(zval **p ,
                                                                            int num_args ,
                                                                            va_list args ,
                                                                            zend_hash_key *key ) ;
extern int __attribute__((__visibility__("default")))  zend_print_variable(zval *var ) ;
extern void __attribute__((__visibility__("default")))  _zval_ptr_dtor(zval **zval_ptr ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_dtor(zval *zvalue ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_ptr_dtor(zval **zvalue ) ;
extern void __attribute__((__visibility__("default")))  _zval_dtor_wrapper(zval *zvalue ) ;
extern void __attribute__((__visibility__("default")))  zval_add_ref(zval **p ) ;
extern void __attribute__((__visibility__("default")))  zend_save_error_handling(zend_error_handling *current ) ;
extern void __attribute__((__visibility__("default")))  zend_replace_error_handling(zend_error_handling_t error_handling ,
                                                                                    zend_class_entry *exception_class ,
                                                                                    zend_error_handling *current ) ;
extern void __attribute__((__visibility__("default")))  zend_restore_error_handling(zend_error_handling *saved ) ;
extern opcode_handler_t __attribute__((__visibility__("default")))  *zend_opcode_handlers ;
extern char *zend_visibility_string(zend_uint fn_flags ) ;
extern  __attribute__((__nothrow__)) int setjmp(struct __jmp_buf_tag *__env ) ;
extern  __attribute__((__nothrow__)) int __sigsetjmp(struct __jmp_buf_tag *__env ,
                                                     int __savemask ) ;
extern  __attribute__((__nothrow__)) int _setjmp(struct __jmp_buf_tag *__env ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void _longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void siglongjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern struct _zend_compiler_globals  __attribute__((__visibility__("default"))) compiler_globals ;
extern int zendparse(void) ;
extern zend_executor_globals __attribute__((__visibility__("default")))  executor_globals ;
zend_php_scanner_globals __attribute__((__visibility__("default")))  language_scanner_globals  ;
extern zend_ini_scanner_globals __attribute__((__visibility__("default")))  ini_scanner_globals ;
extern int __attribute__((__visibility__("default")))  zend_stack_init(zend_stack *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_push(zend_stack *stack ,
                                                                       void const   *element ,
                                                                       int size ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_top(zend_stack const   *stack ,
                                                                      void **element ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_del_top(zend_stack *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_int_top(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_is_empty(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_destroy(zend_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  **zend_stack_base(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_count(zend_stack const   *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_stack_apply(zend_stack *stack ,
                                                                         int type ,
                                                                         int (*apply_function)(void *element ) ) ;
extern void __attribute__((__visibility__("default")))  zend_stack_apply_with_argument(zend_stack *stack ,
                                                                                       int type ,
                                                                                       int (*apply_function)(void *element ,
                                                                                                             void *arg ) ,
                                                                                       void *arg ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init_ex(zend_ptr_stack *stack ,
                                                                               zend_bool persistent ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_push(zend_ptr_stack *stack ,
                                                                              int count 
                                                                              , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_pop(zend_ptr_stack *stack ,
                                                                             int count 
                                                                             , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_destroy(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_apply(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_clean(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ,
                                                                             zend_bool free_elements ) ;
extern int __attribute__((__visibility__("default")))  zend_ptr_stack_num_elements(zend_ptr_stack *stack ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ,
                                                                                 void *c ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;
  void **tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 3 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8415\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8409\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8410\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 3 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8408\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8416\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8411\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8412\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8413\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8414\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8417\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8418\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8419\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top += 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8422\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8423\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___3 = b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___4 = c;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 2 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8439\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8433\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8434\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 2 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8432\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8440\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8435\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8436\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8437\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8438\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8441\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8442\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8443\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top += 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___3 = b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ,
                                                                                void **c ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *c = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top -= 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top -= 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_push)(zend_ptr_stack *stack ,
                                                                               void *ptr ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 1 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8474\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8468\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8469\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 1 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8467\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8475\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8470\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8471\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8472\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8473\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8478\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void *( __attribute__((__always_inline__)) zend_ptr_stack_pop)(zend_ptr_stack *stack ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (*(stack->top_element));
}
}
extern void __attribute__((__visibility__("default")))  zend_object_std_init(zend_object *object ,
                                                                             zend_class_entry *ce ) ;
extern void __attribute__((__visibility__("default")))  zend_object_std_dtor(zend_object *object ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_new(zend_object **object ,
                                                                                      zend_class_entry *class_type ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_destroy_object(zend_object *object ,
                                                                                    zend_object_handle handle ) ;
extern zend_object __attribute__((__visibility__("default")))  *zend_objects_get_address(zval const   *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_clone_members(zend_object *new_object ,
                                                                                   zend_object_value new_obj_val ,
                                                                                   zend_object *old_object ,
                                                                                   zend_object_handle handle ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_clone_obj(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_free_object_storage(zend_object *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_init(zend_objects_store *objects ,
                                                                                zend_uint init_size ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_call_destructors(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_mark_destructed(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_destroy(zend_objects_store *objects ) ;
extern zend_object_handle __attribute__((__visibility__("default")))  zend_objects_store_put(void *object ,
                                                                                             void (*dtor)(void *object ,
                                                                                                          zend_object_handle handle ) ,
                                                                                             void (*storage)(void *object ) ,
                                                                                             void (*clone)(void *object ,
                                                                                                           void **object_clone ) ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref_by_handle(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref_by_handle_ex(zend_object_handle handle ,
                                                                                                zend_object_handlers const   *handlers ) ;
__inline static void ( __attribute__((__always_inline__)) zend_objects_store_del_ref_by_handle)(zend_object_handle handle ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_objects_store_del_ref_by_handle_ex(handle,
                                          (zend_object_handlers const   *)((void *)0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern zend_uint __attribute__((__visibility__("default")))  zend_objects_store_get_refcount(zval *object ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_store_clone_obj(zval *object ) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object(zval const   *object ) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object_by_handle(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_set_object(zval *zobject ,
                                                                                     void *object ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_ctor_failed(zval *zobject ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_free_object_storage(zend_objects_store *objects ) ;
extern zval __attribute__((__visibility__("default")))  *zend_object_create_proxy(zval *object ,
                                                                                  zval *member ) ;
extern zend_object_handlers __attribute__((__visibility__("default")))  *zend_get_std_object_handlers(void) ;
extern HashTable __attribute__((__visibility__("default")))  module_registry ;
extern void module_destructor(zend_module_entry *module ) ;
extern int module_registry_cleanup(zend_module_entry *module ) ;
extern int module_registry_request_startup(zend_module_entry *module ) ;
extern int module_registry_unload_temp(zend_module_entry const   *module ) ;
extern void __attribute__((__visibility__("default")))  zend_init_fpu(void) ;
extern void __attribute__((__visibility__("default")))  zend_shutdown_fpu(void) ;
extern void __attribute__((__visibility__("default")))  zend_ensure_fpu_mode(void) ;
extern fpu_control_t __fpu_control ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32be ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16be ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf8 ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_functions(zend_multibyte_functions const   *functions ) ;
extern zend_multibyte_functions const __attribute__((__visibility__("default")))  *zend_multibyte_get_functions(void) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_fetch_encoding(char const   *name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_multibyte_get_encoding_name(zend_encoding const   *encoding ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_check_lexer_compatibility(zend_encoding const   *encoding ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_detector(unsigned char const   *string ,
                                                                                                         size_t length ,
                                                                                                         zend_encoding const   **list ,
                                                                                                         size_t list_size ) ;
extern size_t __attribute__((__visibility__("default")))  zend_multibyte_encoding_converter(unsigned char **to ,
                                                                                            size_t *to_length ,
                                                                                            unsigned char const   *from ,
                                                                                            size_t from_length ,
                                                                                            zend_encoding const   *encoding_to ,
                                                                                            zend_encoding const   *encoding_from ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_parse_encoding_list(char const   *encoding_list ,
                                                                                          size_t encoding_list_len ,
                                                                                          zend_encoding const   ***return_list ,
                                                                                          size_t *return_size ,
                                                                                          int persistent ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_internal_encoding(void) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_script_encoding(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding(zend_encoding const   **encoding_list ,
                                                                                          size_t encoding_list_size ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_internal_encoding(zend_encoding const   *encoding ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding_by_string(char const   *new_value ,
                                                                                                    size_t new_value_length ) ;
extern void init_compiler(void) ;
extern void shutdown_compiler(void) ;
extern void zend_init_compiler_data_structures(void) ;
extern void zend_init_compiler_context(void) ;
extern zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_file)(zend_file_handle *file_handle ,
                                                                                       int type ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_string)(zval *source_string ,
                                                                                         char *filename ) ;
int __attribute__((__visibility__("default")))  lex_scan(zval *zendlval ) ;
void startup_scanner(void) ;
void shutdown_scanner(void) ;
extern char __attribute__((__visibility__("default")))  *zend_set_compiled_filename(char const   *new_compiled_filename ) ;
extern void __attribute__((__visibility__("default")))  zend_restore_compiled_filename(char *original_compiled_filename ) ;
extern char __attribute__((__visibility__("default")))  *zend_get_compiled_filename(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_compiled_lineno(void) ;
size_t __attribute__((__visibility__("default")))  zend_get_scanned_file_offset(void) ;
extern void zend_resolve_non_class_name(znode *element_name ,
                                        zend_bool check_namespace ) ;
extern void zend_resolve_class_name(znode *class_name , ulong fetch_type ,
                                    int check_ns_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_compiled_variable_name(zend_op_array const   *op_array ,
                                                                                               zend_uint var ,
                                                                                               int *name_len ) ;
extern void zend_do_binary_op(zend_uchar op , znode *result ,
                              znode const   *op1 , znode const   *op2 ) ;
extern void zend_do_unary_op(zend_uchar op , znode *result , znode const   *op1 ) ;
extern void zend_do_binary_assign_op(zend_uchar op , znode *result ,
                                     znode const   *op1 , znode const   *op2 ) ;
extern void zend_do_assign(znode *result , znode *variable , znode *value ) ;
extern void zend_do_assign_ref(znode *result , znode const   *lvar ,
                               znode const   *rvar ) ;
extern void fetch_simple_variable(znode *result , znode *varname , int bp ) ;
extern void fetch_simple_variable_ex(znode *result , znode *varname , int bp ,
                                     zend_uchar op ) ;
extern void zend_do_indirect_references(znode *result ,
                                        znode const   *num_references ,
                                        znode *variable ) ;
extern void zend_do_fetch_static_variable(znode *varname ,
                                          znode const   *static_assignment ,
                                          int fetch_type ) ;
extern void zend_do_fetch_global_variable(znode *varname ,
                                          znode const   *static_assignment ,
                                          int fetch_type ) ;
extern void fetch_array_begin(znode *result , znode *varname , znode *first_dim ) ;
extern void fetch_array_dim(znode *result , znode const   *parent ,
                            znode const   *dim ) ;
extern void fetch_string_offset(znode *result , znode const   *parent ,
                                znode const   *offset ) ;
extern void zend_do_fetch_static_member(znode *result , znode *class_znode ) ;
extern void zend_do_print(znode *result , znode const   *arg ) ;
extern void zend_do_echo(znode const   *arg ) ;
extern unary_op_type __attribute__((__visibility__("default")))  get_unary_op(int opcode ) ;
extern binary_op_type __attribute__((__visibility__("default")))  get_binary_op(int opcode ) ;
extern void zend_do_while_cond(znode const   *expr , znode *close_bracket_token ) ;
extern void zend_do_while_end(znode const   *while_token ,
                              znode const   *close_bracket_token ) ;
extern void zend_do_do_while_begin(void) ;
extern void zend_do_do_while_end(znode const   *do_token ,
                                 znode const   *expr_open_bracket ,
                                 znode const   *expr ) ;
extern void zend_do_if_cond(znode const   *cond , znode *closing_bracket_token ) ;
extern void zend_do_if_after_statement(znode const   *closing_bracket_token ,
                                       unsigned char initialize ) ;
extern void zend_do_if_end(void) ;
extern void zend_do_for_cond(znode const   *expr ,
                             znode *second_semicolon_token ) ;
extern void zend_do_for_before_statement(znode const   *cond_start ,
                                         znode const   *second_semicolon_token ) ;
extern void zend_do_for_end(znode const   *second_semicolon_token ) ;
extern void zend_do_pre_incdec(znode *result , znode const   *op1 ,
                               zend_uchar op ) ;
extern void zend_do_post_incdec(znode *result , znode const   *op1 ,
                                zend_uchar op ) ;
extern void zend_do_begin_variable_parse(void) ;
extern void zend_do_end_variable_parse(znode *variable , int type ,
                                       int arg_offset ) ;
extern void zend_check_writable_variable(znode const   *variable ) ;
extern void zend_do_free(znode *op1 ) ;
extern void zend_do_add_string(znode *result , znode const   *op1 , znode *op2 ) ;
extern void zend_do_add_variable(znode *result , znode const   *op1 ,
                                 znode const   *op2 ) ;
extern int zend_do_verify_access_types(znode const   *current_access_type ,
                                       znode const   *new_modifier ) ;
extern void zend_do_begin_function_declaration(znode *function_token ,
                                               znode *function_name ,
                                               int is_method ,
                                               int return_reference ,
                                               znode *fn_flags_znode ) ;
extern void zend_do_end_function_declaration(znode const   *function_token ) ;
extern void zend_do_receive_arg(zend_uchar op , znode *varname ,
                                znode const   *offset ,
                                znode const   *initialization ,
                                znode *class_type , zend_bool pass_by_reference ) ;
extern int zend_do_begin_function_call(znode *function_name ,
                                       zend_bool check_namespace ) ;
extern void zend_do_begin_method_call(znode *left_bracket ) ;
extern void zend_do_clone(znode *result , znode const   *expr ) ;
extern void zend_do_begin_dynamic_function_call(znode *function_name ,
                                                int prefix_len ) ;
extern void zend_do_fetch_class(znode *result , znode *class_name ) ;
extern void zend_do_build_full_name(znode *result , znode *prefix ,
                                    znode *name , int is_class_member ) ;
extern int zend_do_begin_class_member_function_call(znode *class_name ,
                                                    znode *method_name ) ;
extern void zend_do_end_function_call(znode *function_name , znode *result ,
                                      znode const   *argument_list ,
                                      int is_method , int is_dynamic_fcall ) ;
extern void zend_do_return(znode *expr , int do_end_vparse ) ;
extern void zend_do_handle_exception(void) ;
extern void zend_do_begin_lambda_function_declaration(znode *result ,
                                                      znode *function_token ,
                                                      int return_reference ,
                                                      int is_static ) ;
extern void zend_do_fetch_lexical_variable(znode *varname , zend_bool is_ref ) ;
extern void zend_do_try(znode *try_token ) ;
extern void zend_do_begin_catch(znode *try_token , znode *catch_class ,
                                znode *catch_var , znode *first_catch ) ;
extern void zend_do_end_catch(znode const   *try_token ) ;
extern void zend_do_throw(znode const   *expr ) ;
extern int __attribute__((__visibility__("default")))  do_bind_function(zend_op_array const   *op_array ,
                                                                        zend_op *opline ,
                                                                        HashTable *function_table ,
                                                                        zend_bool compile_time ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *do_bind_class(zend_op_array const   *op_array ,
                                                                                   zend_op const   *opline ,
                                                                                   HashTable *class_table ,
                                                                                   zend_bool compile_time ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *do_bind_inherited_class(zend_op_array const   *op_array ,
                                                                                             zend_op const   *opline ,
                                                                                             HashTable *class_table ,
                                                                                             zend_class_entry *parent_ce ,
                                                                                             zend_bool compile_time ) ;
extern void __attribute__((__visibility__("default")))  zend_do_inherit_interfaces(zend_class_entry *ce ,
                                                                                   zend_class_entry const   *iface ) ;
extern void __attribute__((__visibility__("default")))  zend_do_implement_interface(zend_class_entry *ce ,
                                                                                    zend_class_entry *iface ) ;
extern void zend_do_implements_interface(znode *interface_znode ) ;
extern void zend_add_trait_precedence(znode *precedence_znode ) ;
extern void zend_add_trait_alias(znode *alias_znode ) ;
extern void zend_do_implements_trait(znode *interface_znode ) ;
extern void __attribute__((__visibility__("default")))  zend_do_implement_trait(zend_class_entry *ce ,
                                                                                zend_class_entry *trait ) ;
extern void __attribute__((__visibility__("default")))  zend_do_bind_traits(zend_class_entry *ce ) ;
extern void zend_prepare_trait_precedence(znode *result ,
                                          znode *method_reference ,
                                          znode *trait_list ) ;
extern void zend_prepare_reference(znode *result , znode *class_name ,
                                   znode *method_name ) ;
extern void zend_prepare_trait_alias(znode *result , znode *method_reference ,
                                     znode *modifiers , znode *alias ) ;
extern void init_trait_alias_list(znode *result , znode const   *trait_alias ) ;
extern void add_trait_alias(znode *result , znode const   *trait_alias ) ;
extern void init_trait_alias(znode *result , znode const   *method_name ,
                             znode const   *alias , znode const   *modifiers ) ;
extern void __attribute__((__visibility__("default")))  zend_do_inheritance(zend_class_entry *ce ,
                                                                            zend_class_entry *parent_ce ) ;
extern void zend_do_early_binding(void) ;
extern void __attribute__((__visibility__("default")))  zend_do_delayed_early_binding(zend_op_array const   *op_array ) ;
extern void zend_do_pass_param(znode *param , zend_uchar op , int offset ) ;
extern void zend_do_boolean_or_begin(znode *expr1 , znode *op_token ) ;
extern void zend_do_boolean_or_end(znode *result , znode const   *expr1 ,
                                   znode const   *expr2 , znode *op_token ) ;
extern void zend_do_boolean_and_begin(znode *expr1 , znode *op_token ) ;
extern void zend_do_boolean_and_end(znode *result , znode const   *expr1 ,
                                    znode const   *expr2 ,
                                    znode const   *op_token ) ;
extern void zend_do_brk_cont(zend_uchar op , znode const   *expr ) ;
extern void zend_do_switch_cond(znode const   *cond ) ;
extern void zend_do_switch_end(znode const   *case_list ) ;
extern void zend_do_case_before_statement(znode const   *case_list ,
                                          znode *case_token ,
                                          znode const   *case_expr ) ;
extern void zend_do_case_after_statement(znode *result ,
                                         znode const   *case_token ) ;
extern void zend_do_default_before_statement(znode const   *case_list ,
                                             znode *default_token ) ;
extern void zend_do_begin_class_declaration(znode const   *class_token ,
                                            znode *class_name ,
                                            znode const   *parent_class_name ) ;
extern void zend_do_end_class_declaration(znode const   *class_token ,
                                          znode const   *parent_token ) ;
extern void zend_do_declare_property(znode const   *var_name ,
                                     znode const   *value ,
                                     zend_uint access_type ) ;
extern void zend_do_declare_class_constant(znode *var_name ,
                                           znode const   *value ) ;
extern void zend_do_fetch_property(znode *result , znode *object ,
                                   znode const   *property ) ;
extern void zend_do_halt_compiler_register(void) ;
extern void zend_do_push_object(znode const   *object ) ;
extern void zend_do_pop_object(znode *object ) ;
extern void zend_do_begin_new_object(znode *new_token , znode *class_type ) ;
extern void zend_do_end_new_object(znode *result , znode const   *new_token ,
                                   znode const   *argument_list ) ;
extern void zend_do_fetch_constant(znode *result , znode *constant_container ,
                                   znode *constant_name , int mode ,
                                   zend_bool check_namespace ) ;
extern void zend_do_shell_exec(znode *result , znode const   *cmd ) ;
extern void zend_do_init_array(znode *result , znode const   *expr ,
                               znode const   *offset , zend_bool is_ref ) ;
extern void zend_do_add_array_element(znode *result , znode const   *expr ,
                                      znode const   *offset , zend_bool is_ref ) ;
extern void zend_do_add_static_array_element(znode *result , znode *offset ,
                                             znode const   *expr ) ;
extern void zend_do_list_init(void) ;
extern void zend_do_list_end(znode *result , znode *expr ) ;
extern void zend_do_add_list_element(znode const   *element ) ;
extern void zend_do_new_list_begin(void) ;
extern void zend_do_new_list_end(void) ;
extern void zend_init_list(void *result , void *item ) ;
extern void zend_add_to_list(void *result , void *item ) ;
extern void zend_do_cast(znode *result , znode const   *expr , int type ) ;
extern void zend_do_include_or_eval(int type , znode *result ,
                                    znode const   *op1 ) ;
extern void zend_do_unset(znode const   *variable ) ;
extern void zend_do_isset_or_isempty(int type , znode *result , znode *variable ) ;
extern void zend_do_instanceof(znode *result , znode const   *expr ,
                               znode const   *class_znode , int type ) ;
extern void zend_do_foreach_begin(znode *foreach_token ,
                                  znode *open_brackets_token , znode *array ,
                                  znode *as_token , int variable ) ;
extern void zend_do_foreach_cont(znode *foreach_token ,
                                 znode const   *open_brackets_token ,
                                 znode const   *as_token , znode *value ,
                                 znode *key ) ;
extern void zend_do_foreach_end(znode const   *foreach_token ,
                                znode const   *as_token ) ;
extern void zend_do_declare_begin(void) ;
extern void zend_do_declare_stmt(znode *var , znode *val ) ;
extern void zend_do_declare_end(znode const   *declare_token ) ;
extern void zend_do_exit(znode *result , znode const   *message ) ;
extern void zend_do_begin_silence(znode *strudel_token ) ;
extern void zend_do_end_silence(znode const   *strudel_token ) ;
extern void zend_do_jmp_set(znode const   *value , znode *jmp_token ,
                            znode *colon_token ) ;
extern void zend_do_jmp_set_else(znode *result , znode const   *false_value ,
                                 znode const   *jmp_token ,
                                 znode const   *colon_token ) ;
extern void zend_do_begin_qm_op(znode const   *cond , znode *qm_token ) ;
extern void zend_do_qm_true(znode const   *true_value , znode *qm_token ,
                            znode *colon_token ) ;
extern void zend_do_qm_false(znode *result , znode const   *false_value ,
                             znode const   *qm_token ,
                             znode const   *colon_token ) ;
extern void zend_do_extended_info(void) ;
extern void zend_do_extended_fcall_begin(void) ;
extern void zend_do_extended_fcall_end(void) ;
extern void zend_do_ticks(void) ;
extern void zend_do_abstract_method(znode const   *function_name ,
                                    znode *modifiers , znode const   *body ) ;
extern void zend_do_declare_constant(znode *name , znode *value ) ;
extern void zend_do_build_namespace_name(znode *result , znode *prefix ,
                                         znode *name ) ;
extern void zend_do_begin_namespace(znode const   *name ,
                                    zend_bool with_brackets ) ;
extern void zend_do_end_namespace(void) ;
extern void zend_verify_namespace(void) ;
extern void zend_do_use(znode *name , znode *new_name , int is_global ) ;
extern void zend_do_end_compilation(void) ;
extern void zend_do_label(znode *label ) ;
extern void zend_do_goto(znode const   *label ) ;
extern void zend_resolve_goto_label(zend_op_array *op_array , zend_op *opline ,
                                    int pass2 ) ;
extern void zend_release_labels(void) ;
extern void __attribute__((__visibility__("default")))  function_add_ref(zend_function *function ) ;
zend_op_array __attribute__((__visibility__("default")))  *compile_file(zend_file_handle *file_handle ,
                                                                        int type ) ;
zend_op_array __attribute__((__visibility__("default")))  *compile_string(zval *source_string ,
                                                                          char *filename ) ;
zend_op_array __attribute__((__visibility__("default")))  *compile_filename(int type ,
                                                                            zval *filename ) ;
extern int __attribute__((__visibility__("default")))  zend_execute_scripts(int type ,
                                                                            zval **retval ,
                                                                            int file_count 
                                                                            , ...) ;
int __attribute__((__visibility__("default")))  open_file_for_scanning(zend_file_handle *file_handle ) ;
extern void __attribute__((__visibility__("default")))  init_op_array(zend_op_array *op_array ,
                                                                      zend_uchar type ,
                                                                      int initial_ops_size ) ;
extern void __attribute__((__visibility__("default")))  destroy_op_array(zend_op_array *op_array ) ;
void __attribute__((__visibility__("default")))  zend_destroy_file_handle(zend_file_handle *file_handle ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_class_data(zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_user_class_data(zend_class_entry **pce ) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_class_data(zend_class_entry *ce ) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_classes(void) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data_full(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_function(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  zend_function_dtor(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_class(zend_class_entry **pce ) ;
extern void zend_class_add_ref(zend_class_entry **ce ) ;
extern void __attribute__((__visibility__("default")))  zend_mangle_property_name(char **dest ,
                                                                                  int *dest_length ,
                                                                                  char const   *src1 ,
                                                                                  int src1_length ,
                                                                                  char const   *src2 ,
                                                                                  int src2_length ,
                                                                                  int internal ) ;
extern int __attribute__((__visibility__("default")))  zend_unmangle_property_name(char const   *mangled_property ,
                                                                                   int mangled_property_len ,
                                                                                   char const   **class_name ,
                                                                                   char const   **prop_name ) ;
extern zend_op *get_next_op(zend_op_array *op_array ) ;
extern void init_op(zend_op *op ) ;
extern int get_next_op_number(zend_op_array *op_array ) ;
extern int print_class(zend_class_entry *class_entry ) ;
extern void print_op_array(zend_op_array *op_array , int optimizations ) ;
extern int __attribute__((__visibility__("default")))  pass_two(zend_op_array *op_array ) ;
extern zend_brk_cont_element *get_next_brk_cont_element(zend_op_array *op_array ) ;
extern void zend_do_first_catch(znode *open_parentheses ) ;
extern void zend_initialize_try_catch_element(znode const   *try_token ) ;
extern void zend_do_mark_last_catch(znode const   *first_catch ,
                                    znode const   *last_additional_catch ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_compiling(void) ;
extern char __attribute__((__visibility__("default")))  *zend_make_compiled_string_description(char const   *name ) ;
extern void __attribute__((__visibility__("default")))  zend_initialize_class_data(zend_class_entry *ce ,
                                                                                   zend_bool nullify_handlers ) ;
extern int zend_get_class_fetch_type(char const   *class_name ,
                                     uint class_name_len ) ;
extern int __attribute__((__visibility__("default")))  zend_register_auto_global(char const   *name ,
                                                                                 uint name_len ,
                                                                                 zend_bool jit ,
                                                                                 zend_bool (*auto_global_callback)(char const   *name ,
                                                                                                                   uint name_len ) ) ;
extern void __attribute__((__visibility__("default")))  zend_activate_auto_globals(void) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global(char const   *name ,
                                                                                 uint name_len ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global_quick(char const   *name ,
                                                                                       uint name_len ,
                                                                                       ulong hashval ) ;
extern size_t __attribute__((__visibility__("default")))  zend_dirname(char *path ,
                                                                       size_t len ) ;
extern int zendlex(znode *zendlval ) ;
extern int zend_add_literal(zend_op_array *op_array , zval const   *zv ) ;
void __attribute__((__visibility__("default")))  zend_save_lexical_state(zend_lex_state *lex_state ) ;
void __attribute__((__visibility__("default")))  zend_restore_lexical_state(zend_lex_state *lex_state ) ;
int __attribute__((__visibility__("default")))  zend_prepare_string_for_scanning(zval *str ,
                                                                                 char *filename ) ;
void __attribute__((__visibility__("default")))  zend_multibyte_yyinput_again(size_t (*old_input_filter)(unsigned char **str ,
                                                                                                         size_t *str_length ,
                                                                                                         unsigned char const   *buf ,
                                                                                                         size_t length ) ,
                                                                              zend_encoding const   *old_encoding ) ;
int __attribute__((__visibility__("default")))  zend_multibyte_set_filter(zend_encoding const   *onetime_encoding ) ;
extern void __attribute__((__visibility__("default")))  zend_highlight(zend_syntax_highlighter_ini *syntax_highlighter_ini ) ;
extern void __attribute__((__visibility__("default")))  zend_strip(void) ;
int __attribute__((__visibility__("default")))  highlight_file(char *filename ,
                                                               zend_syntax_highlighter_ini *syntax_highlighter_ini___0 ) ;
int __attribute__((__visibility__("default")))  highlight_string(zval *str ,
                                                                 zend_syntax_highlighter_ini *syntax_highlighter_ini___0 ,
                                                                 char *str_name ) ;
extern void __attribute__((__visibility__("default")))  zend_html_putc(char c ) ;
extern void __attribute__((__visibility__("default")))  zend_html_puts(char const   *s ,
                                                                       uint len ) ;
extern zend_syntax_highlighter_ini syntax_highlighter_ini ;
extern void clean_module_constants(int module_number ) ;
extern void free_zend_constant(zend_constant *c ) ;
extern int zend_startup_constants(void) ;
extern int zend_shutdown_constants(void) ;
extern void zend_register_standard_constants(void) ;
extern void clean_non_persistent_constants(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant(char const   *name ,
                                                                         uint name_len ,
                                                                         zval *result ) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant_ex(char const   *name ,
                                                                            uint name_len ,
                                                                            zval *result ,
                                                                            zend_class_entry *scope ,
                                                                            ulong flags ) ;
extern void __attribute__((__visibility__("default")))  zend_register_long_constant(char const   *name ,
                                                                                    uint name_len ,
                                                                                    long lval ,
                                                                                    int flags ,
                                                                                    int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_double_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      double dval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_string_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      char *strval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_stringl_constant(char const   *name ,
                                                                                       uint name_len ,
                                                                                       char *strval ,
                                                                                       uint strlen ,
                                                                                       int flags ,
                                                                                       int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_register_constant(zend_constant *c ) ;
extern void zend_copy_constants(HashTable *target , HashTable *sourc ) ;
extern void copy_zend_constant(zend_constant *c ) ;
extern zend_constant *zend_quick_get_constant(zend_literal const   *key ,
                                              ulong flags ) ;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors(void (*ld)(void * ) ,
                                                                                      void (*pld)(void * ) ,
                                                                                      int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors_ex(void (*ld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         void (*pld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         char const   *type_name ,
                                                                                         int module_number ) ;
extern void list_entry_destructor(void *ptr ) ;
extern void plist_entry_destructor(void *ptr ) ;
extern void zend_clean_module_rsrc_dtors(int module_number ) ;
extern int zend_init_rsrc_list(void) ;
extern int zend_init_rsrc_plist(void) ;
extern void zend_destroy_rsrc_list(HashTable *ht ) ;
extern int zend_init_rsrc_list_dtors(void) ;
extern void zend_destroy_rsrc_list_dtors(void) ;
extern int __attribute__((__visibility__("default")))  zend_list_insert(void *ptr ,
                                                                        int type ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_addref(int id ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_delete(int id ) ;
extern void __attribute__((__visibility__("default")))  *_zend_list_find(int id ,
                                                                         int *type ) ;
extern int __attribute__((__visibility__("default")))  zend_register_resource(zval *rsrc_result ,
                                                                              void *rsrc_pointer ,
                                                                              int rsrc_type ) ;
extern void __attribute__((__visibility__("default")))  *zend_fetch_resource(zval **passed_id ,
                                                                             int default_id ,
                                                                             char const   *resource_type_name ,
                                                                             int *found_resource_type ,
                                                                             int num_resource_types 
                                                                             , ...) ;
extern char const __attribute__((__visibility__("default")))  *zend_rsrc_list_get_rsrc_type(int resource ) ;
extern int __attribute__((__visibility__("default")))  zend_fetch_list_dtor_id(char *type_name ) ;
extern int __attribute__((__visibility__("default")))  le_index_ptr ;
extern void __attribute__((__visibility__("default")))  (*zend_execute)(zend_op_array *op_array ) ;
extern void __attribute__((__visibility__("default")))  (*zend_execute_internal)(zend_execute_data *execute_data_ptr ,
                                                                                 int return_value_used ) ;
extern void init_executor(void) ;
extern void shutdown_executor(void) ;
extern void shutdown_destructors(void) ;
extern void __attribute__((__visibility__("default")))  execute(zend_op_array *op_array ) ;
extern void __attribute__((__visibility__("default")))  execute_internal(zend_execute_data *execute_data_ptr ,
                                                                         int return_value_used ) ;
extern int __attribute__((__visibility__("default")))  zend_is_true(zval *op ) ;
__inline static void ( __attribute__((__always_inline__)) safe_free_zval_ptr_rel)(zval *p ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )p != (unsigned int )executor_globals.uninitialized_zval_ptr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8494\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8492\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)p)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8490\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(p);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8491\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8493\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)p);
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8495\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_lookup_class(char const   *name ,
                                                                         int name_length ,
                                                                         zend_class_entry ***ce ) ;
extern int __attribute__((__visibility__("default")))  zend_lookup_class_ex(char const   *name ,
                                                                            int name_length ,
                                                                            zend_literal const   *key ,
                                                                            int use_autoload ,
                                                                            zend_class_entry ***ce ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string(char *str ,
                                                                        zval *retval_ptr ,
                                                                        char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl(char *str ,
                                                                         int str_len ,
                                                                         zval *retval_ptr ,
                                                                         char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string_ex(char *str ,
                                                                           zval *retval_ptr ,
                                                                           char *string_name ,
                                                                           int handle_exceptions ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl_ex(char *str ,
                                                                            int str_len ,
                                                                            zval *retval_ptr ,
                                                                            char *string_name ,
                                                                            int handle_exceptions ) ;
extern char __attribute__((__visibility__("default")))  *zend_verify_arg_class_kind(zend_arg_info const   *cur_arg_info ,
                                                                                    ulong fetch_type ,
                                                                                    char const   **class_name ,
                                                                                    zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_verify_arg_error(int error_type ,
                                                                             zend_function const   *zf ,
                                                                             zend_uint arg_num ,
                                                                             char const   *need_msg ,
                                                                             char const   *need_kind ,
                                                                             char const   *given_msg ,
                                                                             char const   *given_kind ) ;
__inline static void ( __attribute__((__always_inline__)) i_zval_ptr_dtor)(zval *zval_ptr ) 
{ zend_uint tmp ;
  zend_uint tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zval_delref_p(zval_ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8500\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zval_refcount_p(zval_ptr);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8501\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8498\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zval_unset_isref_p(zval_ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8499\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8502\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    gc_zval_check_possible_root(zval_ptr);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8509\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )zval_ptr != (unsigned int )(& executor_globals.uninitialized_zval)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8505\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)zval_ptr)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8503\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(zval_ptr);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8504\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8506\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(zval_ptr);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8507\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)zval_ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8508\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8512\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static int ( __attribute__((__always_inline__)) i_zend_is_true)(zval *op ) 
{ int result ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval tmp___1 ;
  int tmp___2 ;
  zval *tmp___3 ;
  zval *tmp___4 ;

  {
  switch ((int )op->type) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8542\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 0: 
  result = 0;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  case 3: 
  case 7: 
  if (op->value.lval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8513\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 2: 
  if (op->value.dval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 6: 
  if (op->value.str.len == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8522\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op->value.str.len == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8520\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(op->value.str.val + 0) == 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8518\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = 0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8519\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = 1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8521\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result = 1;
    }
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 4: 
  tmp___0 = zend_hash_num_elements((HashTable const   *)op->value.ht);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8523\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8524\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 5: 
  if ((int )op->type == 5) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8540\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(op->value.obj.handlers)->get_class_entry != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8538\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op->value.obj.handlers)->cast_object) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8527\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = (*((op->value.obj.handlers)->cast_object))(op, & tmp___1, 3);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8528\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___2 == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8525\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result = (int )tmp___1.value.lval;
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8526\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8537\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((op->value.obj.handlers)->get) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8533\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = (*((op->value.obj.handlers)->get))(op);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8534\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = tmp___4;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8535\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )tmp___3->type != 5) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8529\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            convert_to_boolean(tmp___3);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8530\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result = (int )tmp___3->value.lval;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8531\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_ptr_dtor(& tmp___3);
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8532\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8536\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8539\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result = 1;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: 
  result = 0;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result);
}
}
extern int __attribute__((__visibility__("default")))  zval_update_constant(zval **pp ,
                                                                            void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_inline_change(zval **pp ,
                                                                                          void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_no_inline_change(zval **pp ,
                                                                                             void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_ex(zval **pp ,
                                                                               void *arg ,
                                                                               zend_class_entry *scope ) ;
__inline static zend_vm_stack ( __attribute__((__always_inline__)) zend_vm_stack_new_page)(int count ) 
{ zend_vm_stack page ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc((((sizeof(*page) + 4U) - 1U) & 4294967292U) + sizeof(void *) * (unsigned int )count);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page = (struct _zend_vm_stack *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->top = (void **)((char *)page + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->end = page->top + count;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->prev = (struct _zend_vm_stack *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (page);
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_init)(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  executor_globals.argument_stack = zend_vm_stack_new_page(16368);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_destroy)(void) 
{ zend_vm_stack stack ;
  zend_vm_stack p ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack = executor_globals.argument_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8561\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )stack != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8560\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = stack->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)stack);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8564\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack = p;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_extend)(int count ) 
{ zend_vm_stack p ;
  int tmp ;
  zend_vm_stack tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (count >= 16368) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8568\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = count;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 16368;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_vm_stack_new_page(tmp);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p->prev = executor_globals.argument_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  executor_globals.argument_stack = p;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_top)(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((executor_globals.argument_stack)->top);
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push)(void *ptr ) 
{ long tmp ;
  void **tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8579\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __builtin_expect((long )(1 > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                           0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8580\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8577\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend(1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8578\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___0 = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push_nocheck)(void *ptr ) 
{ void **tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_pop)(void) 
{ void *el ;
  zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8594\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8595\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  el = *((executor_globals.argument_stack)->top);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8590\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8592\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8593\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (el);
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_alloc)(size_t size ) 
{ void *ret ;
  int extra ;
  void **old_top ;
  long tmp ;
  long tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  size = (size + (sizeof(void *) - 1U)) / sizeof(void *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (4U > sizeof(void *)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8604\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    extra = (int )((4UL - ((unsigned long )(executor_globals.argument_stack)->top & 3UL)) / (unsigned long )sizeof(void *));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8605\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __builtin_expect((long )((unsigned long )((size + (size_t )extra) + (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *)) > (unsigned long )((executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top)),
                           0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8606\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8599\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend((int )size);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8600\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      old_top = (executor_globals.argument_stack)->top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8601\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += extra;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8602\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)old_top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8603\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8611\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8609\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )((int )size > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                                 0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8610\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8607\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_vm_stack_extend((int )size);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8608\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ret = (void *)(executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (executor_globals.argument_stack)->top += size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free_int)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (executor_globals.argument_stack)->top = (void **)ptr;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (4U > sizeof(void *)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8627\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ptr = (void *)((char *)ptr - (((sizeof(void *) + 4U) - 1U) & 4294967292U));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8628\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = *((void ***)ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8629\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = (void **)ptr;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_push_args)(int count ) 
{ zend_vm_stack p ;
  void *data ;
  zend_vm_stack r ;
  long tmp ;
  int tmp___0 ;
  void **tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  void **tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((executor_globals.argument_stack)->top - (void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) < count),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___2) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8657\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )(executor_globals.argument_stack)->end),
                               0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8658\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8648\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      p = executor_globals.argument_stack;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8649\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend(count + 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8650\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8651\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8652\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8640\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = count;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8641\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        count --;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8642\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0 > 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8634\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8643\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (p->top) --;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8644\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        data = *(p->top);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8645\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __builtin_expect((long )((unsigned int )p->top == (unsigned int )((void **)((char *)p + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                               0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8646\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8635\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          r = p;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8636\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (executor_globals.argument_stack)->prev = p->prev;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8637\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          p = p->prev;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8638\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)r);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8639\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8647\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) + count) = data;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8653\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (executor_globals.argument_stack)->top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8654\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((executor_globals.argument_stack)->top) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8655\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (tmp___1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8656\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8664\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___4);
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_clear_multiple)(void) 
{ void **p ;
  int delete_count ;
  zval *q ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = (executor_globals.argument_stack)->top - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  delete_count = (int )((unsigned long )*p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8674\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8666\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    delete_count --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8667\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (delete_count >= 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8665\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8668\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    q = *((zval **)p);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8670\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *p = (void *)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8671\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i_zval_ptr_dtor(q);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8675\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_vm_stack_free_int((void *)p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8676\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static zval **( __attribute__((__always_inline__)) zend_vm_stack_get_arg)(int requested_arg ) 
{ void **p ;
  int arg_count ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8679\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = ((executor_globals.current_execute_data)->prev_execute_data)->function_state.arguments;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arg_count = (int )((unsigned long )*p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )(requested_arg > arg_count), 0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8677\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zval **)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8678\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((((zval **)p - arg_count) + requested_arg) - 1);
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_ptr_stack_2_pop(stack, & a, & b);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *object = (zval *)a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *fbc = (zend_function *)b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                      zend_class_entry **called_scope ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;
  void *c ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_ptr_stack_3_pop(stack, & a, & b, & c);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *called_scope = (zend_class_entry *)a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *object = (zval *)b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *fbc = (zend_function *)c;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void execute_new_code(void) ;
extern char const __attribute__((__visibility__("default")))  *get_active_class_name(char const   **space ) ;
extern char const __attribute__((__visibility__("default")))  *get_active_function_name(void) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_executed_filename(void) ;
extern uint __attribute__((__visibility__("default")))  zend_get_executed_lineno(void) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_executing(void) ;
extern void __attribute__((__visibility__("default")))  zend_set_timeout(long seconds ,
                                                                         int reset_signals ) ;
extern void __attribute__((__visibility__("default")))  zend_unset_timeout(void) ;
extern void __attribute__((__visibility__("default")))  zend_timeout(int dummy ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class(char const   *class_name ,
                                                                                      uint class_name_len ,
                                                                                      int fetch_type ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class_by_name(char const   *class_name ,
                                                                                              uint class_name_len ,
                                                                                              zend_literal const   *key ,
                                                                                              int fetch_type ) ;
extern void zend_verify_abstract_class(zend_class_entry *ce ) ;
extern zval __attribute__((__visibility__("default")))  **zend_get_compiled_variable_value(zend_execute_data const   *execute_data_ptr ,
                                                                                           zend_uint var ) ;
extern int __attribute__((__visibility__("default")))  zend_set_user_opcode_handler(zend_uchar opcode ,
                                                                                    int (*handler)(zend_execute_data *execute_data ) ) ;
extern user_opcode_handler_t __attribute__((__visibility__("default")))  zend_get_user_opcode_handler(zend_uchar opcode ) ;
extern zval __attribute__((__visibility__("default")))  *zend_get_zval_ptr(int op_type ,
                                                                           znode_op const   *node ,
                                                                           temp_variable const   *Ts ,
                                                                           zend_free_op *should_free ,
                                                                           int type ) ;
extern zval __attribute__((__visibility__("default")))  **zend_get_zval_ptr_ptr(int op_type ,
                                                                                znode_op const   *node ,
                                                                                temp_variable const   *Ts ,
                                                                                zend_free_op *should_free ,
                                                                                int type ) ;
extern int __attribute__((__visibility__("default")))  zend_do_fcall(zend_execute_data *execute_data ) ;
extern int zend_next_free_module(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_parameters(int ht ,
                                                                           int param_count 
                                                                           , ...) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array(int ht ,
                                                                                  int param_count ,
                                                                                  zval **argument_array ) ;
extern int __attribute__((__visibility__("default"),
__deprecated__))  zend_get_parameters_ex(int param_count  , ...) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array_ex(int param_count ,
                                                                                     zval ***argument_array ) ;
extern int __attribute__((__visibility__("default")))  zend_copy_parameters_array(int param_count ,
                                                                                  zval *argument_array ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters(int num_args ,
                                                                             char const   *type_spec 
                                                                             , ...) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters_ex(int flags ,
                                                                                int num_args ,
                                                                                char const   *type_spec 
                                                                                , ...) ;
extern char __attribute__((__visibility__("default")))  *zend_zval_type_name(zval const   *arg ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters(int num_args ,
                                                                                    zval *this_ptr ,
                                                                                    char const   *type_spec 
                                                                                    , ...) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters_ex(int flags ,
                                                                                       int num_args ,
                                                                                       zval *this_ptr ,
                                                                                       char const   *type_spec 
                                                                                       , ...) ;
extern int __attribute__((__visibility__("default")))  zend_register_functions(zend_class_entry *scope ,
                                                                               zend_function_entry const   *functions ,
                                                                               HashTable *function_table ,
                                                                               int type ) ;
extern void __attribute__((__visibility__("default")))  zend_unregister_functions(zend_function_entry const   *functions ,
                                                                                  int count ,
                                                                                  HashTable *function_table ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_module(zend_module_entry *module_entry ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_internal_module(zend_module_entry *module_entry ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_module_ex(zend_module_entry *module ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_module_ex(zend_module_entry *module ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_modules(void) ;
extern void __attribute__((__visibility__("default")))  zend_collect_module_handlers(void) ;
extern void __attribute__((__visibility__("default")))  zend_destroy_modules(void) ;
extern void __attribute__((__visibility__("default")))  zend_check_magic_method_implementation(zend_class_entry const   *ce ,
                                                                                               zend_function const   *fptr ,
                                                                                               int error_type ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class(zend_class_entry *class_entry ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class_ex(zend_class_entry *class_entry ,
                                                                                                     zend_class_entry *parent_ce ,
                                                                                                     char *parent_name ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_interface(zend_class_entry *orig_class_entry ) ;
extern void __attribute__((__visibility__("default")))  zend_class_implements(zend_class_entry *class_entry ,
                                                                              int num_interfaces 
                                                                              , ...) ;
extern int __attribute__((__visibility__("default")))  zend_register_class_alias_ex(char const   *name ,
                                                                                    int name_len ,
                                                                                    zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  zend_disable_function(char *function_name ,
                                                                             uint function_name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_disable_class(char *class_name ,
                                                                          uint class_name_length ) ;
extern void __attribute__((__visibility__("default")))  zend_wrong_param_count(void) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable_ex(zval *callable ,
                                                                                 zval *object_ptr ,
                                                                                 uint check_flags ,
                                                                                 char **callable_name ,
                                                                                 int *callable_name_len ,
                                                                                 zend_fcall_info_cache *fcc ,
                                                                                 char **error ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable(zval *callable ,
                                                                              uint check_flags ,
                                                                              char **callable_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_make_callable(zval *callable ,
                                                                                char **callable_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_module_version(char const   *module_name ) ;
extern int __attribute__((__visibility__("default")))  zend_get_module_started(char const   *module_name ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property(zend_class_entry *ce ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *property ,
                                                                             int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_ex(zend_class_entry *ce ,
                                                                                char const   *name ,
                                                                                int name_length ,
                                                                                zval *property ,
                                                                                int access_type ,
                                                                                char const   *doc_comment ,
                                                                                int doc_comment_len ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_null(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_bool(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_long(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_double(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_string(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_stringl(zend_class_entry *ce ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_len ,
                                                                                     int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant(zend_class_entry *ce ,
                                                                                   char const   *name ,
                                                                                   size_t name_length ,
                                                                                   zval *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_null(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_long(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_bool(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        zend_bool value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_double(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_stringl(zend_class_entry *ce ,
                                                                                           char const   *name ,
                                                                                           size_t name_length ,
                                                                                           char const   *value ,
                                                                                           size_t value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_string(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          char const   *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_class_constants(zend_class_entry *class_type ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property(zend_class_entry *scope ,
                                                                             zval *object ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_null(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_bool(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_long(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_double(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_string(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_stringl(zend_class_entry *scope ,
                                                                                     zval *object ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zval *value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_null(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_bool(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_long(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_double(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_string(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          char const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_stringl(zend_class_entry *scope ,
                                                                                           char const   *name ,
                                                                                           int name_length ,
                                                                                           char const   *value ,
                                                                                           int value_length ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_property(zend_class_entry *scope ,
                                                                            zval *object ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool silent ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zend_bool silent ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_class_entry(zval const   *zobject ) ;
extern int __attribute__((__visibility__("default")))  zend_get_object_classname(zval const   *object ,
                                                                                 char const   **class_name ,
                                                                                 zend_uint *class_name_len ) ;
extern char __attribute__((__visibility__("default")))  *zend_get_type_by_const(int type ) ;
extern int __attribute__((__visibility__("default")))  _array_init(zval *arg ,
                                                                   uint size ) ;
extern int __attribute__((__visibility__("default")))  _object_init(zval *arg ) ;
extern int __attribute__((__visibility__("default")))  _object_init_ex(zval *arg ,
                                                                       zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  _object_and_properties_init(zval *arg ,
                                                                                   zend_class_entry *ce ,
                                                                                   HashTable *properties ) ;
extern void __attribute__((__visibility__("default")))  object_properties_init(zend_object *object ,
                                                                               zend_class_entry *class_type ) ;
extern void __attribute__((__visibility__("default")))  zend_merge_properties(zval *obj ,
                                                                              HashTable *properties ,
                                                                              int destroy_ht ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_function(zval *arg ,
                                                                          char const   *key ,
                                                                          void (*function_ptr)(int ht ,
                                                                                               zval *return_value ,
                                                                                               zval **return_value_ptr ,
                                                                                               zval *this_ptr ,
                                                                                               int return_value_used ) ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_long_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         long n ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_null_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_bool_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         int b ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_resource_ex(zval *arg ,
                                                                             char const   *key ,
                                                                             uint key_len ,
                                                                             int r ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_double_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           double d ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_string_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           char *str ,
                                                                           int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_stringl_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            char *str ,
                                                                            uint length ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_zval_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_index_long(zval *arg ,
                                                                      ulong idx ,
                                                                      long n ) ;
extern int __attribute__((__visibility__("default")))  add_index_null(zval *arg ,
                                                                      ulong idx ) ;
extern int __attribute__((__visibility__("default")))  add_index_bool(zval *arg ,
                                                                      ulong idx ,
                                                                      int b ) ;
extern int __attribute__((__visibility__("default")))  add_index_resource(zval *arg ,
                                                                          ulong idx ,
                                                                          int r ) ;
extern int __attribute__((__visibility__("default")))  add_index_double(zval *arg ,
                                                                        ulong idx ,
                                                                        double d ) ;
extern int __attribute__((__visibility__("default")))  add_index_string(zval *arg ,
                                                                        ulong idx ,
                                                                        char const   *str ,
                                                                        int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_stringl(zval *arg ,
                                                                         ulong idx ,
                                                                         char const   *str ,
                                                                         uint length ,
                                                                         int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_zval(zval *arg ,
                                                                      ulong index ,
                                                                      zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_long(zval *arg ,
                                                                           long n ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_null(zval *arg ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_bool(zval *arg ,
                                                                           int b ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_resource(zval *arg ,
                                                                               int r ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_double(zval *arg ,
                                                                             double d ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_string(zval *arg ,
                                                                             char const   *str ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_stringl(zval *arg ,
                                                                              char const   *str ,
                                                                              uint length ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_zval(zval *arg ,
                                                                           zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_string_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               void **dest ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_stringl_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                char const   *str ,
                                                                                uint length ,
                                                                                void **dest ,
                                                                                int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_long(zval *arg ,
                                                                          ulong idx ,
                                                                          long l ,
                                                                          void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_double(zval *arg ,
                                                                            ulong idx ,
                                                                            double d ,
                                                                            void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_string(zval *arg ,
                                                                            ulong idx ,
                                                                            char const   *str ,
                                                                            void **dest ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_stringl(zval *arg ,
                                                                             ulong idx ,
                                                                             char const   *str ,
                                                                             uint length ,
                                                                             void **dest ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_long_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            long l ) ;
extern int __attribute__((__visibility__("default")))  add_property_null_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_property_bool_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            int b ) ;
extern int __attribute__((__visibility__("default")))  add_property_resource_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                long r ) ;
extern int __attribute__((__visibility__("default")))  add_property_double_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              double d ) ;
extern int __attribute__((__visibility__("default")))  add_property_string_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              char const   *str ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_stringl_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               uint length ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_zval_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            zval *value ) ;
extern int __attribute__((__visibility__("default")))  call_user_function(HashTable *function_table ,
                                                                          zval **object_pp ,
                                                                          zval *function_name ,
                                                                          zval *retval_ptr ,
                                                                          zend_uint param_count ,
                                                                          zval **params ) ;
extern int __attribute__((__visibility__("default")))  call_user_function_ex(HashTable *function_table ,
                                                                             zval **object_pp ,
                                                                             zval *function_name ,
                                                                             zval **retval_ptr_ptr ,
                                                                             zend_uint param_count ,
                                                                             zval ***params ,
                                                                             int no_separation ,
                                                                             HashTable *symbol_table ) ;
extern zend_fcall_info const __attribute__((__visibility__("default")))  empty_fcall_info ;
extern zend_fcall_info_cache const __attribute__((__visibility__("default")))  empty_fcall_info_cache ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_init(zval *callable ,
                                                                            uint check_flags ,
                                                                            zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            char **callable_name ,
                                                                            char **error ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_clear(zend_fcall_info *fci ,
                                                                                   int free_mem ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_save(zend_fcall_info *fci ,
                                                                                  int *param_count ,
                                                                                  zval ****params ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_restore(zend_fcall_info *fci ,
                                                                                     int param_count ,
                                                                                     zval ***params ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_args(zend_fcall_info *fci ,
                                                                            zval *args ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argp(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            zval ***argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argv(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            va_list *argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argn(zend_fcall_info *fci ,
                                                                            int argc 
                                                                            , ...) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_call(zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            zval **retval ,
                                                                            zval *args ) ;
extern int __attribute__((__visibility__("default")))  zend_call_function(zend_fcall_info *fci ,
                                                                          zend_fcall_info_cache *fci_cache ) ;
extern int __attribute__((__visibility__("default")))  zend_set_hash_symbol(zval *symbol ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool is_ref ,
                                                                            int num_symbol_tables 
                                                                            , ...) ;
extern void __attribute__((__visibility__("default")))  zend_delete_variable(zend_execute_data *ex ,
                                                                             HashTable *ht ,
                                                                             char const   *name ,
                                                                             int name_len ,
                                                                             ulong hash_value ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable(char const   *name ,
                                                                                   int name_len ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable_ex(char const   *name ,
                                                                                      int name_len ,
                                                                                      ulong hash_value ) ;
extern void __attribute__((__visibility__("default")))  zend_reset_all_cv(HashTable *symbol_table ) ;
extern void __attribute__((__visibility__("default")))  zend_rebuild_symbol_table(void) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_function(int ht ,
                                                                                      zval *return_value ,
                                                                                      zval **return_value_ptr ,
                                                                                      zval *this_ptr ,
                                                                                      int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_class(int ht ,
                                                                                   zval *return_value ,
                                                                                   zval **return_value_ptr ,
                                                                                   zval *this_ptr ,
                                                                                   int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zend_exception_set_previous(zval *exception ,
                                                                                    zval *add_previous ) ;
extern void __attribute__((__visibility__("default")))  zend_exception_save(void) ;
extern void __attribute__((__visibility__("default")))  zend_exception_restore(void) ;
extern void zend_throw_exception_internal(zval *exception ) ;
extern void zend_register_default_exception(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_exception_get_default(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_error_exception(void) ;
extern void __attribute__((__visibility__("default")))  zend_register_default_classes(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception(zend_class_entry *exception_ce ,
                                                                              char *message ,
                                                                              long code ) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception_ex(zend_class_entry *exception_ce ,
                                                                                 long code ,
                                                                                 char *format 
                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  zend_throw_exception_object(zval *exception ) ;
extern void __attribute__((__visibility__("default")))  zend_clear_exception(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_error_exception(zend_class_entry *exception_ce ,
                                                                                    char *message ,
                                                                                    long code ,
                                                                                    int severity ) ;
extern void __attribute__((__visibility__("default")))  (*zend_throw_exception_hook)(zval *ex ) ;
extern void __attribute__((__visibility__("default")))  zend_exception_error(zval *exception ,
                                                                             int severity ) ;
extern int zend_spprintf(char **message , int max_len , char *format  , ...) ;
extern  __attribute__((__nothrow__)) int access(char const   *__name ,
                                                int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int faccessat(int __fd ,
                                                   char const   *__file ,
                                                   int __type , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) __off_t lseek(int __fd , __off_t __offset ,
                                                   int __whence ) ;
extern int close(int __fd ) ;
extern ssize_t read(int __fd , void *__buf , size_t __nbytes ) ;
extern ssize_t write(int __fd , void const   *__buf , size_t __n ) ;
extern ssize_t pread(int __fd , void *__buf , size_t __nbytes ,
                     __off_t __offset ) ;
extern ssize_t pwrite(int __fd , void const   *__buf , size_t __n ,
                      __off_t __offset ) ;
extern  __attribute__((__nothrow__)) int pipe(int *__pipedes ) ;
extern  __attribute__((__nothrow__)) unsigned int alarm(unsigned int __seconds ) ;
extern unsigned int sleep(unsigned int __seconds ) ;
extern  __attribute__((__nothrow__)) __useconds_t ualarm(__useconds_t __value ,
                                                         __useconds_t __interval ) ;
extern int usleep(__useconds_t __useconds ) ;
extern int pause(void) ;
extern  __attribute__((__nothrow__)) int chown(char const   *__file ,
                                               __uid_t __owner ,
                                               __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchown(int __fd , __uid_t __owner ,
                                                __gid_t __group ) ;
extern  __attribute__((__nothrow__)) int lchown(char const   *__file ,
                                                __uid_t __owner ,
                                                __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchownat(int __fd ,
                                                  char const   *__file ,
                                                  __uid_t __owner ,
                                                  __gid_t __group , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int chdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchdir(int __fd ) ;
extern  __attribute__((__nothrow__)) char *getcwd(char *__buf , size_t __size ) ;
extern  __attribute__((__nothrow__)) char *getwd(char *__buf )  __attribute__((__nonnull__(1),
__deprecated__)) ;
extern  __attribute__((__nothrow__)) int dup(int __fd ) ;
extern  __attribute__((__nothrow__)) int dup2(int __fd , int __fd2 ) ;
extern char **__environ ;
extern  __attribute__((__nothrow__)) int execve(char const   *__path ,
                                                char * const  *__argv ,
                                                char * const  *__envp )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int fexecve(int __fd ,
                                                 char * const  *__argv ,
                                                 char * const  *__envp )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int execv(char const   *__path ,
                                               char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execle(char const   *__path ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execl(char const   *__path ,
                                               char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execvp(char const   *__file ,
                                                char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execlp(char const   *__file ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int nice(int __inc ) ;
extern  __attribute__((__noreturn__)) void _exit(int __status ) ;
extern  __attribute__((__nothrow__)) long pathconf(char const   *__path ,
                                                   int __name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long fpathconf(int __fd , int __name ) ;
extern  __attribute__((__nothrow__)) long sysconf(int __name ) ;
extern  __attribute__((__nothrow__)) size_t confstr(int __name , char *__buf ,
                                                    size_t __len ) ;
extern  __attribute__((__nothrow__)) __pid_t getpid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getppid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getpgrp(void) ;
extern  __attribute__((__nothrow__)) __pid_t __getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) int setpgid(__pid_t __pid , __pid_t __pgid ) ;
extern  __attribute__((__nothrow__)) int setpgrp(void) ;
extern  __attribute__((__nothrow__)) __pid_t setsid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getsid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __uid_t getuid(void) ;
extern  __attribute__((__nothrow__)) __uid_t geteuid(void) ;
extern  __attribute__((__nothrow__)) __gid_t getgid(void) ;
extern  __attribute__((__nothrow__)) __gid_t getegid(void) ;
extern  __attribute__((__nothrow__)) int getgroups(int __size , __gid_t *__list ) ;
extern  __attribute__((__nothrow__)) int setuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int setreuid(__uid_t __ruid ,
                                                  __uid_t __euid ) ;
extern  __attribute__((__nothrow__)) int seteuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int setgid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int setregid(__gid_t __rgid ,
                                                  __gid_t __egid ) ;
extern  __attribute__((__nothrow__)) int setegid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) __pid_t fork(void) ;
extern  __attribute__((__nothrow__)) __pid_t vfork(void) ;
extern  __attribute__((__nothrow__)) char *ttyname(int __fd ) ;
extern  __attribute__((__nothrow__)) int ttyname_r(int __fd , char *__buf ,
                                                   size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int isatty(int __fd ) ;
extern  __attribute__((__nothrow__)) int ttyslot(void) ;
extern  __attribute__((__nothrow__)) int link(char const   *__from ,
                                              char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int linkat(int __fromfd ,
                                                char const   *__from ,
                                                int __tofd ,
                                                char const   *__to ,
                                                int __flags )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int symlink(char const   *__from ,
                                                 char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) ssize_t readlink(char const   * __restrict  __path ,
                                                      char * __restrict  __buf ,
                                                      size_t __len )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int symlinkat(char const   *__from ,
                                                   int __tofd ,
                                                   char const   *__to )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) ssize_t readlinkat(int __fd ,
                                                        char const   * __restrict  __path ,
                                                        char * __restrict  __buf ,
                                                        size_t __len )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int unlink(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int unlinkat(int __fd ,
                                                  char const   *__name ,
                                                  int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int rmdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) __pid_t tcgetpgrp(int __fd ) ;
extern  __attribute__((__nothrow__)) int tcsetpgrp(int __fd , __pid_t __pgrp_id ) ;
extern char *getlogin(void) ;
extern int getlogin_r(char *__name , size_t __name_len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int setlogin(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern char *optarg ;
extern int optind ;
extern int opterr ;
extern int optopt ;
extern  __attribute__((__nothrow__)) int getopt(int ___argc ,
                                                char * const  *___argv ,
                                                char const   *__shortopts ) ;
extern  __attribute__((__nothrow__)) int gethostname(char *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int sethostname(char const   *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int sethostid(long __id ) ;
extern  __attribute__((__nothrow__)) int getdomainname(char *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int setdomainname(char const   *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int vhangup(void) ;
extern  __attribute__((__nothrow__)) int revoke(char const   *__file )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int profil(unsigned short *__sample_buffer ,
                                                size_t __size ,
                                                size_t __offset ,
                                                unsigned int __scale )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int acct(char const   *__name ) ;
extern  __attribute__((__nothrow__)) char *getusershell(void) ;
extern  __attribute__((__nothrow__)) void endusershell(void) ;
extern  __attribute__((__nothrow__)) void setusershell(void) ;
extern  __attribute__((__nothrow__)) int daemon(int __nochdir , int __noclose ) ;
extern  __attribute__((__nothrow__)) int chroot(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern char *getpass(char const   *__prompt )  __attribute__((__nonnull__(1))) ;
extern int fsync(int __fd ) ;
extern long gethostid(void) ;
extern  __attribute__((__nothrow__)) void sync(void) ;
extern  __attribute__((__nothrow__)) int getpagesize(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int getdtablesize(void) ;
extern  __attribute__((__nothrow__)) int truncate(char const   *__file ,
                                                  __off_t __length )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int ftruncate(int __fd , __off_t __length ) ;
extern  __attribute__((__nothrow__)) int brk(void *__addr ) ;
extern  __attribute__((__nothrow__)) void *sbrk(intptr_t __delta ) ;
extern  __attribute__((__nothrow__)) long syscall(long __sysno  , ...) ;
extern int lockf(int __fd , int __cmd , __off_t __len ) ;
extern int fdatasync(int __fildes ) ;
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int chmod(char const   *__file ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int lchmod(char const   *__file ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchmod(int __fd , __mode_t __mode ) ;
extern  __attribute__((__nothrow__)) int fchmodat(int __fd ,
                                                  char const   *__file ,
                                                  __mode_t __mode , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) __mode_t umask(__mode_t __mask ) ;
extern  __attribute__((__nothrow__)) int mkdir(char const   *__path ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int mkdirat(int __fd ,
                                                 char const   *__path ,
                                                 __mode_t __mode )  __attribute__((__nonnull__(2))) ;
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkfifo(char const   *__path ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int mkfifoat(int __fd ,
                                                  char const   *__path ,
                                                  __mode_t __mode )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int utimensat(int __fd ,
                                                   char const   *__path ,
                                                   struct timespec  const  *__times ,
                                                   int __flags )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int futimens(int __fd ,
                                                  struct timespec  const  *__times ) ;
extern  __attribute__((__nothrow__)) int __fxstat(int __ver , int __fildes ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) int __xstat(int __ver ,
                                                 char const   *__filename ,
                                                 struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __lxstat(int __ver ,
                                                  char const   *__filename ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __fxstatat(int __ver , int __fildes ,
                                                    char const   *__filename ,
                                                    struct stat *__stat_buf ,
                                                    int __flag )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) int __xmknod(int __ver ,
                                                  char const   *__path ,
                                                  __mode_t __mode ,
                                                  __dev_t *__dev )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int __xmknodat(int __ver , int __fd ,
                                                    char const   *__path ,
                                                    __mode_t __mode ,
                                                    __dev_t *__dev )  __attribute__((__nonnull__(3,5))) ;
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int stat(char const   * __restrict  __path ,
                         struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int lstat(char const   * __restrict  __path ,
                          struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __lxstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern int fstat(int __fd , struct stat *__statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstat(3, __fd, __statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern int fstatat(int __fd , char const   * __restrict  __filename ,
                            struct stat * __restrict  __statbuf , int __flag ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstatat(3, __fd, (char const   *)__filename,
                   (struct stat *)__statbuf, __flag);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
__inline extern int mknod(char const   *__path , __mode_t __mode ,
                          __dev_t __dev ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8701\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xmknod(1, __path, __mode, & __dev);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8702\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
__inline extern int mknodat(int __fd , char const   *__path , __mode_t __mode ,
                            __dev_t __dev ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8703\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xmknodat(1, __fd, __path, __mode, & __dev);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8704\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
extern  __attribute__((__nothrow__)) unsigned short const   **__ctype_b_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_tolower_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_toupper_loc(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isalnum(int  ) ;
extern  __attribute__((__nothrow__)) int isalpha(int  ) ;
extern  __attribute__((__nothrow__)) int iscntrl(int  ) ;
extern  __attribute__((__nothrow__)) int isdigit(int  ) ;
extern  __attribute__((__nothrow__)) int islower(int  ) ;
extern  __attribute__((__nothrow__)) int isgraph(int  ) ;
extern  __attribute__((__nothrow__)) int isprint(int  ) ;
extern  __attribute__((__nothrow__)) int ispunct(int  ) ;
extern  __attribute__((__nothrow__)) int isspace(int  ) ;
extern  __attribute__((__nothrow__)) int isupper(int  ) ;
extern  __attribute__((__nothrow__)) int isxdigit(int  ) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
extern  __attribute__((__nothrow__)) int isblank(int  ) ;
extern  __attribute__((__nothrow__)) int isascii(int __c ) ;
extern  __attribute__((__nothrow__)) int toascii(int __c ) ;
extern  __attribute__((__nothrow__)) int _toupper(int  ) ;
extern  __attribute__((__nothrow__)) int _tolower(int  ) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern int tolower(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__c >= -128) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__c < 256) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8705\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __ctype_tolower_loc();
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8707\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
__inline extern int toupper(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8717\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__c >= -128) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__c < 256) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8712\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __ctype_toupper_loc();
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8713\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8714\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8718\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
extern  __attribute__((__nothrow__)) int isalnum_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalpha_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int iscntrl_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int islower_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isgraph_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isprint_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int ispunct_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isspace_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isupper_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isxdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isblank_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int __tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int utime(char const   *__file ,
                                               struct utimbuf  const  *__file_times )  __attribute__((__nonnull__(1))) ;
extern DIR *opendir(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern DIR *fdopendir(int __fd ) ;
extern int closedir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern struct dirent *readdir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int readdir_r(DIR * __restrict  __dirp ,
                     struct dirent * __restrict  __entry ,
                     struct dirent ** __restrict  __result )  __attribute__((__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) void rewinddir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void seekdir(DIR *__dirp , long __pos )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long telldir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int dirfd(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int scandir(char const   * __restrict  __dir ,
                   struct dirent *** __restrict  __namelist ,
                   int (*__selector)(struct dirent  const  * ) ,
                   int (*__cmp)(struct dirent  const  ** ,
                                struct dirent  const  ** ) )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int alphasort(struct dirent  const  **__e1 ,
                                                   struct dirent  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) __ssize_t getdirentries(int __fd ,
                                                             char * __restrict  __buf ,
                                                             size_t __nbytes ,
                                                             __off_t * __restrict  __basep )  __attribute__((__nonnull__(2,4))) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_startup(void) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_shutdown(void) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd_ex(size_t *length ) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd(char *buf ,
                                                                        size_t size ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir(char const   *path ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir_file(char const   *path ,
                                                                          int (*p_chdir)(char const   *path ) ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath(char const   *path ,
                                                                        char **filepath ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath_ex(char const   *path ,
                                                                           char **filepath ,
                                                                           int (*verify_path)(cwd_state const   * ) ) ;
extern char __attribute__((__visibility__("default")))  *virtual_realpath(char const   *path ,
                                                                          char *real_path ) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_fopen(char const   *path ,
                                                                       char const   *mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_open(char const   *path ,
                                                                    int flags 
                                                                    , ...) ;
extern int __attribute__((__visibility__("default")))  virtual_creat(char const   *path ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_rename(char *oldname ,
                                                                      char *newname ) ;
extern int __attribute__((__visibility__("default")))  virtual_stat(char const   *path ,
                                                                    struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_lstat(char const   *path ,
                                                                     struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_unlink(char const   *path ) ;
extern int __attribute__((__visibility__("default")))  virtual_mkdir(char const   *pathname ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_rmdir(char const   *pathname ) ;
extern DIR __attribute__((__visibility__("default")))  *virtual_opendir(char const   *pathname ) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_popen(char const   *command ,
                                                                       char const   *type ) ;
extern int __attribute__((__visibility__("default")))  virtual_access(char const   *pathname ,
                                                                      int mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_utime(char const   *filename ,
                                                                     struct utimbuf *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_chmod(char const   *filename ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_chown(char const   *filename ,
                                                                     uid_t owner ,
                                                                     gid_t group ,
                                                                     int link ) ;
extern int __attribute__((__visibility__("default")))  virtual_file_ex(cwd_state *state ,
                                                                       char const   *path ,
                                                                       int (*verify_path)(cwd_state const   * ) ,
                                                                       int use_realpath ) ;
extern char __attribute__((__visibility__("default")))  *tsrm_realpath(char const   *path ,
                                                                       char *real_path ) ;
extern virtual_cwd_globals cwd_globals ;
extern void __attribute__((__visibility__("default")))  realpath_cache_clean(void) ;
extern void __attribute__((__visibility__("default")))  realpath_cache_del(char const   *path ,
                                                                           int path_len ) ;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  *realpath_cache_lookup(char const   *path ,
                                                                                                int path_len ,
                                                                                                time_t t ) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_size(void) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_max_buckets(void) ;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  **realpath_cache_get_buckets(void) ;
static size_t encoding_filter_script_to_internal(unsigned char **to ,
                                                 size_t *to_length ,
                                                 unsigned char const   *from ,
                                                 size_t from_length ) 
{ zend_encoding const   *internal_encoding ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  size_t __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_multibyte_get_internal_encoding();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  internal_encoding = (zend_encoding const   *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (internal_encoding) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_multibyte_check_lexer_compatibility(internal_encoding);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8719\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8720\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __assert_fail("internal_encoding && zend_multibyte_check_lexer_compatibility(internal_encoding)",
                    "Zend/zend_language_scanner.l", 126U,
                    "encoding_filter_script_to_internal");
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __assert_fail("internal_encoding && zend_multibyte_check_lexer_compatibility(internal_encoding)",
                  "Zend/zend_language_scanner.l", 126U,
                  "encoding_filter_script_to_internal");
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_multibyte_encoding_converter(to, to_length, from, from_length,
                                              internal_encoding,
                                              language_scanner_globals.script_encoding);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )tmp___1);
}
}
static size_t encoding_filter_script_to_intermediate(unsigned char **to ,
                                                     size_t *to_length ,
                                                     unsigned char const   *from ,
                                                     size_t from_length ) 
{ size_t __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_multibyte_encoding_converter(to, to_length, from, from_length,
                                          (zend_encoding const   *)zend_multibyte_encoding_utf8,
                                          language_scanner_globals.script_encoding);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )tmp);
}
}
static size_t encoding_filter_intermediate_to_script(unsigned char **to ,
                                                     size_t *to_length ,
                                                     unsigned char const   *from ,
                                                     size_t from_length ) 
{ size_t __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_multibyte_encoding_converter(to, to_length, from, from_length,
                                          language_scanner_globals.script_encoding,
                                          (zend_encoding const   *)zend_multibyte_encoding_utf8);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )tmp);
}
}
static size_t encoding_filter_intermediate_to_internal(unsigned char **to ,
                                                       size_t *to_length ,
                                                       unsigned char const   *from ,
                                                       size_t from_length ) 
{ zend_encoding const   *internal_encoding ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  size_t __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_multibyte_get_internal_encoding();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  internal_encoding = (zend_encoding const   *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (internal_encoding) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8735\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_multibyte_check_lexer_compatibility(internal_encoding);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8736\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8733\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __assert_fail("internal_encoding && zend_multibyte_check_lexer_compatibility(internal_encoding)",
                    "Zend/zend_language_scanner.l", 144U,
                    "encoding_filter_intermediate_to_internal");
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __assert_fail("internal_encoding && zend_multibyte_check_lexer_compatibility(internal_encoding)",
                  "Zend/zend_language_scanner.l", 144U,
                  "encoding_filter_intermediate_to_internal");
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_multibyte_encoding_converter(to, to_length, from, from_length,
                                              internal_encoding,
                                              (zend_encoding const   *)zend_multibyte_encoding_utf8);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )tmp___1);
}
}
static void _yy_push_state(int new_state ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& language_scanner_globals.state_stack,
                  (void const   *)((void *)(& language_scanner_globals.yy_state)),
                  (int )sizeof(int ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = new_state;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void yy_pop_state(void) 
{ int *stack_state ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& language_scanner_globals.state_stack),
                 (void **)(& stack_state));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = *stack_state;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& language_scanner_globals.state_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void yy_scan_buffer(char *str , unsigned int len ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_cursor = (unsigned char *)str;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_limit = language_scanner_globals.yy_cursor + len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! language_scanner_globals.yy_start) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8750\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_start = language_scanner_globals.yy_cursor;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8751\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void startup_scanner(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.parse_error = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8757\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8758\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc_len = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8759\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.doc_comment = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8760\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.doc_comment_len = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8761\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& language_scanner_globals.state_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void shutdown_scanner(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.heredoc) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8763\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.heredoc);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8764\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.heredoc_len = 0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.parse_error = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& language_scanner_globals.state_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8772\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.doc_comment);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8773\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.doc_comment_len = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8774\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_save_lexical_state(zend_lex_state *lex_state ) 
{ char __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8775\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_leng = language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8776\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_start = language_scanner_globals.yy_start;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8777\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_text = language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8778\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_cursor = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_marker = language_scanner_globals.yy_marker;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_limit = language_scanner_globals.yy_limit;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->state_stack = language_scanner_globals.state_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8782\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& language_scanner_globals.state_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8783\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->in = language_scanner_globals.yy_in;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8784\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->yy_state = language_scanner_globals.yy_state;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8785\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_compiled_filename();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8786\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->filename = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8787\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->lineno = (unsigned int )compiler_globals.zend_lineno;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8788\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->script_org = language_scanner_globals.script_org;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8789\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->script_org_size = language_scanner_globals.script_org_size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8790\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->script_filtered = language_scanner_globals.script_filtered;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8791\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->script_filtered_size = language_scanner_globals.script_filtered_size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8792\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->input_filter = language_scanner_globals.input_filter;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8793\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->output_filter = language_scanner_globals.output_filter;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8794\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lex_state->script_encoding = language_scanner_globals.script_encoding;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8795\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_restore_lexical_state(zend_lex_state *lex_state ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8803\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = lex_state->yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_start = lex_state->yy_start;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_text = lex_state->yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_cursor = lex_state->yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8807\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = lex_state->yy_marker;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8808\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_limit = lex_state->yy_limit;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8809\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& language_scanner_globals.state_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8810\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.state_stack = lex_state->state_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8811\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_in = lex_state->in;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8812\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = lex_state->yy_state;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8813\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.zend_lineno = (int )lex_state->lineno;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8814\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_restore_compiled_filename(lex_state->filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8815\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.script_filtered) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8796\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)language_scanner_globals.script_filtered);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8816\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_org = lex_state->script_org;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8817\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_org_size = lex_state->script_org_size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8818\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_filtered = lex_state->script_filtered;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_filtered_size = lex_state->script_filtered_size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.input_filter = lex_state->input_filter;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.output_filter = lex_state->output_filter;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8822\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_encoding = lex_state->script_encoding;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8823\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.heredoc) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.heredoc);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.heredoc = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.heredoc_len = 0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8824\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_destroy_file_handle(zend_file_handle *file_handle ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8827\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_del_element(& compiler_globals.open_files, (void *)file_handle,
                         (int (*)(void * , void * ))(& zend_compare_file_handles));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8828\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle->opened_path = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8829\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (file_handle->free_filename) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8825\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    file_handle->filename = (char const   *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8826\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8830\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static zend_encoding const   *zend_multibyte_detect_utf_encoding(unsigned char const   *script ,
                                                                 size_t script_size ) 
{ unsigned char const   *p ;
  int wchar_size ;
  int le ;
  void *tmp ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  wchar_size = 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  le = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8864\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = script;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8865\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8837\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((size_t )(p - script) < script_size) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8831\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8838\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = memchr((void const   *)p, 0,
                 (script_size - (size_t )(p - script)) - 2U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = (unsigned char const   *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! p) {
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8832\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(p + 1) == 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8835\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(p + 2) == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8833\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        wchar_size = 4;
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8834\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8836\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8842\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p += 4;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = script;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8851\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((size_t )(p - script) < script_size) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8843\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8852\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*p == 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8845\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*((p + wchar_size) - 1) != 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8844\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        le = 0;
        break;
      } else {
        goto _L;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8850\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if ((int const   )*p != 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8848\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*((p + wchar_size) - 1) == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8846\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          le = 1;
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8847\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8849\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8853\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p += wchar_size;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (wchar_size == 2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8856\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (le) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8854\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_multibyte_encoding_utf16le;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_multibyte_encoding_utf16be;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_encoding const   *)tmp___0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8860\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (le) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_multibyte_encoding_utf32le;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8859\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_multibyte_encoding_utf32be;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8861\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_encoding const   *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8869\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_encoding const   *)((void *)0));
}
}
static zend_encoding const   *zend_multibyte_detect_unicode(void) 
{ zend_encoding const   *script_encoding ;
  int bom_size ;
  unsigned char *pos1 ;
  unsigned char *pos2 ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  void *tmp___4 ;
  int tmp___5 ;
  zend_encoding const   *tmp___6 ;
  void *tmp___7 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8950\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  script_encoding = (zend_encoding const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8951\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.script_org_size < sizeof("\377\376\000\000") - 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8870\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_encoding const   *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8871\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8952\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = memcmp((void const   *)language_scanner_globals.script_org,
                   (void const   *)"\000\000\376\377",
                   sizeof("\000\000\376\377") - 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8953\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8887\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = memcmp((void const   *)language_scanner_globals.script_org,
                     (void const   *)"\377\376\000\000",
                     sizeof("\377\376\000\000") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8888\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8883\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = memcmp((void const   *)language_scanner_globals.script_org,
                       (void const   *)"\376\377", sizeof("\376\377") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8884\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8879\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = memcmp((void const   *)language_scanner_globals.script_org,
                         (void const   *)"\377\376", sizeof("\377\376") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8880\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8875\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = memcmp((void const   *)language_scanner_globals.script_org,
                       (void const   *)"\357\273\277",
                       sizeof("\357\273\277") - 1U);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8876\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8872\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8873\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            script_encoding = (zend_encoding const   *)zend_multibyte_encoding_utf8;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8874\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            bom_size = (int )(sizeof("\357\273\277") - 1U);
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8877\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          script_encoding = (zend_encoding const   *)zend_multibyte_encoding_utf16le;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8878\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          bom_size = (int )(sizeof("\377\376") - 1U);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8881\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        script_encoding = (zend_encoding const   *)zend_multibyte_encoding_utf16be;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8882\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        bom_size = (int )(sizeof("\376\377") - 1U);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8885\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      script_encoding = (zend_encoding const   *)zend_multibyte_encoding_utf32le;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8886\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      bom_size = (int )(sizeof("\377\376\000\000") - 1U);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8889\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    script_encoding = (zend_encoding const   *)zend_multibyte_encoding_utf32be;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8890\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    bom_size = (int )(sizeof("\000\000\376\377") - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8954\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (script_encoding) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8891\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_org += bom_size;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8892\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_org_size -= (size_t )bom_size;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (script_encoding);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8894\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8955\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___7 = memchr((void const   *)language_scanner_globals.script_org, 0,
                   language_scanner_globals.script_org_size);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8956\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pos1 = (unsigned char *)tmp___7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8957\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (pos1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    pos2 = language_scanner_globals.script_org;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8946\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8938\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )(pos1 - pos2) >= sizeof("__HALT_COMPILER();") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8895\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8939\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = memchr((void const   *)pos2, '_', (unsigned int )(pos1 - pos2));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8940\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      pos2 = (unsigned char *)tmp___4;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8941\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! pos2) {
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8896\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      pos2 ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = strncasecmp((char const   *)((char *)pos2), "_HALT_COMPILER",
                            sizeof("_HALT_COMPILER") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8944\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___5 == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8934\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        pos2 += sizeof("_HALT_COMPILER") - 1U;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8935\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8904\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*pos2 == 32) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8897\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8903\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*pos2 == 9) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8898\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8902\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*pos2 == 13) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8899\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8901\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*pos2 == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8900\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8905\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          pos2 ++;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8936\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*pos2 == 40) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8930\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          pos2 ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8931\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8913\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*pos2 == 32) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8906\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8912\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*pos2 == 9) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8907\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8911\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*pos2 == 13) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8908\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8910\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*pos2 == 10) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "8909\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  } else {
                    break;
                  }
                }
              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8914\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            pos2 ++;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8932\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*pos2 == 41) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8926\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            pos2 ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8927\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8922\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*pos2 == 32) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8915\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8921\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*pos2 == 9) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8916\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8920\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*pos2 == 13) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "8917\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "8919\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int )*pos2 == 10) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "8918\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    } else {
                      break;
                    }
                  }
                }
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8923\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              pos2 ++;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8928\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*pos2 == 59) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8924\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              return ((zend_encoding const   *)((void *)0));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8925\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8929\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8933\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8937\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8947\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___6 = zend_multibyte_detect_utf_encoding((unsigned char const   *)language_scanner_globals.script_org,
                                                 language_scanner_globals.script_org_size);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8948\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (tmp___6);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8949\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8958\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_encoding const   *)((void *)0));
}
}
static zend_encoding const   *zend_multibyte_find_script_encoding(void) 
{ zend_encoding const   *script_encoding ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8971\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.detect_unicode) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8961\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    script_encoding = zend_multibyte_detect_unicode();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8962\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )script_encoding != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8959\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (script_encoding);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8960\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8963\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8972\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! compiler_globals.script_encoding_list) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8964\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_encoding const   *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8967\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compiler_globals.script_encoding_list_size) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8965\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((zend_encoding const   *)((void *)0));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8966\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8973\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.script_encoding_list_size > 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8968\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_multibyte_encoding_detector((unsigned char const   *)language_scanner_globals.script_org,
                                           language_scanner_globals.script_org_size,
                                           compiler_globals.script_encoding_list,
                                           compiler_globals.script_encoding_list_size);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8969\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_encoding const   *)tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8970\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8974\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (*(compiler_globals.script_encoding_list + 0));
}
}
int __attribute__((__visibility__("default")))  zend_multibyte_set_filter(zend_encoding const   *onetime_encoding ) 
{ zend_encoding const   *internal_encoding ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp ;
  zend_encoding const   *script_encoding ;
  zend_encoding const   *tmp___0 ;
  zend_encoding const   *tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8997\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_multibyte_get_internal_encoding();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8998\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  internal_encoding = (zend_encoding const   *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8999\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (onetime_encoding) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = onetime_encoding;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8976\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_multibyte_find_script_encoding();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8977\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = tmp___0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9000\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  script_encoding = tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! script_encoding) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8978\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8979\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.script_encoding = script_encoding;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9003\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.input_filter = (size_t (*)(unsigned char **str ,
                                                      size_t *str_length ,
                                                      unsigned char const   *buf ,
                                                      size_t length ))((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9004\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.output_filter = (size_t (*)(unsigned char **str ,
                                                       size_t *str_length ,
                                                       unsigned char const   *buf ,
                                                       size_t length ))((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9005\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! internal_encoding) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.script_encoding == (unsigned int )internal_encoding) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8984\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      tmp___2 = zend_multibyte_check_lexer_compatibility(language_scanner_globals.script_encoding);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8985\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8980\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.input_filter = (size_t (*)(unsigned char **str ,
                                                            size_t *str_length ,
                                                            unsigned char const   *buf ,
                                                            size_t length ))((void *)0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8981\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.output_filter = (size_t (*)(unsigned char **str ,
                                                             size_t *str_length ,
                                                             unsigned char const   *buf ,
                                                             size_t length ))((void *)0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.input_filter = & encoding_filter_script_to_intermediate;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.output_filter = & encoding_filter_intermediate_to_script;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8987\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9006\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = zend_multibyte_check_lexer_compatibility(internal_encoding);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9007\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.input_filter = & encoding_filter_script_to_internal;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8990\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.output_filter = (size_t (*)(unsigned char **str ,
                                                         size_t *str_length ,
                                                         unsigned char const   *buf ,
                                                         size_t length ))((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8995\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = zend_multibyte_check_lexer_compatibility(language_scanner_globals.script_encoding);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8991\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.input_filter = (size_t (*)(unsigned char **str ,
                                                          size_t *str_length ,
                                                          unsigned char const   *buf ,
                                                          size_t length ))((void *)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8992\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.output_filter = & encoding_filter_script_to_internal;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8993\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.input_filter = & encoding_filter_script_to_intermediate;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8994\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.output_filter = & encoding_filter_intermediate_to_internal;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9008\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
int __attribute__((__visibility__("default")))  open_file_for_scanning(zend_file_handle *file_handle ) 
{ char const   *file_path ;
  char *buf ;
  size_t size ;
  size_t offset ;
  long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zend_file_handle *fh ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  size_t diff ;
  char const __attribute__((__visibility__("default")))  *tmp___2 ;
  size_t tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9052\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_path = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9053\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9054\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.start_lineno == 2U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9017\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )file_handle->type == 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9015\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (file_handle->handle.fp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9011\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = ftell(file_handle->handle.fp);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9012\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset = (unsigned int )tmp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9013\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (offset == 4294967295U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9009\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset = 0U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9010\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9014\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9016\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9018\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9055\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_stream_fixup(file_handle, & buf, & size);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9056\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9019\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9020\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9057\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(& compiler_globals.open_files, (void *)file_handle);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9058\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )file_handle->handle.stream.handle >= (unsigned int )((void *)file_handle)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9027\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )file_handle->handle.stream.handle <= (unsigned int )((void *)(file_handle + 1))) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9021\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_llist_get_last_ex(& compiler_globals.open_files,
                                       (zend_llist_position *)((void *)0));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9022\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fh = (zend_file_handle *)tmp___1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9023\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      diff = (size_t )((char *)file_handle->handle.stream.handle - (char *)file_handle);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9024\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fh->handle.stream.handle = (void *)((char *)fh + diff);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9025\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      file_handle->handle.stream.handle = fh->handle.stream.handle;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9026\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9028\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9059\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_in = file_handle;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_start = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (size != 4294967295U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9043\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.multibyte) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9037\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.script_org = (unsigned char *)buf;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9038\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.script_org_size = size;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9039\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9040\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_multibyte_set_filter((zend_encoding const   *)((void *)0));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9041\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (language_scanner_globals.input_filter) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9032\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = (*(language_scanner_globals.input_filter))(& language_scanner_globals.script_filtered,
                                                             & language_scanner_globals.script_filtered_size,
                                                             (unsigned char const   *)language_scanner_globals.script_org,
                                                             language_scanner_globals.script_org_size);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9033\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (4294967295U == tmp___3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9029\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = zend_multibyte_get_encoding_name(language_scanner_globals.script_encoding);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9030\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error_noreturn(1 << 6L,
                              "Could not convert the script from the detected encoding \"%s\" to a compatible encoding",
                              tmp___2);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9031\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9034\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        buf = (char *)language_scanner_globals.script_filtered;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9035\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        size = language_scanner_globals.script_filtered_size;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9036\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9042\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9044\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_start = (unsigned char *)buf - offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9045\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    yy_scan_buffer(buf, size);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error_noreturn(1 << 6L, "zend_stream_mmap() failed");
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (file_handle->opened_path) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9047\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    file_path = (char const   *)file_handle->opened_path;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9048\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    file_path = file_handle->filename;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_set_compiled_filename(file_path);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.start_lineno) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9049\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.zend_lineno = (int )compiler_globals.start_lineno;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9050\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.start_lineno = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9051\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.zend_lineno = 1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.increment_lineno = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
zend_op_array __attribute__((__visibility__("default")))  *compile_file(zend_file_handle *file_handle ,
                                                                        int type ) 
{ zend_lex_state original_lex_state ;
  zend_op_array *op_array ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zend_op_array *original_active_op_array ;
  zend_op_array *retval ;
  int compiler_result ;
  zend_bool compilation_successful ;
  znode retval_znode ;
  zend_bool original_in_compilation ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(zend_op_array ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array = (zend_op_array *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  original_active_op_array = compiler_globals.active_op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval = (zend_op_array *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compilation_successful = (zend_bool )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  original_in_compilation = compiler_globals.in_compilation;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval_znode.op_type = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval_znode.u.constant.type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval_znode.u.constant.value.lval = 1L;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_unset_isref_p(& retval_znode.u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_set_refcount_p(& retval_znode.u.constant, 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_save_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval = op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = open_file_for_scanning(file_handle);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (type == 1 << 3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9068\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_message_dispatcher(2L, (void const   *)file_handle->filename);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9069\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_bailout((char *)"Zend/zend_language_scanner.l", 569U);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9070\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_message_dispatcher(1L, (void const   *)file_handle->filename);
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compilation_successful = (unsigned char)0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9075\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op_array(op_array, (unsigned char)2, 64);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9076\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.in_compilation = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9077\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.active_op_array = op_array;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9078\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_init_compiler_context();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9079\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_result = zendparse();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9080\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_return(& retval_znode, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9081\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.in_compilation = original_in_compilation;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9082\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_result == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9073\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_bailout((char *)"Zend/zend_language_scanner.l", 583U);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9083\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compilation_successful = (unsigned char)1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (retval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9088\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.active_op_array = original_active_op_array;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9089\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compilation_successful) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9084\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      pass_two(op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_release_labels();
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9087\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      retval = (zend_op_array *)((void *)0);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9090\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_restore_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9108\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_op_array __attribute__((__visibility__("default")))  *)retval);
}
}
zend_op_array __attribute__((__visibility__("default")))  *compile_filename(int type ,
                                                                            zval *filename ) 
{ zend_file_handle file_handle ;
  zval tmp ;
  zend_op_array *retval ;
  char *opened_path ;
  zend_op_array __attribute__((__visibility__("default")))  *tmp___0 ;
  int dummy ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  size_t tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opened_path = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )filename->type != 6) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = *filename;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9112\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& tmp);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9113\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )tmp.type != 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9109\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _convert_to_string(& tmp);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9110\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9114\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    filename = & tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9115\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.filename = (char const   *)filename->value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.free_filename = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.type = (enum __anonenum_zend_stream_type_38 )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.opened_path = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.handle.fp = (FILE *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = (*zend_compile_file)(& file_handle, type);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval = (zend_op_array *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (retval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9128\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (file_handle.handle.stream.handle) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9122\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      dummy = 1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9123\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! file_handle.opened_path) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9116\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = _estrndup((char const   *)filename->value.str.val,
                            (unsigned int )filename->value.str.len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9117\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opened_path = (char *)tmp___1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9118\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        file_handle.opened_path = opened_path;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9119\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9124\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = strlen((char const   *)file_handle.opened_path);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9125\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_hash_add_or_update(& executor_globals.included_files,
                               (char const   *)file_handle.opened_path,
                               tmp___2 + 1U, (void *)(& dummy), sizeof(int ),
                               (void **)((void *)0), 1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9126\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (opened_path) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9120\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)opened_path);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9121\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9127\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9129\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_destroy_file_handle(& file_handle);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )filename == (unsigned int )(& tmp)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9130\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(& tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9131\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_op_array __attribute__((__visibility__("default")))  *)retval);
}
}
int __attribute__((__visibility__("default")))  zend_prepare_string_for_scanning(zval *str ,
                                                                                 char *filename ) 
{ char *buf ;
  size_t size ;
  char *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp___2 ;
  char const __attribute__((__visibility__("default")))  *tmp___3 ;
  size_t tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )str->value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9151\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )str->value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9145\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _safe_emalloc(1U, (unsigned int )str->value.str.len, 32U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9146\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = (char *)tmp___0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9147\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      memcpy((void */* __restrict  */)tmp,
             (void const   */* __restrict  */)str->value.str.val,
             (unsigned int )(str->value.str.len + 32));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9148\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      str->value.str.val = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9149\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = _safe_erealloc((void *)str->value.str.val, 1U,
                               (unsigned int )str->value.str.len, 32U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9150\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      str->value.str.val = (char *)tmp___1;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9152\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = _safe_erealloc((void *)str->value.str.val, 1U,
                             (unsigned int )str->value.str.len, 32U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9153\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    str->value.str.val = (char *)tmp___1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(str->value.str.val + str->value.str.len), 0, 32U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_in = (zend_file_handle *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_start = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  buf = str->value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  size = (unsigned int )str->value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.multibyte) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9162\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_org = (unsigned char *)buf;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9163\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_org_size = size;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9164\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9165\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = zend_multibyte_get_internal_encoding();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9166\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_multibyte_set_filter((zend_encoding const   *)tmp___2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9167\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.input_filter) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9157\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = (*(language_scanner_globals.input_filter))(& language_scanner_globals.script_filtered,
                                                           & language_scanner_globals.script_filtered_size,
                                                           (unsigned char const   *)language_scanner_globals.script_org,
                                                           language_scanner_globals.script_org_size);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9158\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (4294967295U == tmp___4) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9154\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_multibyte_get_encoding_name(language_scanner_globals.script_encoding);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9155\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error_noreturn(1 << 6L,
                            "Could not convert the script from the detected encoding \"%s\" to a compatible encoding",
                            tmp___3);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9156\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9159\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      buf = (char *)language_scanner_globals.script_filtered;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9160\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      size = language_scanner_globals.script_filtered_size;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9161\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9168\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_scan_buffer(buf, size);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_set_compiled_filename((char const   *)filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.zend_lineno = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.increment_lineno = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
size_t __attribute__((__visibility__("default")))  zend_get_scanned_file_offset(void) 
{ size_t offset ;
  size_t original_offset ;
  size_t length ;
  unsigned char *p ;
  size_t tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset = (size_t )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_start);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.input_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9194\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    original_offset = offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9196\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9188\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p = (unsigned char *)((void *)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9189\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = (*(language_scanner_globals.input_filter))(& p, & length,
                                                       (unsigned char const   *)language_scanner_globals.script_org,
                                                       offset);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9190\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (4294967295U == tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9181\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned int __attribute__((__visibility__("default")))  )4294967295U);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9182\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9191\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)p);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9192\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (length > original_offset) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9183\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset --;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9186\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (length < original_offset) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9184\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9185\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9193\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (original_offset != length) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9187\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9197\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int __attribute__((__visibility__("default")))  )offset);
}
}
zend_op_array __attribute__((__visibility__("default")))  *compile_string(zval *source_string ,
                                                                          char *filename ) 
{ zend_lex_state original_lex_state ;
  zend_op_array *op_array ;
  void __attribute__((__visibility__("default")))  *tmp ;
  zend_op_array *original_active_op_array ;
  zend_op_array *retval ;
  zval tmp___0 ;
  int compiler_result ;
  zend_bool original_in_compilation ;
  zend_bool orig_interactive ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(zend_op_array ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array = (zend_op_array *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9233\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  original_active_op_array = compiler_globals.active_op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9234\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  original_in_compilation = compiler_globals.in_compilation;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9235\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (source_string->value.str.len == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_op_array __attribute__((__visibility__("default")))  *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9236\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_compilation = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = *source_string;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_copy_ctor(& tmp___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )tmp___0.type != 6) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _convert_to_string(& tmp___0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  source_string = & tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_save_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_prepare_string_for_scanning(source_string, filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    retval = (zend_op_array *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    orig_interactive = (zend_bool )compiler_globals.interactive;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.interactive = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9223\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op_array(op_array, (unsigned char)4, 64);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9224\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.interactive = (int )orig_interactive;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.active_op_array = op_array;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_init_compiler_context();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_result = zendparse();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9229\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.script_filtered) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9208\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)language_scanner_globals.script_filtered);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9209\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9210\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9230\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_result == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9211\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.active_op_array = original_active_op_array;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9212\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.unclean_shutdown = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9213\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      destroy_op_array(op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9214\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9215\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      retval = (zend_op_array *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9216\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_return((znode *)((void *)0), 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9217\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.active_op_array = original_active_op_array;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9218\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      pass_two(op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9219\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_release_labels();
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9220\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      retval = op_array;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_restore_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(& tmp___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_compilation = original_in_compilation;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_op_array __attribute__((__visibility__("default")))  *)retval);
}
}
int __attribute__((__visibility__("default")))  highlight_file(char *filename ,
                                                               zend_syntax_highlighter_ini *syntax_highlighter_ini___0 ) 
{ zend_lex_state original_lex_state ;
  zend_file_handle file_handle ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.type = (enum __anonenum_zend_stream_type_38 )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.filename = (char const   *)filename;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.free_filename = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9258\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  file_handle.opened_path = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9259\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_save_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9260\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = open_file_for_scanning(& file_handle);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9248\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_message_dispatcher(3L, (void const   *)filename);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_restore_lexical_state(& original_lex_state);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9251\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9262\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_highlight(syntax_highlighter_ini___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9263\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.script_filtered) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9252\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)language_scanner_globals.script_filtered);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9253\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9254\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_destroy_file_handle(& file_handle);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_restore_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
int __attribute__((__visibility__("default")))  highlight_string(zval *str ,
                                                                 zend_syntax_highlighter_ini *syntax_highlighter_ini___0 ,
                                                                 char *str_name ) 
{ zend_lex_state original_lex_state ;
  zval tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9273\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = *str;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9274\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  str = & tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_copy_ctor(str);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_save_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_prepare_string_for_scanning(str, str_name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9267\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_restore_lexical_state(& original_lex_state);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9268\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9269\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_highlight(syntax_highlighter_ini___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.script_filtered) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9270\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)language_scanner_globals.script_filtered);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9271\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9272\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_restore_lexical_state(& original_lex_state);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(str);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
void __attribute__((__visibility__("default")))  zend_multibyte_yyinput_again(size_t (*old_input_filter)(unsigned char **str ,
                                                                                                         size_t *str_length ,
                                                                                                         unsigned char const   *buf ,
                                                                                                         size_t length ) ,
                                                                              zend_encoding const   *old_encoding ) 
{ size_t length ;
  unsigned char *new_yy_start ;
  char const __attribute__((__visibility__("default")))  *tmp ;
  size_t tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! language_scanner_globals.input_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9288\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.script_filtered) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9285\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)language_scanner_globals.script_filtered);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9286\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.script_filtered = (unsigned char *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9287\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9289\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered_size = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9290\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length = language_scanner_globals.script_org_size;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9291\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    new_yy_start = language_scanner_globals.script_org;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9295\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (*(language_scanner_globals.input_filter))(& new_yy_start,
                                                         & length,
                                                         (unsigned char const   *)language_scanner_globals.script_org,
                                                         language_scanner_globals.script_org_size);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9296\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (4294967295U == tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9292\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_multibyte_get_encoding_name(language_scanner_globals.script_encoding);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9293\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error_noreturn(1 << 6L,
                          "Could not convert the script from the detected encoding \"%s\" to a compatible encoding",
                          tmp);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9294\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9297\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered = new_yy_start;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9298\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.script_filtered_size = length;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_cursor = new_yy_start + (language_scanner_globals.yy_cursor - language_scanner_globals.yy_start);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = new_yy_start + (language_scanner_globals.yy_marker - language_scanner_globals.yy_start);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_text = new_yy_start + (language_scanner_globals.yy_text - language_scanner_globals.yy_start);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_limit = new_yy_start + (language_scanner_globals.yy_limit - language_scanner_globals.yy_start);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_start = new_yy_start;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_scan_escape_string(zval *zendlval , char *str , int len ,
                                    char quote_type ) 
{ register char *s ;
  register char *t ;
  char *end ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp ;
  char *tmp___0 ;
  char *tmp___1 ;
  char *tmp___2 ;
  char *tmp___3 ;
  char *tmp___4 ;
  char *tmp___5 ;
  char *tmp___6 ;
  char *tmp___7 ;
  char *tmp___8 ;
  char *tmp___9 ;
  char hex_buf[3] ;
  char *tmp___10 ;
  long tmp___11 ;
  char *tmp___12 ;
  char *tmp___13 ;
  char octal_buf[4] ;
  char *tmp___14 ;
  long tmp___15 ;
  char *tmp___16 ;
  char *tmp___17 ;
  char *tmp___18 ;
  size_t sz ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9306\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = (char const   *)str;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9307\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l = len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9308\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = zendlval;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9309\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9310\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _estrndup(__s, (unsigned int )__l);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.val = (char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9312\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  t = zendlval->value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  s = t;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end = s + zendlval->value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9438\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )s < (unsigned int )end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9313\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9439\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s == 92) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9427\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      s ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9428\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )s >= (unsigned int )end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9314\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = t;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9315\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        t ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9316\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___0 = (char )'\\';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9317\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      switch ((int )*s) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9395\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 110: 
      tmp___1 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___1 = (char )'\n';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9398\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9399\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 114: 
      tmp___2 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9400\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9401\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___2 = (char )'\r';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9402\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9403\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 116: 
      tmp___3 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9404\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9405\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___3 = (char )'\t';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9406\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9407\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 102: 
      tmp___4 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9408\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9409\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___4 = (char )'\f';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9410\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9411\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 118: 
      tmp___5 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9412\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9413\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___5 = (char )'\v';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9414\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9415\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 101: 
      tmp___6 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9416\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9417\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___6 = (char )'\033';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9418\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9419\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 34: 
      case 96: 
      if ((int )*s != (int )quote_type) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9318\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___7 = t;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9319\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        t ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9320\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___7 = (char )'\\';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9321\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = t;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9322\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        t ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9323\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___8 = *s;
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9324\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9420\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 92: 
      case 36: 
      tmp___9 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9421\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9422\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___9 = *s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9423\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9424\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 120: 
      case 88: 
      if ((int )*(s + 1) >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9325\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(s + 1) <= 57) {
          goto _L___1;
        } else {
          goto _L___4;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9363\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___4: /* CIL Label */ 
        if ((int )*(s + 1) >= 97) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9326\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(s + 1) <= 102) {
            goto _L___1;
          } else {
            goto _L___3;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9362\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___3: /* CIL Label */ 
          if ((int )*(s + 1) >= 65) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9355\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(s + 1) <= 70) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9343\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___1: /* CIL Label */ 
              hex_buf[0] = (char)0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9344\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              hex_buf[1] = (char)0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9345\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              hex_buf[2] = (char)0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9346\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (zendlval->value.str.len) --;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9347\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              s ++;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9348\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              hex_buf[0] = *s;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9349\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (zendlval->value.str.len) --;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9350\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*(s + 1) >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9330\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*(s + 1) <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9327\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  s ++;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9328\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  hex_buf[1] = *s;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9329\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  (zendlval->value.str.len) --;
                } else {
                  goto _L___0;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9342\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((int )*(s + 1) >= 97) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9334\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*(s + 1) <= 102) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9331\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    s ++;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9332\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    hex_buf[1] = *s;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9333\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    (zendlval->value.str.len) --;
                  } else {
                    goto _L;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9341\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L: /* CIL Label */ 
                  if ((int )*(s + 1) >= 65) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9339\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int )*(s + 1) <= 70) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9335\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      s ++;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9336\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      hex_buf[1] = *s;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9337\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      (zendlval->value.str.len) --;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9338\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9340\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9351\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___10 = t;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9352\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              t ++;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9353\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___11 = strtol((char const   */* __restrict  */)(hex_buf),
                                (char **/* __restrict  */)((void *)0), 16);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9354\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *tmp___10 = (char )tmp___11;
            } else {
              goto _L___2;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9356\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___2: /* CIL Label */ 
            tmp___12 = t;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9357\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            t ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9358\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *tmp___12 = (char )'\\';
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9359\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___13 = t;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9360\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            t ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9361\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *tmp___13 = *s;
          }
        }
      }
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9425\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      default: ;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9426\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*s >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9388\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*s <= 55) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9377\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          octal_buf[0] = (char)0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9378\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          octal_buf[1] = (char)0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9379\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          octal_buf[2] = (char)0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9380\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          octal_buf[3] = (char)0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9381\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          octal_buf[0] = *s;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9382\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (zendlval->value.str.len) --;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9383\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(s + 1) >= 48) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9375\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(s + 1) <= 55) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9370\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              s ++;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9371\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              octal_buf[1] = *s;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9372\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (zendlval->value.str.len) --;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9373\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*(s + 1) >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9368\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*(s + 1) <= 55) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9364\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  s ++;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9365\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  octal_buf[2] = *s;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9366\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  (zendlval->value.str.len) --;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9367\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9369\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9374\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9376\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9384\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___14 = t;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9385\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          t ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9386\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___15 = strtol((char const   */* __restrict  */)(octal_buf),
                            (char **/* __restrict  */)((void *)0), 8);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9387\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp___14 = (char )tmp___15;
        } else {
          goto _L___5;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9389\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___5: /* CIL Label */ 
        tmp___16 = t;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9390\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        t ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9391\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___16 = (char )'\\';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9392\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___17 = t;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9393\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        t ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9394\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___17 = *s;
      }
      break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9429\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___18 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9430\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9431\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___18 = *s;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9440\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9432\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.zend_lineno) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9437\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*s == 13) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9435\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(s + 1) != 10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9433\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.zend_lineno) ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9434\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9436\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9441\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *t = (char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9442\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9443\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s = zendlval->value.str.val;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9444\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz,
                                                (unsigned char const   *)((unsigned char *)s),
                                                (unsigned int )zendlval->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9445\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9446\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)s);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9447\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static unsigned char const   yybm[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0};
static unsigned char const   yybm___0[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128};
static unsigned char const   yybm___1[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128};
static unsigned char const   yybm___2[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128};
static unsigned char const   yybm___3[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )192,      (unsigned char const   )64,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )64,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )192,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )60,      (unsigned char const   )60,      (unsigned char const   )44,      (unsigned char const   )44, 
        (unsigned char const   )44,      (unsigned char const   )44,      (unsigned char const   )44,      (unsigned char const   )44, 
        (unsigned char const   )44,      (unsigned char const   )44,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )36, 
        (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )4, 
        (unsigned char const   )0,      (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )36, 
        (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )36,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4, 
        (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4,      (unsigned char const   )4};
static unsigned char const   yybm___4[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )64, 
        (unsigned char const   )0,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64, 
        (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64,      (unsigned char const   )64};
static unsigned char const   yybm___5[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )0,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128, 
        (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128,      (unsigned char const   )128};
static unsigned char const   yybm___6[256]  = 
  {      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )240,      (unsigned char const   )240,      (unsigned char const   )112,      (unsigned char const   )112, 
        (unsigned char const   )112,      (unsigned char const   )112,      (unsigned char const   )112,      (unsigned char const   )112, 
        (unsigned char const   )112,      (unsigned char const   )112,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )80, 
        (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )16, 
        (unsigned char const   )0,      (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )80, 
        (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )80,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )0, 
        (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )0,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16, 
        (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16,      (unsigned char const   )16};
int __attribute__((__visibility__("default")))  lex_scan(zval *zendlval ) 
{ unsigned char yych ;
  unsigned int yyaccept ;
  unsigned char *ptr ;
  void *tmp ;
  int tmp___0 ;
  int readsize ;
  size_t sz ;
  size_t tmp___1 ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;
  char *p ;
  char *boundary ;
  unsigned char *bracket ;
  void const   *tmp___3 ;
  char *p___0 ;
  char *boundary___0 ;
  unsigned char *tmp___4 ;
  size_t sz___0 ;
  char __attribute__((__visibility__("default")))  *tmp___5 ;
  size_t sz___1 ;
  char __attribute__((__visibility__("default")))  *tmp___6 ;
  size_t sz___2 ;
  char __attribute__((__visibility__("default")))  *tmp___7 ;
  unsigned char *tmp___8 ;
  size_t sz___3 ;
  char __attribute__((__visibility__("default")))  *tmp___9 ;
  size_t sz___4 ;
  char __attribute__((__visibility__("default")))  *tmp___10 ;
  size_t sz___5 ;
  char __attribute__((__visibility__("default")))  *tmp___11 ;
  int newline ;
  unsigned char *tmp___12 ;
  unsigned char *end ;
  int tmp___13 ;
  size_t sz___6 ;
  char __attribute__((__visibility__("default")))  *tmp___14 ;
  size_t sz___7 ;
  char __attribute__((__visibility__("default")))  *tmp___15 ;
  size_t sz___8 ;
  char __attribute__((__visibility__("default")))  *tmp___16 ;
  size_t sz___9 ;
  char __attribute__((__visibility__("default")))  *tmp___17 ;
  char *p___1 ;
  char *boundary___1 ;
  int __attribute__((__visibility__("default")))  tmp___18 ;
  int *tmp___19 ;
  double __attribute__((__visibility__("default")))  tmp___20 ;
  double __attribute__((__visibility__("default")))  tmp___21 ;
  int *tmp___22 ;
  unsigned char *tmp___23 ;
  register char *s ;
  register char *t ;
  char *end___0 ;
  int bprefix ;
  int tmp___24 ;
  unsigned char *tmp___25 ;
  char __attribute__((__visibility__("default")))  *tmp___26 ;
  char *tmp___27 ;
  char *tmp___28 ;
  char *tmp___29 ;
  char *tmp___30 ;
  size_t sz___10 ;
  int bprefix___0 ;
  int tmp___31 ;
  unsigned char *tmp___32 ;
  double __attribute__((__visibility__("default")))  tmp___33 ;
  char *bin ;
  int len ;
  double __attribute__((__visibility__("default")))  tmp___34 ;
  char *hex ;
  int len___0 ;
  double __attribute__((__visibility__("default")))  tmp___35 ;
  size_t sz___11 ;
  char __attribute__((__visibility__("default")))  *tmp___36 ;
  int doc_com ;
  unsigned char *tmp___37 ;
  char *p___2 ;
  char *boundary___2 ;
  char __attribute__((__visibility__("default")))  *tmp___38 ;
  char *s___0 ;
  int bprefix___1 ;
  int tmp___39 ;
  int tmp___40 ;
  char __attribute__((__visibility__("default")))  *tmp___41 ;
  unsigned char *end___1 ;
  int tmp___42 ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___43 ;
  char *filename ;
  char __attribute__((__visibility__("default")))  *tmp___44 ;
  size_t filename_len ;
  size_t tmp___45 ;
  char *dirname ;
  char __attribute__((__visibility__("default")))  *tmp___46 ;
  void __attribute__((__visibility__("default")))  *tmp___47 ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___48 ;
  int tmp___49 ;
  int tmp___50 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___51 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___52 ;
  int tmp___53 ;
  int tmp___54 ;
  size_t tmp___55 ;
  char const   *class_name ;
  char const   *tmp___56 ;
  char const   *func_name ;
  char const   *tmp___57 ;
  size_t len___1 ;
  size_t tmp___58 ;
  size_t tmp___59 ;
  char const   *tmp___60 ;
  char const   *tmp___61 ;
  char const   *tmp___62 ;
  char const   *func_name___0 ;
  size_t tmp___63 ;
  char __attribute__((__visibility__("default")))  *tmp___64 ;
  char *filename___0 ;
  char __attribute__((__visibility__("default")))  *tmp___65 ;
  size_t tmp___66 ;
  char __attribute__((__visibility__("default")))  *tmp___67 ;
  char const   *trait_name ;
  size_t tmp___68 ;
  char __attribute__((__visibility__("default")))  *tmp___69 ;
  char const   *class_name___0 ;
  char __attribute__((__visibility__("default")))  *tmp___70 ;
  size_t tmp___71 ;
  char __attribute__((__visibility__("default")))  *tmp___72 ;
  char *p___3 ;
  char *boundary___3 ;
  size_t sz___12 ;
  char __attribute__((__visibility__("default")))  *tmp___73 ;
  size_t sz___13 ;
  char __attribute__((__visibility__("default")))  *tmp___74 ;
  int newline___0 ;
  unsigned char *tmp___75 ;
  unsigned char *end___2 ;
  int tmp___76 ;
  size_t sz___14 ;
  char __attribute__((__visibility__("default")))  *tmp___77 ;
  char *p___4 ;
  char *boundary___4 ;
  char __attribute__((__visibility__("default")))  *tmp___78 ;
  size_t __s1_len___0 ;
  size_t __s2_len___0 ;
  int tmp___79 ;
  int tmp___80 ;
  int tmp___81 ;
  unsigned char const   *__s2___0 ;
  register int __result___1 ;
  int tmp___82 ;
  unsigned char const   *__s1___0 ;
  register int __result___2 ;
  int tmp___83 ;
  int tmp___84 ;
  int tmp___85 ;
  size_t sz___15 ;
  char __attribute__((__visibility__("default")))  *tmp___86 ;
  size_t sz___16 ;
  char __attribute__((__visibility__("default")))  *tmp___87 ;
  char __attribute__((__visibility__("default")))  *tmp___88 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  restart: 
  language_scanner_globals.yy_text = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yymore_restart: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.yy_state < 5) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9459\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.yy_state < 2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9456\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (language_scanner_globals.yy_state < 1) {
        goto yyc_ST_IN_SCRIPTING;
      } else {
        goto yyc_ST_LOOKING_FOR_PROPERTY;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9458\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (language_scanner_globals.yy_state < 3) {
        goto yyc_ST_BACKQUOTE;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9457\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (language_scanner_globals.yy_state < 4) {
          goto yyc_ST_DOUBLE_QUOTES;
        } else {
          goto yyc_ST_HEREDOC;
        }
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9463\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.yy_state < 7) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9460\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (language_scanner_globals.yy_state < 6) {
        goto yyc_ST_LOOKING_FOR_VARNAME;
      } else {
        goto yyc_ST_VAR_OFFSET;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9462\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (language_scanner_globals.yy_state < 8) {
        goto yyc_INITIAL;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9461\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (language_scanner_globals.yy_state < 9) {
          goto yyc_ST_END_HEREDOC;
        } else {
          goto yyc_ST_NOWDOC;
        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12013\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_INITIAL: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 8) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9464\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9465\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12014\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 60) {
    goto yy4;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9466\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 63) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9469\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 37) {
      goto yy7;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9467\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9470\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 63) {
      goto yy5;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9468\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9475\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 83) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9472\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 83) {
        goto yy9;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9471\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9474\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 115) {
        goto yy9;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9473\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy3: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  inline_char_handler: 
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = memchr((void const   *)language_scanner_globals.yy_cursor, '<',
                 (unsigned int )(language_scanner_globals.yy_limit - language_scanner_globals.yy_cursor));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ptr = (unsigned char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9493\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9478\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.yy_cursor = ptr + 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9479\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.yy_cursor = language_scanner_globals.yy_limit;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9494\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      switch ((int )*(language_scanner_globals.yy_cursor)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9485\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 63: 
      if (compiler_globals.short_tags) {
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9482\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = strncasecmp((char const   *)((char *)language_scanner_globals.yy_cursor + 1),
                              "php", 3U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9483\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9481\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor + 1) == 61) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9480\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          break;
        }
      }
      continue;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9486\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 37: 
      if (compiler_globals.asp_tags) {
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9484\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      continue;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9487\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 115: 
      case 83: 
      (language_scanner_globals.yy_cursor) --;
      goto yymore_restart;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9488\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      default: ;
      continue;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9489\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) --;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9490\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12024\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  inline_html: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9499\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9500\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                          & sz,
                                                          (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                          language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9501\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    readsize = (int )tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9502\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9503\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )readsize < language_scanner_globals.yy_leng) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9497\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9495\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + readsize;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9496\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.yy_leng = (unsigned int )readsize;
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9498\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                        language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9506\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary = p + language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9519\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9514\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p < (unsigned int )boundary) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9507\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9515\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9508\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9513\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9511\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9509\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9510\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9512\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9516\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )311);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy4: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy5: 
  yyaccept = 1U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12033\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12034\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12035\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9521\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 61) {
      goto yy45;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9520\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9524\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 80) {
      goto yy47;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9522\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9525\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 112) {
      goto yy47;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9523\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12036\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy6: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12037\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.short_tags) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9526\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9527\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9528\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )372);
  } else {
    goto inline_char_handler;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12038\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy7: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy43;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9531\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.asp_tags) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9532\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9533\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9534\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9535\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9536\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )372);
  } else {
    goto inline_char_handler;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy9: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12045\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy11;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9537\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12046\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 99) {
    goto yy11;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9538\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12047\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy10: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12048\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (yyaccept <= 0U) {
    goto yy3;
  } else {
    goto yy6;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12049\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy11: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12050\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12051\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy12;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9539\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12052\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9540\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12053\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy12: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12054\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12055\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy13;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12056\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12057\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy13: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12058\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12059\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy14;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9543\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9544\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy14: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy15;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9545\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9546\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy15: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9547\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9548\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy17;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy16: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 8) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9549\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy17: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm[(int )yych] & 128) {
    goto yy16;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy18;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9553\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy18: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy19;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9554\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy19: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy20;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9557\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy20: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12085\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12086\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 71) {
    goto yy21;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9558\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12087\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 103) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9559\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12088\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy21: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12089\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12090\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy22;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9560\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9561\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy22: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy23;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy23: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 71) {
    goto yy24;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9564\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 103) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9565\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy24: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy25;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9566\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9567\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy25: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9568\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 13) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9573\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 0x08) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9570\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9574\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 10) {
      goto yy25;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9571\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9575\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 12) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9572\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy25;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9580\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9577\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 0x1F) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9576\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy25;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9579\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 61) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9578\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12108\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy27: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12109\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 5) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9581\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9582\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12110\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 33) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 12) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9585\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 0x08) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9583\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9586\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 10) {
        goto yy27;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9584\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9589\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 13) {
        goto yy27;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9587\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9590\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 32) {
        goto yy27;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9588\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy10;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9600\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 79) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9594\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 34) {
        goto yy30;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9592\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9595\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 39) {
        goto yy31;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9593\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9598\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 80) {
        goto yy29;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9596\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9599\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 112) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9597\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy29: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy42;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9601\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 104) {
    goto yy42;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9602\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12116\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy30: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12117\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy39;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9603\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12119\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 112) {
    goto yy39;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9604\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12120\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy31: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12121\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy32;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9605\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9606\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12124\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy32: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12125\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy33;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9607\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9608\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy33: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy34;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9610\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy34: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 39) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9611\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy35: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9612\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9613\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 13) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 0x08) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9614\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 10) {
      goto yy35;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9615\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 12) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9616\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy35;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9621\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 0x1F) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9620\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy35;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9623\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 62) {
        goto yy10;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9622\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = zend_memrchr((void const   *)((char *)language_scanner_globals.yy_text),
                         '<',
                         language_scanner_globals.yy_leng - (sizeof("script language=php>") - 1U));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  bracket = (unsigned char *)tmp___3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )bracket != (unsigned int )language_scanner_globals.yy_text) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = bracket;
    goto inline_html;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p___0 = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary___0 = p___0 + language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9639\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9634\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p___0 < (unsigned int )boundary___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9627\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9635\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p___0 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9628\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9633\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p___0 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9631\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p___0 + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9629\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9630\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9632\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9636\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p___0 ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )372);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy39: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy40;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy40: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12156\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy41;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9642\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9643\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy41: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 34) {
    goto yy35;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy42: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy35;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 112) {
    goto yy35;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9646\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy43: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.asp_tags) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9647\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9648\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9649\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9650\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9651\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )373);
  } else {
    goto inline_char_handler;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy45: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )373);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy47: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy48;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9652\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9653\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy48: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy49;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9654\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy10;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9655\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy49: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 12) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9658\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 0x08) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9656\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9659\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 11) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9657\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 13) {
      goto yy52;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9660\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 32) {
      goto yy10;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9661\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy50: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy51: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + (language_scanner_globals.yy_leng - 1U)) == 10) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.zend_lineno) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9667\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*((char *)language_scanner_globals.yy_text + (language_scanner_globals.yy_leng - 1U)) == 13) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9665\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.zend_lineno) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9666\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )372);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy52: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy50;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9668\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy51;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_BACKQUOTE: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9670\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9672\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 36) {
      goto yy60;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9675\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy58;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9673\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9676\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 123) {
      goto yy57;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9674\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy60;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12201\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12202\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9680\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy56;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9677\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9681\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy63;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9678\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9682\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 95) {
      goto yy63;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9679\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9689\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9684\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 97) {
        goto yy63;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9683\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9687\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 123) {
        goto yy66;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9685\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9688\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy63;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9686\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy56: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9690\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9691\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9694\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9692\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9693\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9695\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9711\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9696\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9712\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9713\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___4) {
    case 96: 
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9707\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 36: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9697\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        break;
      } else {
        goto _L___0;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9703\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___0: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9698\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          break;
        } else {
          goto _L;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9702\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9701\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9700\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(language_scanner_globals.yy_cursor) == 123) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9699\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
      }
    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 123: 
    if ((int )*(language_scanner_globals.yy_cursor) == 36) {
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9704\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 92: 
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9705\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9710\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9714\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_scan_escape_string(zendlval, (char *)language_scanner_globals.yy_text,
                          (int )language_scanner_globals.yy_leng, (char )'`');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )314);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy57: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 36) {
    goto yy61;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy56;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy58: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12215\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'`');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy60: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy56;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy61: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12220\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12221\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.lval = (long )'{';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12222\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12223\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9717\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 1U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12224\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )379);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12225\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy63: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12227\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9719\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12229\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12230\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___0[(int )yych] & 128) {
    goto yy63;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9720\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 45) {
    goto yy68;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 91) {
    goto yy70;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12233\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy65: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12234\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___0 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___0,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9725\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9726\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                        language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9727\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___5;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9728\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12235\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12236\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy66: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(5);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )378);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy68: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 62) {
    goto yy72;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9729\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy69: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  goto yy65;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy70: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9730\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9731\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_leng) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12248\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(6);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12249\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9732\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___1 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9733\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___1,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9734\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9735\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___6 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                        language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9736\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12250\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12251\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12252\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy72: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12253\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12254\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9741\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy69;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9738\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9742\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy73;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9739\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9743\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 94) {
      goto yy69;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9740\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9747\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy69;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9744\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9748\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy73;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9745\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9749\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 126) {
      goto yy69;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9746\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy73: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9750\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9751\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng -= 3U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12258\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12259\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9752\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___2 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9753\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___2,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___2;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___7 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                        language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___7;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12260\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12262\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_DOUBLE_QUOTES: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9759\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12263\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 35) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9761\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 34) {
      goto yy80;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9760\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy82;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9764\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 36) {
      goto yy77;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9762\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 123) {
      goto yy79;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9763\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy82;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy77: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12267\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9769\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy78;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9766\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9770\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy85;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9767\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 95) {
      goto yy85;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9768\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9778\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9773\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 97) {
        goto yy85;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9772\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9776\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 123) {
        goto yy88;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9774\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9777\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy85;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9775\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12268\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy78: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12269\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment_len) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9779\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor += compiler_globals.doc_comment_len - 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9780\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
    goto double_quotes_scan_done;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9781\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12270\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9782\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9783\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12271\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9786\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9784\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9785\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12272\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9788\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9804\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___8 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9805\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___8) {
    case 34: 
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 36: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9789\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        break;
      } else {
        goto _L___2;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___2: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9790\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          break;
        } else {
          goto _L___1;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9794\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___1: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9793\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9792\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(language_scanner_globals.yy_cursor) == 123) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9791\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
      }
    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 123: 
    if ((int )*(language_scanner_globals.yy_cursor) == 36) {
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9796\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 92: 
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9797\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9798\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9806\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12273\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  double_quotes_scan_done: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12274\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_scan_escape_string(zendlval, (char *)language_scanner_globals.yy_text,
                          (int )language_scanner_globals.yy_leng, (char )'\"');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )314);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy79: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 36) {
    goto yy83;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9807\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy78;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy80: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'\"');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy82: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy78;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12285\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy83: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12286\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.lval = (long )'{';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9808\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9809\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 1U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )379);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy85: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12292\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12293\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12294\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9810\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9811\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___1[(int )yych] & 128) {
    goto yy85;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9812\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 45) {
    goto yy90;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9813\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 91) {
    goto yy92;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9814\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy87: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___3 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___3,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9817\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___3;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9818\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___9 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                        language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9819\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___9;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9820\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy88: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(5);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )378);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy90: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12309\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 62) {
    goto yy94;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9821\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12310\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy91: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  goto yy87;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12311\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy92: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9822\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9823\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_leng) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(6);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9824\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___4 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9825\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___4,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9826\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___4;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9827\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___10 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9828\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___10;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9829\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy94: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9833\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy91;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9830\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9834\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy95;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9831\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9835\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 94) {
      goto yy91;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9832\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy91;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9836\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy95;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9837\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 126) {
      goto yy91;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9838\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy95: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9842\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9843\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng -= 3U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9844\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___5 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9845\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___5,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9846\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___5;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9847\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___11 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9848\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___11;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9849\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12327\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12328\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_END_HEREDOC: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12329\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9850\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9851\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12330\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12331\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_cursor += compiler_globals.heredoc_len - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )compiler_globals.heredoc_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = compiler_globals.heredoc;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = compiler_globals.heredoc_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12337\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12338\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc_len = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12339\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12340\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )377);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_HEREDOC: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9852\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9853\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12343\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 36) {
    goto yy103;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9854\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12344\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 123) {
    goto yy105;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9855\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy106;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12345\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy103: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12346\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12347\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9859\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy104;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9856\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9860\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy109;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9857\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9861\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 95) {
      goto yy109;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9868\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9863\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 97) {
        goto yy109;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9862\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9866\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 123) {
        goto yy112;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9864\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9867\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy109;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9865\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12348\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy104: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12349\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  newline = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12350\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9869\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9870\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12351\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9919\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9871\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9920\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___12 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9921\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___12) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9913\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 13: 
    if ((int )*(language_scanner_globals.yy_cursor) == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9872\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9873\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9914\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 10: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9874\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        goto _L___4;
      } else {
        goto _L___6;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9898\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___6: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9875\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          goto _L___4;
        } else {
          goto _L___5;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9897\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___5: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          goto _L___4;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9896\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9894\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___4: /* CIL Label */ 
            if (compiler_globals.heredoc_len < language_scanner_globals.yy_limit - language_scanner_globals.yy_cursor) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9891\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___13 = memcmp((void const   *)language_scanner_globals.yy_cursor,
                                (void const   *)compiler_globals.heredoc,
                                (unsigned int )compiler_globals.heredoc_len);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9892\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___13) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9876\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9888\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                end = language_scanner_globals.yy_cursor + compiler_globals.heredoc_len;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9889\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*end == 59) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9877\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  end ++;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9878\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "9890\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*end == 10) {
                  goto _L___3;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "9887\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*end == 13) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9883\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L___3: /* CIL Label */ 
                    if ((int )*(language_scanner_globals.yy_cursor + -2) == 13) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9881\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int )*(language_scanner_globals.yy_cursor + -1) == 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "9879\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        newline = 2;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "9880\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        newline = 1;
                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "9882\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      newline = 1;
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9884\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    compiler_globals.increment_lineno = (unsigned char)1;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9885\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    language_scanner_globals.yy_state = 8;
                    goto heredoc_scan_done;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "9886\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9893\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9895\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9915\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 36: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9899\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        break;
      } else {
        goto _L___8;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9905\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___8: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9900\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          break;
        } else {
          goto _L___7;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9904\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___7: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9903\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "9902\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(language_scanner_globals.yy_cursor) == 123) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "9901\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
      }
    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9916\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 123: 
    if ((int )*(language_scanner_globals.yy_cursor) == 36) {
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9906\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9917\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 92: 
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9911\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) != 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9909\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) != 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9907\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (language_scanner_globals.yy_cursor) ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "9908\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9910\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9912\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9918\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9922\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  heredoc_scan_done: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_scan_escape_string(zendlval, (char *)language_scanner_globals.yy_text,
                          (int )(language_scanner_globals.yy_leng - (unsigned int )newline),
                          (char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )314);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy105: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 36) {
    goto yy107;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9923\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy104;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy106: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12360\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy104;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy107: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12362\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12363\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.lval = (long )'{';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12364\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12365\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9924\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9925\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 1U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12366\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )379);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12367\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy109: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12368\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12369\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12370\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9926\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9927\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12371\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12372\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___2[(int )yych] & 128) {
    goto yy109;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9928\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12373\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 45) {
    goto yy114;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9929\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12374\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 91) {
    goto yy116;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9930\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12375\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy111: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12376\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9931\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___6 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9932\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___6,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9933\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___6;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9934\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___14 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9935\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___14;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9936\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12377\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12378\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12379\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy112: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12380\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12381\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(5);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12382\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )378);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12383\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy114: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12384\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12385\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 62) {
    goto yy118;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9937\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12386\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy115: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  goto yy111;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12387\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy116: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12388\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12389\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9938\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9939\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_leng) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(6);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9940\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___7 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9941\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___7,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9942\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___7;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9943\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___15 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9944\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___15;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy118: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9949\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy115;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9946\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9950\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy119;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9947\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 94) {
      goto yy115;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9948\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9955\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy115;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9952\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9956\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy119;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9953\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 126) {
      goto yy115;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy119: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9958\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = ((unsigned char *)((char *)language_scanner_globals.yy_text) + language_scanner_globals.yy_leng) - 3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9959\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng -= 3U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9960\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___8 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9961\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___8,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9962\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___8;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9963\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___16 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9964\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___16;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9965\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_IN_SCRIPTING: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 16) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9966\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9967\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  switch ((int )yych) {
  case 0x00: 
  case 0x01: 
  case 0x02: 
  case 0x03: 
  case 0x04: 
  case 0x05: 
  case 0x06: 
  case 0x07: 
  case 0x08: 
  case 11: 
  case 12: 
  case 0x0E: 
  case 0x0F: 
  case 0x10: 
  case 0x11: 
  case 0x12: 
  case 0x13: 
  case 0x14: 
  case 0x15: 
  case 0x16: 
  case 0x17: 
  case 0x18: 
  case 0x19: 
  case 0x1A: 
  case 0x1B: 
  case 0x1C: 
  case 0x1D: 
  case 0x1E: 
  case 0x1F: 
  goto yy183;
  case 9: 
  case 10: 
  case 13: 
  case 32: 
  goto yy139;
  case 33: 
  goto yy152;
  case 34: 
  goto yy179;
  case 35: 
  goto yy175;
  case 36: 
  goto yy164;
  case 37: 
  goto yy158;
  case 38: 
  goto yy159;
  case 39: 
  goto yy177;
  case 40: 
  goto yy146;
  case 41: 
  case 44: 
  case 59: 
  case 64: 
  case 91: 
  case 93: 
  case 126: 
  goto yy165;
  case 42: 
  goto yy155;
  case 43: 
  goto yy151;
  case 45: 
  goto yy137;
  case 46: 
  goto yy157;
  case 47: 
  goto yy156;
  case 48: 
  goto yy171;
  case 49: 
  case 50: 
  case 51: 
  case 52: 
  case 53: 
  case 54: 
  case 55: 
  case 56: 
  case 57: 
  goto yy173;
  case 58: 
  goto yy141;
  case 60: 
  goto yy153;
  case 61: 
  goto yy149;
  case 62: 
  goto yy154;
  case 63: 
  goto yy166;
  case 65: 
  case 97: 
  goto yy132;
  case 66: 
  case 98: 
  goto yy134;
  case 67: 
  case 99: 
  goto yy127;
  case 68: 
  case 100: 
  goto yy125;
  case 69: 
  case 101: 
  goto yy123;
  case 70: 
  case 102: 
  goto yy126;
  case 71: 
  case 103: 
  goto yy135;
  case 73: 
  case 105: 
  goto yy130;
  case 76: 
  case 108: 
  goto yy150;
  case 78: 
  case 110: 
  goto yy144;
  case 79: 
  case 111: 
  goto yy162;
  case 80: 
  case 112: 
  goto yy136;
  case 82: 
  case 114: 
  goto yy128;
  case 83: 
  case 115: 
  goto yy133;
  case 84: 
  case 116: 
  goto yy129;
  case 85: 
  case 117: 
  goto yy147;
  case 86: 
  case 118: 
  goto yy145;
  case 87: 
  case 119: 
  goto yy131;
  case 88: 
  case 120: 
  goto yy163;
  case 92: 
  goto yy142;
  case 94: 
  goto yy161;
  case 95: 
  goto yy148;
  case 96: 
  goto yy181;
  case 123: 
  goto yy167;
  case 124: 
  goto yy160;
  case 125: 
  goto yy169;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9968\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  goto yy174;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy123: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  switch ((int )yych) {
  case 67: 
  case 99: 
  goto yy726;
  case 76: 
  case 108: 
  goto yy727;
  case 77: 
  case 109: 
  goto yy728;
  case 78: 
  case 110: 
  goto yy729;
  case 86: 
  case 118: 
  goto yy730;
  case 88: 
  case 120: 
  goto yy731;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9969\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  goto yy186;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12408\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy124: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12409\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9970\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___9 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9971\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___9,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9972\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___9;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9973\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___17 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___17;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12410\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12411\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )307);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy125: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12413\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12414\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9982\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 72) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9977\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 69) {
        goto yy708;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9976\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9980\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 73) {
        goto yy709;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9978\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9981\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 78) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9979\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy710;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 104) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9984\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 101) {
        goto yy708;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9987\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 105) {
        goto yy709;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9988\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy710;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9986\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12415\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy126: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12416\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12417\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 85) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "9996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 78) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9991\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 73) {
        goto yy687;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9990\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9994\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 79) {
        goto yy688;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9992\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9995\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 84) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9993\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy689;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10003\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 110) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "9998\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 105) {
        goto yy687;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9997\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10001\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 111) {
        goto yy688;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "9999\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10002\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 117) {
        goto yy689;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10000\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12418\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy127: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10010\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 75) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10005\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 65) {
        goto yy652;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10004\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10008\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 76) {
        goto yy653;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10006\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10009\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 78) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10007\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy654;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10017\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 107) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10012\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 97) {
        goto yy652;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10011\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10015\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 108) {
        goto yy653;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10013\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10016\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy654;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10014\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy128: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12422\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12423\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy634;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10018\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy634;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10019\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy129: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 82) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10022\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 72) {
      goto yy622;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10020\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10023\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 81) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10021\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy623;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10028\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 104) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10025\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 103) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10024\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy622;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10027\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 114) {
        goto yy623;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10026\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy130: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10037\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 76) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10030\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 70) {
        goto yy569;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10029\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10034\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 77) {
        goto yy571;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10031\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10035\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 78) {
        goto yy572;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10032\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10036\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 82) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10033\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy573;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 109) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10040\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 102) {
        goto yy569;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10038\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10041\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 108) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10039\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy571;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10044\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 110) {
        goto yy572;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10042\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10045\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 115) {
        goto yy573;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10043\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy131: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12432\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy564;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10047\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 104) {
    goto yy564;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10048\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy132: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 77) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10050\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 66) {
        goto yy546;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10049\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10054\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 78) {
        goto yy547;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10051\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10055\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 81) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10052\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10056\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 82) {
        goto yy548;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10053\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy549;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10068\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 110) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10060\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 98) {
        goto yy546;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10058\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10061\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 109) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10059\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy547;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10065\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 113) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10062\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10066\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 114) {
        goto yy548;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10063\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10067\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 115) {
        goto yy549;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10064\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy133: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 87) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 84) {
      goto yy534;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10069\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 86) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10070\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy535;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10077\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 116) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 115) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10073\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy534;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10076\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 119) {
        goto yy535;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10075\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy134: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 59) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10082\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 34) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10079\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 33) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10078\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy526;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10081\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 39) {
        goto yy527;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10080\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10089\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 82) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 60) {
        goto yy525;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10083\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 81) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10084\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy528;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10088\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 114) {
        goto yy528;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10087\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy135: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10092\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 76) {
      goto yy515;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10090\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 78) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10091\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy516;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10098\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 108) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10095\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 107) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10094\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy515;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10097\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy516;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10096\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy136: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 85) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10101\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 82) {
      goto yy491;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10099\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10102\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 84) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10100\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy492;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10107\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 114) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10104\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 113) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10103\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy491;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10106\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 117) {
        goto yy492;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10105\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy137: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 60) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 45) {
      goto yy487;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10108\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10112\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 61) {
      goto yy485;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10110\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10113\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 62) {
      goto yy489;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10111\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy138: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )*((char *)language_scanner_globals.yy_text + 0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy139: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy484;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy140: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10124\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p___1 = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary___1 = p___1 + language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10126\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10121\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p___1 < (unsigned int )boundary___1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10114\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10122\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p___1 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10115\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10120\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p___1 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10118\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p___1 + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10116\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10117\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10119\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10123\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p___1 ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )375);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy141: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 58) {
    goto yy481;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10127\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy142: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )384);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy144: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12473\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 69) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10130\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 65) {
      goto yy469;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10128\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10131\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 68) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10129\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy470;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10136\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10133\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10132\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy469;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10135\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 101) {
        goto yy470;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10134\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy145: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy466;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10137\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12477\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 97) {
    goto yy466;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10138\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12478\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy146: 
  yyaccept = 1U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10157\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 68) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10147\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 32) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10141\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 9) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10139\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10142\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 0x1F) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10140\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10145\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 64) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10143\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10146\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 67) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10144\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10156\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 73) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10150\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 70) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10148\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10151\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 72) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10149\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10154\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 79) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10152\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10155\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 81) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10153\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10177\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 102) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10166\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 98) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10160\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 85) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10158\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10161\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 96) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10159\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10164\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 100) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10162\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10165\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 101) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10163\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10176\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 111) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10169\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 105) {
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10167\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10170\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 110) {
          goto yy138;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10168\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy391;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10175\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 115) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10172\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 113) {
            goto yy138;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10171\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy391;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10174\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych == 117) {
            goto yy391;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10173\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy138;
        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy147: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10180\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 78) {
      goto yy382;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10178\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10181\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 82) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10179\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy383;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10186\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 110) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10183\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 109) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10182\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy382;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10185\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 115) {
        goto yy383;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10184\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy148: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 95) {
    goto yy300;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10187\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy149: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12490\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12491\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 60) {
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10188\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12492\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 61) {
    goto yy294;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10189\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 62) {
    goto yy296;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10190\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy150: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy290;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10191\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 105) {
    goto yy290;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy151: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 43) {
    goto yy288;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10193\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy286;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10194\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy152: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12504\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy283;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12505\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy153: 
  yyaccept = 1U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 59) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10197\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 47) {
      goto yy255;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10196\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 60) {
      goto yy253;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10198\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 61) {
      goto yy256;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10199\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 62) {
      goto yy258;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10200\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy138;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy154: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12512\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 60) {
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12513\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 61) {
    goto yy249;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12514\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 62) {
    goto yy247;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12515\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy155: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12516\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12517\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy245;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12518\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy156: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12519\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12520\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 46) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10209\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 42) {
      goto yy237;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10208\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      goto yy239;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10210\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10213\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 61) {
      goto yy240;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10211\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy138;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12521\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy157: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12522\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12523\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12524\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    goto yy233;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12525\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy235;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12526\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy158: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12527\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12528\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 60) {
    goto yy138;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12529\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 61) {
    goto yy229;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10218\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12530\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 62) {
    goto yy227;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10219\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12531\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy159: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 38) {
    goto yy223;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10220\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12534\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy225;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12535\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy160: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy221;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12538\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 124) {
    goto yy219;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10223\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12539\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy161: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12540\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12541\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy217;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10224\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12542\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy162: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy215;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy215;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy163: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy212;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 111) {
    goto yy212;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy164: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10232\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy138;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10229\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10233\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy209;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10230\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10234\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 94) {
      goto yy138;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10231\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy209;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10238\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy138;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10235\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10239\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy209;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10236\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 126) {
      goto yy138;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10237\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy209;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy165: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy166: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 62) {
    goto yy205;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy167: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'{');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy169: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.doc_comment);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10243\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10244\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.doc_comment_len = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___18 = zend_stack_is_empty((zend_stack const   *)(& language_scanner_globals.state_stack));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___18) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10245\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10246\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    yy_pop_state();
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12568\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'}');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12569\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy171: 
  yyaccept = 2U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 69) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10255\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10249\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 46) {
        goto yy187;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10247\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10250\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy190;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10248\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10253\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 66) {
        goto yy198;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10251\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10254\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 69) {
        goto yy192;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10252\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10265\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 98) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10258\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 88) {
        goto yy197;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10256\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10259\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 98) {
        goto yy198;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10257\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10264\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 101) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10261\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych >= 101) {
          goto yy192;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10260\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10263\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 120) {
          goto yy197;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10262\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy172: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.yy_leng < 10U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10266\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.lval = strtol((char const   */* __restrict  */)((char *)language_scanner_globals.yy_text),
                                  (char **/* __restrict  */)((void *)0), 0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10275\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___19 = __errno_location();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10276\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___19 = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10277\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.lval = strtol((char const   */* __restrict  */)((char *)language_scanner_globals.yy_text),
                                  (char **/* __restrict  */)((void *)0), 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10278\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___22 = __errno_location();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10279\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (*tmp___22 == 34) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10271\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*((char *)language_scanner_globals.yy_text + 0) == 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10267\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___20 = zend_oct_strtod((char const   *)((char *)language_scanner_globals.yy_text),
                                   (char const   **)((void *)0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10268\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.dval = (double )tmp___20;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10269\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___21 = zend_strtod((char const   *)((char *)language_scanner_globals.yy_text),
                               (char const   **)((void *)0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10270\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.dval = (double )tmp___21;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10272\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->type = (unsigned char)2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10273\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )306);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10274\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )305);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy173: 
  yyaccept = 2U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10282\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 46) {
      goto yy187;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10280\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10283\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      goto yy172;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10281\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy190;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10288\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 69) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10285\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 68) {
        goto yy172;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10284\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy192;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10287\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 101) {
        goto yy192;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10286\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy172;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy174: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy175: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy176: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10300\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10289\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10301\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___23 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10302\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___23) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10295\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 13: 
    if ((int )*(language_scanner_globals.yy_cursor) == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10290\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10291\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10296\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 10: 
    (compiler_globals.zend_lineno) ++;
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10297\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 37: 
    if (! compiler_globals.asp_tags) {
      continue;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10292\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10298\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 63: 
    if ((int )*(language_scanner_globals.yy_cursor) == 62) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10293\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) --;
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10294\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10299\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )370);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12590\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy177: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12591\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy178: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12592\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + 0) != 39) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10303\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___24 = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10304\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___24 = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12593\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  bprefix = tmp___24;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12594\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10317\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10314\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) == 39) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10305\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (language_scanner_globals.yy_cursor) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10306\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10311\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___25 = language_scanner_globals.yy_cursor;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10312\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (language_scanner_globals.yy_cursor) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10313\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*tmp___25 == 92) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10309\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10307\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (language_scanner_globals.yy_cursor) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10308\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10310\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10315\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_limit - language_scanner_globals.yy_text);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10316\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )314);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12595\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___26 = _estrndup((char const   *)(((char *)language_scanner_globals.yy_text + bprefix) + 1),
                       (language_scanner_globals.yy_leng - (unsigned int )bprefix) - 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)tmp___26;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )((language_scanner_globals.yy_leng - (unsigned int )bprefix) - 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  t = zendlval->value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  s = t;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end___0 = s + zendlval->value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10339\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )s < (unsigned int )end___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10318\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10340\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s == 92) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10329\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      s ++;
      switch ((int )*s) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10319\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 92: 
      case 39: 
      tmp___27 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10320\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10321\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___27 = *s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10322\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (zendlval->value.str.len) --;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10323\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      default: 
      tmp___28 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10324\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10325\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___28 = (char )'\\';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10326\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___29 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10327\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10328\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___29 = *s;
      break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10330\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___30 = t;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10331\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      t ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10332\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___30 = *s;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10341\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10333\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.zend_lineno) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10338\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*s == 13) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10336\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(s + 1) != 10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10334\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.zend_lineno) ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10335\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10337\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10342\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *t = (char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10343\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___10 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10344\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s = zendlval->value.str.val;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10345\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___10,
                                                (unsigned char const   *)((unsigned char *)s),
                                                (unsigned int )zendlval->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10346\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___10;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10347\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)s);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10348\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )315);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy179: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy180: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12608\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + 0) != 34) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10349\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___31 = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10350\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___31 = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12609\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  bprefix___0 = tmp___31;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12610\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10351\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10370\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___32 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10371\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___32) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10362\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 34: 
    language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10363\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_scan_escape_string(zendlval,
                            ((char *)language_scanner_globals.yy_text + bprefix___0) + 1,
                            (int )((language_scanner_globals.yy_leng - (unsigned int )bprefix___0) - 2U),
                            (char )'\"');
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10364\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )315);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10365\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 36: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10352\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        break;
      } else {
        goto _L___10;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10358\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___10: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10353\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          break;
        } else {
          goto _L___9;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10357\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___9: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10356\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10355\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )*(language_scanner_globals.yy_cursor) == 123) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "10354\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
      }
    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10366\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 123: 
    if ((int )*(language_scanner_globals.yy_cursor) == 36) {
      break;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10359\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    continue;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10367\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 92: 
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10360\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10361\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10372\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) --;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.doc_comment_len = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text) - language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_text + language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'\"');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy181: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )'`');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12619\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy183: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12620\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10373\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10374\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_error(1 << 7L,
             "Unexpected character in input:  \'%c\' (ASCII=%d) state=%d",
             *((char *)language_scanner_globals.yy_text + 0),
             *((char *)language_scanner_globals.yy_text + 0),
             language_scanner_globals.yy_state);
  goto restart;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy185: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10375\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10376\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12626\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy186: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10377\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy124;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12628\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy187: 
  yyaccept = 3U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12630\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10378\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10379\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 8) {
    goto yy187;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10380\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy192;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10381\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy192;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10382\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy189: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12637\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___33 = zend_strtod((char const   *)((char *)language_scanner_globals.yy_text),
                         (char const   **)((void *)0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12638\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.dval = (double )tmp___33;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12639\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12640\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )306);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12641\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy190: 
  yyaccept = 2U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12643\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12644\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10383\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10384\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12645\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12646\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10387\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 46) {
      goto yy187;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10385\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10388\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      goto yy172;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10386\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy190;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10393\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 69) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10390\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 68) {
        goto yy172;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10389\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10392\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 101) {
        goto yy172;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10391\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12647\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy192: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 44) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10395\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 43) {
      goto yy194;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10394\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10399\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 45) {
      goto yy194;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10400\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10401\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      goto yy195;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10398\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy193: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (yyaccept <= 2U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10403\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (yyaccept <= 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10402\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (yyaccept <= 0U) {
        goto yy124;
      } else {
        goto yy138;
      }
    } else {
      goto yy172;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10405\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (yyaccept <= 4U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10404\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (yyaccept <= 3U) {
        goto yy189;
      } else {
        goto yy238;
      }
    } else {
      goto yy254;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy194: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10406\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych >= 58) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10407\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy195: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10408\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12658\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy189;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    goto yy195;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy189;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy197: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 32) {
    goto yy202;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10412\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12664\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy198: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12665\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12666\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 16) {
    goto yy199;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12667\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy199: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12668\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10414\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10415\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12669\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12670\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 16) {
    goto yy199;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10416\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12671\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  bin = (char *)language_scanner_globals.yy_text + 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len = (int )(language_scanner_globals.yy_leng - 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12674\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10418\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*bin == 48) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10417\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10419\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    bin ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10420\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    len --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12675\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (len < 32) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10421\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.lval = strtol((char const   */* __restrict  */)bin,
                                  (char **/* __restrict  */)((void *)0), 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10422\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10423\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )305);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10424\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___34 = zend_bin_strtod((char const   *)bin, (char const   **)((void *)0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10425\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.dval = (double )tmp___34;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10426\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10427\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )306);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12676\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy202: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12677\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10428\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10429\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12678\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12679\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 32) {
    goto yy202;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10430\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hex = (char *)language_scanner_globals.yy_text + 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len___0 = (int )(language_scanner_globals.yy_leng - 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*hex == 48) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10431\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10433\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hex ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    len___0 --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (len___0 < 8) {
    goto _L___11;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10449\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (len___0 == 8) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10444\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*hex <= 55) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10437\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___11: /* CIL Label */ 
        if (len___0 == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10435\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zendlval->value.lval = 0L;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10436\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zendlval->value.lval = strtol((char const   */* __restrict  */)hex,
                                        (char **/* __restrict  */)((void *)0),
                                        16);
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10438\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->type = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10439\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((int __attribute__((__visibility__("default")))  )305);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10440\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___35 = zend_hex_strtod((char const   *)hex,
                                   (char const   **)((void *)0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10441\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.dval = (double )tmp___35;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10442\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10443\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((int __attribute__((__visibility__("default")))  )306);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10445\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___35 = zend_hex_strtod((char const   *)hex,
                                 (char const   **)((void *)0));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10446\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->value.dval = (double )tmp___35;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10447\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->type = (unsigned char)2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )306);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy205: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy207;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10450\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 13) {
    goto yy208;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10451\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy206: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_state = 7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )374);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy207: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy206;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy208: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy207;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10452\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy206;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy209: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12701\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10453\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10454\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12702\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12703\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10461\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10456\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy209;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10455\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10459\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy211;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10457\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10460\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy209;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10458\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10468\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10463\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 95) {
        goto yy209;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10462\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10466\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy209;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10464\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10467\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy209;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10465\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12704\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy211: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12705\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10469\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___11 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10470\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___11,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10471\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___11;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10472\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___36 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10473\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___36;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10474\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12706\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12707\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12708\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy212: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12709\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy213;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10475\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12712\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy213: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12713\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12714\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12715\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12716\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )264);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12717\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy215: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12718\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12719\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10478\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12720\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12721\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )263);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12722\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy217: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12723\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )269);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy219: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )278);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy221: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )270);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy223: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )279);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12734\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy225: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12735\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12736\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )271);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12737\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy227: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy231;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10479\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 13) {
    goto yy232;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy228: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.asp_tags) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 7;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10482\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10483\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10484\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10485\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )374);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10486\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10487\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      language_scanner_globals.yy_leng = 1U;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10489\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )*((char *)language_scanner_globals.yy_text + 0));
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy229: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )272);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy231: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy228;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy232: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy231;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10490\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy228;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy233: 
  yyaccept = 3U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 68) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10495\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      goto yy189;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10493\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10496\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      goto yy233;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10494\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy189;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10499\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 69) {
      goto yy192;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10497\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10500\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 101) {
      goto yy192;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10498\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy189;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12757\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy235: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12758\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12759\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )273);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12760\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy237: 
  yyaccept = 4U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12761\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12763\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12764\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 42) {
    goto yy242;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10501\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12765\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy238: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12766\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.yy_leng > 2U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    doc_com = 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10506\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.doc_comment) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10502\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)compiler_globals.doc_comment);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10503\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.doc_comment = (char *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10504\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10507\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10508\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    doc_com = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12767\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10513\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10509\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___37 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*tmp___37 == 42) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10511\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) == 47) {
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10510\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10512\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12768\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 7L, "Unterminated comment starting line %d",
               compiler_globals.zend_lineno);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p___2 = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary___2 = p___2 + language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10531\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10526\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p___2 < (unsigned int )boundary___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10519\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10527\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p___2 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10520\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10525\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p___2 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10523\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p___2 + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10521\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10522\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10524\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10528\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p___2 ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (doc_com) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10532\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___38 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10533\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)tmp___38;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10534\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10535\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )371);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10536\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12772\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )370);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12773\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy239: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12774\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy176;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12775\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy240: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12776\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12777\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )274);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12778\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy242: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 64) {
    goto yy243;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10537\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy243: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12782\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10538\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10539\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12783\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12784\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 64) {
    goto yy243;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10540\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy238;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12785\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy245: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12786\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12787\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )275);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12788\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy247: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12789\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12790\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy251;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12791\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12792\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )286);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12793\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy249: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12794\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12795\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )284);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12796\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy251: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12797\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12798\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )267);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12799\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy253: 
  yyaccept = 5U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12800\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12801\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12802\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12803\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 59) {
    goto yy254;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 60) {
    goto yy269;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10543\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 61) {
    goto yy267;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10544\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy254: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12807\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )287);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12808\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy255: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12809\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12810\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy260;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10545\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12811\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 115) {
    goto yy260;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10546\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12812\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy256: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12813\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12814\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )285);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12815\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy258: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12816\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy259: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12817\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )282);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12818\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy260: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy261;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10547\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10548\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12822\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy261: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12823\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12824\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy262;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10549\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12825\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12826\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy262: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12827\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12828\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy263;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12829\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12830\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy263: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12831\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy264;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10553\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10554\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy264: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12835\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12836\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy265;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12837\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12838\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy265: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12839\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10557\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10558\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12840\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12841\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 13) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 0x08) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10559\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 10) {
      goto yy265;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10560\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10564\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 12) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10561\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy265;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10566\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 0x1F) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10565\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy265;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10568\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 62) {
        goto yy205;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10567\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy267: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12843\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12844\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )268);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12845\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy269: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12846\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10570\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10571\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12848\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 128) {
    goto yy269;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10572\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12849\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 90) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10579\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 38) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10574\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 34) {
        goto yy274;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10573\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10577\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 39) {
        goto yy273;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10575\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10578\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10576\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10586\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10581\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 95) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10580\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10584\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy271;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10582\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10585\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 126) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10583\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12850\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy271: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12851\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10587\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10588\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12852\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12853\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 64) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10597\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 12) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10590\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 10) {
        goto yy278;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10589\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10594\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 13) {
        goto yy280;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10591\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10595\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 47) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10592\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10596\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy271;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10593\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10608\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 95) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10600\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy271;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10598\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10601\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 94) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10599\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy271;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10605\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10602\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10606\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy271;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10603\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 126) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10604\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy271;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12854\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy273: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12855\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12856\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 39) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12857\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy282;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10610\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12858\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10611\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy282;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12859\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy274: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 34) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10612\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy276;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10613\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10614\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy276;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12864\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy275: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12865\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10615\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10616\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy276: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 90) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10623\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10618\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 34) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10617\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10621\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy275;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10619\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10622\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10620\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy275;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10625\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 95) {
        goto yy275;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10624\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10628\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy275;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10626\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10629\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 126) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10627\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy275;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12869\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy277: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12870\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12871\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy278;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10631\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12872\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 13) {
    goto yy280;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10632\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12873\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy278: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12874\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy279: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12875\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + 0) != 60) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10633\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___39 = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10634\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___39 = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12876\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  bprefix___1 = tmp___39;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12877\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = compiler_globals.heredoc;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12878\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = compiler_globals.heredoc_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12879\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (compiler_globals.zend_lineno) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12880\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*((char *)language_scanner_globals.yy_text + (language_scanner_globals.yy_leng - 2U)) == 13) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10635\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___40 = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10636\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___40 = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12881\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc_len = (int )((((language_scanner_globals.yy_leng - (unsigned int )bprefix___1) - 3U) - 1U) - (unsigned int )tmp___40);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12882\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  s___0 = ((char *)language_scanner_globals.yy_text + bprefix___1) + 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12883\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s___0 == 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10637\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10639\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*s___0 == 9) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10638\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s___0 ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10642\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.heredoc_len) --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12884\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*s___0 == 39) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10643\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s___0 ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.heredoc_len -= 2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 9;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10649\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*s___0 == 34) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10646\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      s___0 ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10647\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.heredoc_len -= 2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10648\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10650\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_state = 4;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12885\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___41 = _estrndup((char const   *)s___0,
                       (unsigned int )compiler_globals.heredoc_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12886\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.heredoc = (char *)tmp___41;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12887\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.heredoc_len < language_scanner_globals.yy_limit - language_scanner_globals.yy_cursor) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10661\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___42 = memcmp((void const   *)language_scanner_globals.yy_cursor,
                      (void const   *)s___0,
                      (unsigned int )compiler_globals.heredoc_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___42) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10651\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10658\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      end___1 = language_scanner_globals.yy_cursor + compiler_globals.heredoc_len;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10659\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*end___1 == 59) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10652\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        end___1 ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10653\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10660\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*end___1 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10654\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        language_scanner_globals.yy_state = 8;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10657\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*end___1 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10655\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          language_scanner_globals.yy_state = 8;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10656\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12888\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )376);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12889\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy280: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12890\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12891\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 10) {
    goto yy278;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy279;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12892\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy281: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12893\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10665\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10666\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12894\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12895\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy282: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12896\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 90) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10673\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 47) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10668\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 39) {
        goto yy277;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10667\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy281;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10669\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10672\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10670\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy281;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10680\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10675\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 95) {
        goto yy281;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10674\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10678\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy281;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10676\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10679\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 126) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10677\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy281;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy283: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12899\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 61) {
    goto yy259;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10681\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12900\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12901\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12902\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )280);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12903\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy286: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12904\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12905\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )277);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12906\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy288: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12907\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12908\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )297);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12909\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy290: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12910\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12911\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy291;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10682\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12912\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10683\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12913\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy291: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12914\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12915\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy292;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10684\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12916\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10685\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12917\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy292: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12918\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12919\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10686\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12920\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12921\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )361);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12922\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy294: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12923\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12924\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 61) {
    goto yy298;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12925\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12926\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )283);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12927\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy296: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12928\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12929\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )360);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12930\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy298: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12931\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12932\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )281);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12933\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy300: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12934\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  switch ((int )yych) {
  case 67: 
  case 99: 
  goto yy302;
  case 68: 
  case 100: 
  goto yy307;
  case 70: 
  case 102: 
  goto yy304;
  case 72: 
  case 104: 
  goto yy301;
  case 76: 
  case 108: 
  goto yy306;
  case 77: 
  case 109: 
  goto yy305;
  case 78: 
  case 110: 
  goto yy308;
  case 84: 
  case 116: 
  goto yy303;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "10688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  goto yy186;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12935\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy301: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12936\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy369;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10689\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 97) {
    goto yy369;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10690\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy302: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12941\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy362;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10691\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12942\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy362;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10692\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12943\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy303: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12944\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12945\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy355;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10693\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12946\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy355;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10694\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12947\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy304: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12948\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12949\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 85) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 73) {
      goto yy339;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10695\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10698\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 84) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10696\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy340;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 105) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10700\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 104) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10699\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy339;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10702\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 117) {
        goto yy340;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10701\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12950\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy305: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12951\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12952\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy331;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12953\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy331;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12954\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy306: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12955\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12956\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy325;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10706\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12957\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 105) {
    goto yy325;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10707\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12958\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy307: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12959\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12960\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy320;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12961\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 105) {
    goto yy320;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12962\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy308: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12963\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12964\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy309;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10710\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12965\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10711\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12966\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy309: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12967\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12968\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 77) {
    goto yy310;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10712\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12969\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 109) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10713\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12970\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy310: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12971\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12972\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy311;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10714\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12973\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12974\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy311: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12975\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12976\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy312;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12977\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10717\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12978\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy312: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12979\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12980\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy313;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12981\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10719\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12982\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy313: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12983\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12984\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy314;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10720\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12985\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12986\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy314: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12987\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12988\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy315;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12989\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12990\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy315: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12991\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12992\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy316;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12993\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10725\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12994\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy316: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12995\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12996\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10726\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12997\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12998\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "12999\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10727\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13000\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10728\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13003\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13004\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10729\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *zendlval = *(compiler_globals.current_namespace);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10730\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(zendlval);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10736\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10731\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = zendlval;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10732\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.len = 0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10733\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___43 = _estrndup("", sizeof("") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.val = (char *)tmp___43;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10735\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)6;
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13005\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )382);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13006\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy320: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13007\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13008\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy321;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13009\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10738\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy321: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10739\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13013\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13014\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10740\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10741\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___44 = zend_get_compiled_filename();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  filename = (char *)tmp___44;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___45 = strlen((char const   *)filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  filename_len = tmp___45;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13024\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! filename) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10742\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    filename = (char *)"";
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10743\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___46 = _estrndup((char const   *)filename, filename_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dirname = (char *)tmp___46;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_dirname(dirname, filename_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s1_len = strlen((char const   *)dirname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s2_len = strlen(".");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! ((unsigned int )((void const   *)(dirname + 1)) - (unsigned int )((void const   *)dirname) == 1U)) {
      goto _L___13;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10750\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (__s1_len >= 4U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10748\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___13: /* CIL Label */ 
        if (! ((unsigned int )((void const   *)("." + 1)) - (unsigned int )((void const   *)".") == 1U)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10744\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___54 = 1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10747\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s2_len >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10745\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___54 = 1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10746\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___54 = 0;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10749\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___54 = 0;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___54) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10751\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___49 = __builtin_strcmp((char const   *)dirname, ".");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10752\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___53 = tmp___49;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10753\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___52 = __builtin_strcmp((char const   *)dirname, ".");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10754\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___53 = tmp___52;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10759\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___52 = __builtin_strcmp((char const   *)dirname, ".");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10760\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___53 = tmp___52;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___53 == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10761\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___47 = _erealloc((void *)dirname, 4096U, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10762\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    dirname = (char *)tmp___47;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10763\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    getcwd(dirname, 4096U);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10764\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___55 = strlen((char const   *)dirname);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )tmp___55;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = dirname;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13033\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13034\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )383);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13035\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy325: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13036\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13037\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy326;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13038\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy326: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy327;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy327: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13045\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10769\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13046\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13047\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13048\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10770\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13049\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13050\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13051\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13052\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13053\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.lval = (long )compiler_globals.zend_lineno;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13054\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13055\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )368);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13056\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy331: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13057\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13058\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy332;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13059\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10773\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy332: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy333;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10774\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10775\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy333: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy334;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10776\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10777\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy334: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy335;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10778\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 100) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10779\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy335: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10780\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10781\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10782\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_class_entry) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10783\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___56 = (compiler_globals.active_class_entry)->name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10784\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___56 = (char const   *)((void *)0);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  class_name = tmp___56;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_op_array) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10785\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___57 = (compiler_globals.active_op_array)->function_name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10786\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___57 = (char const   *)((void *)0);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13085\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  func_name = tmp___57;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13086\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len___1 = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13087\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___58 = strlen(class_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    len___1 += tmp___58 + 2U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13088\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (func_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10790\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___59 = strlen(func_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10791\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    len___1 += tmp___59;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10792\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13089\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (func_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10793\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___60 = func_name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10794\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___60 = "";
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13090\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (func_name) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___61 = "::";
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10796\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___61 = "";
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___61 = "";
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___62 = class_name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___62 = "";
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = zend_spprintf(& zendlval->value.str.val, 0,
                                          (char *)"%s%s%s", tmp___62, tmp___61,
                                          tmp___60);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )366);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy339: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy350;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy350;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy340: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy341;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10804\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy341: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy342;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10805\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10806\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy342: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13108\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13109\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy343;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10807\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13110\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10808\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy343: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy344;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10809\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10810\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy344: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13116\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13117\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy345;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10811\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10812\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13119\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy345: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13120\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13121\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy346;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10813\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10814\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy346: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13124\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13125\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10817\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  func_name___0 = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_op_array) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10818\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    func_name___0 = (compiler_globals.active_op_array)->function_name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10819\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! func_name___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10820\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    func_name___0 = "";
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10821\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___63 = strlen(func_name___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )tmp___63;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___64 = _estrndup(func_name___0, (unsigned int )zendlval->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)tmp___64;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )367);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy350: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy351;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10822\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10823\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy351: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10824\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10825\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10826\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13156\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___65 = zend_get_compiled_filename();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  filename___0 = (char *)tmp___65;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! filename___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10827\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    filename___0 = (char *)"";
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10828\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___66 = strlen((char const   *)filename___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )tmp___66;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___67 = _estrndup((char const   *)filename___0,
                       (unsigned int )zendlval->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)tmp___67;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )369);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy355: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy356;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10829\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10830\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy356: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy357;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10831\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10832\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy357: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy358;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10833\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10834\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy358: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10835\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10836\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10837\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_name = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_class_entry) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (288U == ((compiler_globals.active_class_entry)->ce_flags & 288U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10838\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      trait_name = (compiler_globals.active_class_entry)->name;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10839\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! trait_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10842\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    trait_name = "";
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10843\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___68 = strlen(trait_name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )tmp___68;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___69 = _estrndup(trait_name, (unsigned int )zendlval->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)tmp___69;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )365);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy362: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy363;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10844\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10845\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy363: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13201\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13202\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy364;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10846\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10847\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy364: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy365;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10848\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10849\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy365: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10850\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10851\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13215\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10852\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  class_name___0 = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_class_entry) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (288U == ((compiler_globals.active_class_entry)->ce_flags & 288U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10853\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->value.str.len = (int )(sizeof("__CLASS__") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10854\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___70 = _estrndup("__CLASS__", (unsigned int )zendlval->value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->value.str.val = (char *)tmp___70;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10856\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->type = (unsigned char)8;
    } else {
      goto _L___14;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10862\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___14: /* CIL Label */ 
    if (compiler_globals.active_class_entry) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      class_name___0 = (compiler_globals.active_class_entry)->name;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10859\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10863\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! class_name___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10860\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      class_name___0 = "";
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10861\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10864\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___71 = strlen(class_name___0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10865\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )tmp___71;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10866\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___72 = _estrndup(class_name___0, (unsigned int )zendlval->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10867\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___72;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10868\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)6;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13220\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )364);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13221\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy369: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13222\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13223\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy370;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10869\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13224\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10870\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13225\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy370: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13227\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy371;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10871\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10872\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13229\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy371: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13230\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 95) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10873\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13233\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13234\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy373;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10874\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13235\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10875\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13236\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy373: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy374;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10876\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10877\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy374: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 77) {
    goto yy375;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10878\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 109) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10879\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy375: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy376;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10880\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10881\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13248\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy376: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13249\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13250\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy377;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10882\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13251\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10883\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13252\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy377: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13253\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13254\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy378;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10884\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10885\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy378: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13258\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy379;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10886\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13259\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10887\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13260\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy379: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13262\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy380;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10888\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13263\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10889\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy380: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10890\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13267\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13268\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )353);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13269\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy382: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13270\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13271\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy386;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10891\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13272\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 115) {
    goto yy386;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10892\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13273\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy383: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13274\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy384;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10894\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy384: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10895\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )340);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy386: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy387;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10896\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13285\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10897\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13286\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy387: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy388;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10898\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10899\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy388: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13292\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10900\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13293\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13294\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )350);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy390: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 7) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10901\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10902\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy391: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10927\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 68) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10914\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 32) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10905\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 9) {
          goto yy390;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10903\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10906\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 0x1F) {
          goto yy193;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10904\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy390;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10913\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 65) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10908\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 64) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10907\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy395;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10911\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 66) {
            goto yy393;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10909\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10912\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 67) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10910\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy398;
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10926\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 73) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10917\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 70) {
          goto yy399;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10915\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10918\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 72) {
          goto yy193;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10916\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy400;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10925\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 79) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10920\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 78) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10919\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy394;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10923\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 81) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10921\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10924\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 82) {
            goto yy397;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10922\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy396;
        }
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 102) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10939\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 97) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10930\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 85) {
          goto yy392;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10928\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10931\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 96) {
          goto yy193;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10929\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy395;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10938\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 99) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10933\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 98) {
            goto yy393;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10932\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy193;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10936\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 100) {
            goto yy398;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10934\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10937\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 101) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10935\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy399;
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10950\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 113) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10944\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 105) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10941\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 104) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10940\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy400;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10943\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych == 111) {
            goto yy394;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10942\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy193;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10949\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 115) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10946\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych <= 114) {
            goto yy397;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10945\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          goto yy396;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "10948\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )yych != 117) {
            goto yy193;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "10947\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy392: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy459;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10952\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 110) {
    goto yy459;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10953\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy393: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10956\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 73) {
      goto yy446;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 78) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10955\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy447;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10962\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 105) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10959\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 104) {
        goto yy193;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10958\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy446;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10961\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy447;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "10960\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy193;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy394: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13309\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 66) {
    goto yy438;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10963\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13310\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 98) {
    goto yy438;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10964\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13311\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy395: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy431;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10965\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy431;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10966\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy396: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy423;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10967\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 116) {
    goto yy423;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10968\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy397: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy421;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10969\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy421;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10970\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy398: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy417;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10971\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 111) {
    goto yy417;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10972\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13327\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy399: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13328\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13329\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy410;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10973\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13330\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy410;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13331\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy400: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy401;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10976\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy401: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13337\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy402;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10977\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13338\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10978\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13339\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy402: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13340\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy403;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10979\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy405;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10980\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13343\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy403: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13344\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13345\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 71) {
    goto yy408;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10981\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13346\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 103) {
    goto yy408;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10982\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13347\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy404: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13348\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10983\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10984\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13349\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13350\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy405: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13351\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10986\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy404;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10985\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy404;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10987\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10990\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "10988\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )295);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy408: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy409;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10991\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10992\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy409: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13360\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy404;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13362\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy404;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10994\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13363\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy410: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13364\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13365\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy411;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10995\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13366\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13367\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy411: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13368\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13369\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy412;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10997\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13370\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10998\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13371\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy412: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13372\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13373\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy413;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "10999\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13374\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11000\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13375\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy413: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13376\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11001\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11002\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13377\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13378\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11004\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy413;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11003\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11007\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy413;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11005\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11008\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11006\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13379\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13380\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13381\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )294);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13382\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy417: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13383\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13384\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy418;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11009\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13385\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11010\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13386\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy418: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13387\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13388\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 66) {
    goto yy419;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11011\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13389\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 98) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11012\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy419: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy420;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11013\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11014\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy420: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy413;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11015\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy413;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11016\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy421: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy422;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11017\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11018\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy422: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy413;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11019\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy413;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11020\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy423: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13408\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy424;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11021\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13409\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11022\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13410\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy424: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13411\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy425;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11023\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13413\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11024\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13414\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy425: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13415\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13416\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy426;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11025\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13417\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11026\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13418\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy426: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 71) {
    goto yy427;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11027\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 103) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11028\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13422\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy427: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13423\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11029\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11030\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11032\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy427;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11031\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy427;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11033\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11036\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11034\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )293);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy431: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy432;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11037\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13432\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11038\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy432: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy433;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11039\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11040\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy433: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 89) {
    goto yy434;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11041\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 121) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11042\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy434: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11043\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11044\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy434;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11045\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11049\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy434;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11047\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11050\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11048\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )292);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy438: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 74) {
    goto yy439;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11051\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 106) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11052\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy439: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy440;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11053\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11054\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy440: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy441;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11055\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11056\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy441: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy442;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11058\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy442: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11059\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11060\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11062\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy442;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11061\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11065\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy442;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11063\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11066\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11064\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )291);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy446: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13473\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy456;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11067\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 110) {
    goto yy456;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11068\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy447: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13477\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy448;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11069\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13478\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11070\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy448: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy449;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy449: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy454;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11073\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy454;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11074\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy451;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy450: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11075\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11076\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13490\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy451: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13491\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11078\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy450;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11077\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11081\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy450;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11079\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11082\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11080\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13492\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )290);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy454: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy455;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11083\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11084\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy455: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy450;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11085\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 110) {
    goto yy450;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11086\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy456: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13504\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13505\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy457;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11087\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11088\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy457: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy458;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11089\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11090\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy458: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13512\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13513\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 89) {
    goto yy427;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11091\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13514\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 121) {
    goto yy427;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11092\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13515\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy459: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13516\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13517\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy460;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13518\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11094\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13519\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy460: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13520\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13521\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy461;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11095\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13522\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11096\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13523\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy461: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13524\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13525\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy462;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11097\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13526\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11098\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13527\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy462: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13528\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11099\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11100\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13529\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13530\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 0x1F) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11102\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 9) {
      goto yy462;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11101\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy193;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11105\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      goto yy462;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11103\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11106\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych != 41) {
      goto yy193;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11104\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13531\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )289);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13534\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy466: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13535\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy467;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11107\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11108\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13538\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy467: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13539\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13540\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13541\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13542\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )349);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy469: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 77) {
    goto yy473;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 109) {
    goto yy473;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy470: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 87) {
    goto yy471;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11112\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 119) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11113\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy471: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11114\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )299);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy473: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy474;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11115\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy474: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy475;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11118\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy475: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy476;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11119\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 112) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11120\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13568\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy476: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13569\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy477;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11121\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11122\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy477: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy478;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11123\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11124\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy478: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy479;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11126\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy479: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11127\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )381);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy481: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )380);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy483: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11128\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11129\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13590\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13591\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy484: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13592\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 12) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11132\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 0x08) {
      goto yy140;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11130\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11133\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 10) {
      goto yy483;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11131\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy140;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11136\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 13) {
      goto yy483;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11134\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11137\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 32) {
      goto yy483;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11135\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy140;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13593\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy485: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13594\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13595\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )276);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy487: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )296);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy489: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )359);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy491: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11140\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 73) {
      goto yy498;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11138\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11141\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 78) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11139\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy499;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11146\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 105) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11143\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 104) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11142\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy498;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11145\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy499;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11144\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy492: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13608\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 66) {
    goto yy493;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11147\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13609\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 98) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11148\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13610\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy493: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy494;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11149\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11150\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy494: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy495;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11151\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11152\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy495: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13619\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13620\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy496;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11153\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11154\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy496: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11155\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13626\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )343);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy498: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13628\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 86) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11158\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 78) {
      goto yy507;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11156\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11159\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 85) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11157\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy508;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11164\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 110) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11161\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 109) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11160\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy507;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11163\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 118) {
        goto yy508;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11162\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13630\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy499: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy500;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11165\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11166\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy500: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy501;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11167\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13637\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11168\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13638\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy501: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13639\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13640\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy502;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11169\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13641\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11170\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy502: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13643\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13644\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy503;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11171\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13645\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11172\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13646\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy503: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13647\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy504;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11173\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11174\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy504: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy505;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11175\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 100) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11176\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy505: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11177\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13658\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )344);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy507: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy513;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11178\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 116) {
    goto yy513;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11179\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy508: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13664\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13665\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy509;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11180\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13666\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11181\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13667\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy509: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13668\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13669\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy510;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11182\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13670\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11183\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13671\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy510: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy511;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11184\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13674\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11185\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13675\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy511: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13676\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13677\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11186\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13678\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13679\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )345);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy513: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11187\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )266);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy515: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy520;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11188\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 111) {
    goto yy520;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11189\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy516: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy517;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11190\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11191\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy517: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy518;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11193\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy518: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11194\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13701\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )333);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13702\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy520: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13703\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13704\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 66) {
    goto yy521;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13705\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 98) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11196\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13706\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy521: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13707\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13708\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy522;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11197\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13709\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11198\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy522: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13712\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy523;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11199\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13713\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11200\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13714\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy523: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13715\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13716\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13717\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13718\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )342);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13719\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy525: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13720\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13721\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 60) {
    goto yy533;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13722\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy526: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13723\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy180;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy527: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy178;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy528: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy529;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy529: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy530;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13734\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy530: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13735\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13736\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 75) {
    goto yy531;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13737\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 107) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11208\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy531: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11209\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )331);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy533: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 60) {
    goto yy269;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11210\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy193;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy534: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy541;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11211\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 97) {
    goto yy541;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy535: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy536;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11213\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy536: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy537;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13757\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13758\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy537: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13759\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13760\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy538;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13761\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11218\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy538: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13763\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13764\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy539;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11219\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13765\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11220\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13766\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy539: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13767\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13768\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )327);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy541: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13772\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13773\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy542;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13774\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11223\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13775\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy542: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13776\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13777\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy543;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11224\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13778\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy543: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy544;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13782\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13783\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy544: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13784\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13785\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13786\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13787\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )348);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13788\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy546: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13789\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13790\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy557;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11229\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13791\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 115) {
    goto yy557;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11230\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13792\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy547: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13793\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13794\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy555;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11231\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13795\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 100) {
    goto yy555;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11232\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13796\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy548: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13797\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13798\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy551;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11233\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13799\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy551;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11234\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13800\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy549: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13801\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13802\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11235\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13803\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )326);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy551: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13807\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy552;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11236\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13808\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11237\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13809\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy552: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13810\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13811\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 89) {
    goto yy553;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11238\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13812\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 121) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11239\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13813\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy553: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13814\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13815\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13816\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13817\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )362);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13818\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy555: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13822\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )265);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13823\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy557: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13824\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13825\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy558;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13826\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11243\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13827\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy558: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13828\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13829\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy559;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11244\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13830\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11245\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13831\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy559: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy560;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11246\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11247\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13835\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy560: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13836\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13837\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy561;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11248\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13838\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13839\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy561: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13840\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13841\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy562;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11251\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13843\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy562: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13844\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13845\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11252\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13846\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )347);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13848\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy564: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13849\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13850\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy565;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11253\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13851\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11254\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13852\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy565: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13853\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13854\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy566;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11255\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13855\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11256\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13856\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy566: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13857\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13858\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy567;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11257\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13859\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11258\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy567: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11259\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13864\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )318);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13865\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy569: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11260\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13869\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )301);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13870\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy571: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13871\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13872\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy613;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11261\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13873\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 112) {
    goto yy613;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11262\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13874\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy572: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13875\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13876\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 84) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11269\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 67) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11264\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 66) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11263\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy580;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11267\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 82) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11265\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11268\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 83) {
        goto yy578;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11266\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy579;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11276\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 114) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11271\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 99) {
        goto yy580;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11270\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11274\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 115) {
        goto yy578;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11272\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11275\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 116) {
        goto yy579;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11273\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13877\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy573: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13878\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13879\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy574;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11277\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13880\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11278\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13881\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy574: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13882\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13883\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy575;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11279\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13884\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11280\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13885\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy575: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13886\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13887\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy576;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11281\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13888\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11282\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13889\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy576: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13890\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13891\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11283\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13892\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13893\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )351);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13894\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy578: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13895\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13896\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy599;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11284\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 116) {
    goto yy599;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11285\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy579: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13899\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13900\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy592;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11286\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13901\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy592;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11287\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13902\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy580: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13903\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13904\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy581;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11288\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13905\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11289\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13906\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy581: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13907\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13908\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy582;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11290\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13909\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11291\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13910\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy582: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13911\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13912\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy583;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11292\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13913\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 100) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11293\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13914\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy583: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13915\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13916\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy584;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11294\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13917\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11295\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13918\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy584: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13919\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13920\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11302\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11297\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11296\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11300\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy585;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11298\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11301\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11299\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11309\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11304\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 95) {
        goto yy586;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11303\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11307\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11305\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11308\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11306\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13921\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy585: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13922\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )262);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13923\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy586: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13924\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13925\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy587;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11310\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13926\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13927\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy587: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13928\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13929\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy588;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11312\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13930\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11313\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13931\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy588: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13932\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13933\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy589;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11314\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13934\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11315\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13935\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy589: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13936\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy590;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11316\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11317\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy590: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13941\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11318\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13942\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13943\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )261);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13944\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy592: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13945\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13946\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy593;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11319\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13947\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11320\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13948\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy593: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13949\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13950\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 70) {
    goto yy594;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11321\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13951\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 102) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11322\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13952\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy594: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13953\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13954\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy595;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11323\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13955\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11324\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13956\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy595: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13957\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13958\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy596;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11325\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13959\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11326\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13960\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy596: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13961\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13962\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy597;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11327\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13963\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11328\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13964\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy597: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13965\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13966\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11329\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13967\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13968\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )356);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13969\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy599: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13970\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13971\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 69) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11332\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 65) {
      goto yy600;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11330\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11333\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 68) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11331\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy601;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11338\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11335\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11334\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11337\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 101) {
        goto yy601;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11336\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13972\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy600: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13973\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13974\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy607;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11339\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13975\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 110) {
    goto yy607;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11340\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13976\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy601: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13977\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13978\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy602;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11341\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13979\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11342\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13980\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy602: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13981\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13982\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy603;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11343\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13983\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 100) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11344\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13984\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy603: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13985\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13986\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy604;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11345\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13987\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11346\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13988\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy604: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13989\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13990\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 70) {
    goto yy605;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11347\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13991\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 102) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11348\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13992\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy605: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13993\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13994\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11349\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13995\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13996\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )341);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13997\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy607: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13998\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "13999\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy608;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11350\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14000\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11351\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy608: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14003\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy609;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11352\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14004\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11353\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14005\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy609: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14006\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14007\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy610;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11354\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14008\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11355\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14009\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy610: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 70) {
    goto yy611;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11356\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 102) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11357\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14013\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy611: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14014\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11358\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )288);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy613: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy614;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11359\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11360\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy614: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14024\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy615;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11361\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11362\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy615: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 77) {
    goto yy616;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11363\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 109) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11364\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy616: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy617;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11365\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14033\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11366\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14034\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy617: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14035\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14036\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy618;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11367\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14037\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14038\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy618: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy619;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11370\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy619: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy620;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11371\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14045\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11372\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14046\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy620: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14047\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14048\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11373\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14049\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14050\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )358);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14051\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy622: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14052\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14053\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy630;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11374\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14054\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy630;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11375\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14055\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy623: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14056\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14057\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 89) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11378\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 65) {
      goto yy626;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11376\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11379\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 88) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11377\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11384\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11381\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11380\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy626;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11383\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 121) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11382\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14058\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14059\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11385\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )337);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy626: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy627;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11386\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11387\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy627: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy628;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11388\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11389\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy628: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11390\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )355);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy630: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy631;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11391\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11392\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy631: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 87) {
    goto yy632;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11393\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 119) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11394\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy632: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14085\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14086\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11395\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14087\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14088\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )339);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14089\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy634: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14090\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 84) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11398\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 81) {
      goto yy636;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11399\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 83) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11404\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 113) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11401\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 112) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11400\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy636;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11403\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 116) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11402\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy648;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11405\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 117) {
    goto yy648;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11406\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy636: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy637;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11407\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11408\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy637: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy638;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy638: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy639;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11412\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14108\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy639: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14109\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14110\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy640;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11414\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy640: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11421\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11416\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11415\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11419\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy641;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11417\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11420\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11418\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11428\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11423\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 95) {
        goto yy642;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11422\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11426\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11424\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11427\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11425\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy641: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14116\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )259);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14117\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy642: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14119\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy643;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11429\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14120\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11430\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14121\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy643: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy644;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11431\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14124\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14125\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy644: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy645;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11433\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy645: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy646;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11435\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11436\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy646: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11437\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )258);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy648: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy649;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11438\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11439\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy649: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy650;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11440\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11441\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy650: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11442\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )336);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy652: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 84) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11449\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 76) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11444\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 75) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11443\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy675;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11447\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 82) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11445\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 83) {
        goto yy674;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11446\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy673;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11456\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 114) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11451\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 108) {
        goto yy675;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11450\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11454\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 115) {
        goto yy674;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11452\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11455\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 116) {
        goto yy673;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11453\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy653: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14156\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 79) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11459\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 65) {
      goto yy665;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11457\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11460\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 78) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11458\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy666;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11465\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11462\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11461\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy665;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11464\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 111) {
        goto yy666;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11463\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy654: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy655;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11466\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11467\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy655: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 84) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11470\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 82) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11468\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11471\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 84) {
      goto yy657;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11469\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11475\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 114) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11472\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 115) {
      goto yy656;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11473\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 116) {
      goto yy657;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11474\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy186;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy656: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy663;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11478\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 116) {
    goto yy663;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11479\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy657: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy658;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy658: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy659;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11482\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11483\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy659: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy660;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11484\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11485\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy660: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy661;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11486\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11487\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy661: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )332);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy663: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11489\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )335);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy665: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy670;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11490\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 115) {
    goto yy670;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy666: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy667;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14201\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11493\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14202\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy667: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy668;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11494\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11495\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy668: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11496\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )298);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy670: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy671;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11497\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11498\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14215\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy671: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11499\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )354);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14220\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy673: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14221\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14222\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy684;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11500\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14223\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 99) {
    goto yy684;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11501\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14224\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy674: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14225\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy682;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11502\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14227\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy682;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11503\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy675: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14229\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14230\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy676;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy676: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14233\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14234\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy677;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11506\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14235\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11507\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14236\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy677: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 66) {
    goto yy678;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11508\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 98) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11509\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy678: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy679;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11510\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11511\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy679: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy680;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11512\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11513\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14248\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy680: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14249\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14250\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14251\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14252\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )363);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14253\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy682: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14254\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )329);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14258\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy684: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14259\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14260\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy685;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14262\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy685: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14263\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )338);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14267\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy687: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14268\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14269\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy704;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11519\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14270\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 110) {
    goto yy704;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11520\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14271\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy688: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14272\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14273\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy697;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11521\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14274\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 114) {
    goto yy697;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11522\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy689: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy690;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11523\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11524\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy690: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy691;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11525\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11526\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy691: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14285\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy692;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11527\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14286\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11528\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy692: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy693;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy693: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14292\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14293\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy694;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11531\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14294\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11532\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy694: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy695;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11533\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11534\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy695: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11535\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )334);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy697: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11544\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11538\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 47) {
        goto yy698;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11536\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11539\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11537\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11542\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 69) {
        goto yy699;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11540\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11543\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11541\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11553\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 100) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11546\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 96) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11545\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11550\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 101) {
        goto yy699;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11547\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11551\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11548\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11552\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11549\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy698: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )320);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14309\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy699: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14310\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14311\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy700;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11554\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy700: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy701;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11557\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy701: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy702;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11558\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11559\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy702: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11560\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )322);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy704: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14327\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14328\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy705;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11561\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14329\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14330\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy705: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14331\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy706;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11564\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy706: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11565\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14337\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14338\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )346);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14339\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy708: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14340\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 70) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11568\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 67) {
      goto yy714;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11566\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 69) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11567\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy715;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11574\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 99) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11571\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 98) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11570\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy714;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11573\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 102) {
        goto yy715;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11572\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy709: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14343\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14344\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy712;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11575\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14345\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy712;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11576\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14346\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy710: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14347\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14348\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11577\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14349\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14350\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )317);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14351\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy712: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11578\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )300);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy714: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy721;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11579\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 108) {
    goto yy721;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11580\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14360\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy715: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14362\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy716;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11581\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14363\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11582\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14364\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy716: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14365\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14366\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 85) {
    goto yy717;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11583\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14367\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 117) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11584\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14368\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy717: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14369\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14370\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy718;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11585\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14371\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11586\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14372\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy718: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14373\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14374\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy719;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11587\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14375\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11588\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14376\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy719: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14377\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14378\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11589\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14379\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14380\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )330);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14381\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy721: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14382\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14383\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy722;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11590\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14384\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14385\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy722: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14386\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14387\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy723;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11592\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14388\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11593\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14389\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy723: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy724;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11594\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11595\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy724: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11596\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )324);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy726: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy788;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11597\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 104) {
    goto yy788;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11598\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy727: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy782;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11599\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 115) {
    goto yy782;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11600\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy728: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14408\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 80) {
    goto yy778;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11601\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14409\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 112) {
    goto yy778;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11602\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14410\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy729: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14411\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy744;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11603\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14413\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 100) {
    goto yy744;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11604\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14414\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy730: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14415\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14416\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy741;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11605\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14417\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 97) {
    goto yy741;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11606\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14418\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy731: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 84) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych == 73) {
      goto yy732;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11610\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 83) {
      goto yy186;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11608\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    goto yy733;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11615\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 105) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11612\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 104) {
        goto yy186;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11611\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11614\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 116) {
        goto yy733;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11613\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy186;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy732: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14422\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14423\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy739;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11616\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 116) {
    goto yy739;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy733: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy734;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy734: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 78) {
    goto yy735;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14432\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 110) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11621\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy735: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 68) {
    goto yy736;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11622\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 100) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11623\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy736: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 83) {
    goto yy737;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 115) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy737: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )357);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy739: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11627\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )300);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy741: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy742;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11629\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy742: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )260);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy744: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  switch ((int )yych) {
  case 68: 
  case 100: 
  goto yy745;
  case 70: 
  case 102: 
  goto yy746;
  case 73: 
  case 105: 
  goto yy747;
  case 83: 
  case 115: 
  goto yy748;
  case 87: 
  case 119: 
  goto yy749;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "11631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  goto yy186;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy745: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy771;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11632\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 101) {
    goto yy771;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11633\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy746: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy763;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11634\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 111) {
    goto yy763;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11635\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy747: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 70) {
    goto yy761;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11636\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14473\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 102) {
    goto yy761;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy748: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 87) {
    goto yy755;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14477\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 119) {
    goto yy755;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11639\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy186;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14478\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy749: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy750;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy750: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy751;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11642\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11643\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy751: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy752;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14490\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy752: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14491\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14492\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy753;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11646\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11647\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy753: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11648\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )319);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy755: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 73) {
    goto yy756;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11649\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 105) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11650\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy756: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14504\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14505\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy757;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11651\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11652\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy757: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy758;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11653\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11654\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy758: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14512\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14513\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy759;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11655\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14514\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11656\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14515\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy759: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14516\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14517\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11657\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14518\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14519\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )328);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14520\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy761: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14521\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14522\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11658\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14523\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14524\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )304);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14525\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy763: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14526\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14527\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy764;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11659\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14528\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11660\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14529\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy764: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14530\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14531\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11663\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 47) {
        goto yy765;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11661\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11664\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11662\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11667\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 69) {
        goto yy766;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11665\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11668\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11666\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11678\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 100) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 96) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11670\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11675\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 101) {
        goto yy766;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11672\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11676\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11673\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11677\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11674\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy765: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )321);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14534\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy766: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14535\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy767;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11679\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11680\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14538\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy767: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14539\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14540\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy768;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11681\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14541\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11682\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14542\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy768: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 72) {
    goto yy769;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11683\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 104) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11684\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy769: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11685\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )323);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy771: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 67) {
    goto yy772;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11686\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 99) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy772: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 76) {
    goto yy773;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11688\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 108) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11689\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy773: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 65) {
    goto yy774;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11690\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 97) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11691\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy774: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 82) {
    goto yy775;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11692\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 114) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11693\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy775: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14568\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14569\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy776;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11694\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11695\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy776: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11696\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )325);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy778: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 84) {
    goto yy779;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 116) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11698\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy779: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 89) {
    goto yy780;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11699\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 121) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11700\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy780: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )352);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy782: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14590\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14591\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 69) {
    goto yy783;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14592\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 101) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14593\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy783: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14594\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14595\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11712\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 47) {
        goto yy784;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11704\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11707\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 57) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11705\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11710\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 73) {
        goto yy785;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11708\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11711\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11709\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 104) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11714\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych != 96) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11713\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11718\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 105) {
        goto yy785;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11715\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11719\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11716\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11720\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy185;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11717\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy784: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )303);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy785: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 70) {
    goto yy786;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 102) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy786: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )302);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy788: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14608\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14609\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 79) {
    goto yy789;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11725\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14610\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych != 111) {
    goto yy186;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11726\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy789: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___3[(int )yych] & 4) {
    goto yy185;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11727\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )316);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_LOOKING_FOR_PROPERTY: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11728\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11729\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 45) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11740\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 13) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11733\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 0x08) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11730\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 10) {
        goto yy793;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11731\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11735\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 12) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11732\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11738\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 32) {
        goto yy793;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11736\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11739\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 44) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11737\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy795;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11753\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 95) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11744\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11741\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11745\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy797;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11742\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11746\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 94) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11743\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy797;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11750\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 96) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11747\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11751\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy797;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11748\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11752\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 126) {
        goto yy799;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11749\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      goto yy797;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14619\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy793: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14620\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy805;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy794: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)language_scanner_globals.yy_text;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11764\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p___3 = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary___3 = p___3 + language_scanner_globals.yy_leng;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11761\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p___3 < (unsigned int )boundary___3) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11754\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11762\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p___3 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11755\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11760\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p___3 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11758\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p___3 + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11756\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11757\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11759\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11763\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p___3 ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14626\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )375);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy795: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14628\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych == 62) {
    goto yy802;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14630\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy796: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11769\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 0U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  goto restart;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy797: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy801;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy798: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14637\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11770\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___12 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___12,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___12;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11773\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___73 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11774\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___73;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11775\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14638\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14639\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )307);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14640\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy799: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14641\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy796;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy800: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14643\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11776\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11777\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14644\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14645\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy801: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14646\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___4[(int )yych] & 64) {
    goto yy800;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11778\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy798;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14647\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy802: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )359);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy804: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11779\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11780\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy805: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___4[(int )yych] & 128) {
    goto yy804;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11781\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy794;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_LOOKING_FOR_VARNAME: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 2) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11782\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11783\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy810;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11784\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy808;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11785\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 94) {
      goto yy810;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11786\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11793\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy810;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11790\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11794\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy808;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11791\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11795\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 126) {
      goto yy810;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11792\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14658\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy808: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy813;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy809: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11796\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___13 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___13,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___13;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___74 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___74;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14664\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14665\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )308);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14666\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy810: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14667\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14668\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 0U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14669\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14670\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _yy_push_state(0);
  goto restart;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14671\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy812: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11804\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11805\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14674\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy813: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14675\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___5[(int )yych] & 128) {
    goto yy812;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11806\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy809;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14676\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_NOWDOC: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14677\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11807\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11808\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14678\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14679\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  newline___0 = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11809\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11810\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11842\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )language_scanner_globals.yy_cursor < (unsigned int )language_scanner_globals.yy_limit) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11811\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11843\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___75 = language_scanner_globals.yy_cursor;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11844\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (language_scanner_globals.yy_cursor) ++;
    switch ((int )*tmp___75) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 13: 
    if ((int )*(language_scanner_globals.yy_cursor) == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11812\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (language_scanner_globals.yy_cursor) ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11813\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 10: 
    if ((int )*(language_scanner_globals.yy_cursor) >= 97) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11814\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(language_scanner_globals.yy_cursor) <= 122) {
        goto _L___16;
      } else {
        goto _L___18;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11838\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___18: /* CIL Label */ 
      if ((int )*(language_scanner_globals.yy_cursor) >= 65) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11815\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*(language_scanner_globals.yy_cursor) <= 90) {
          goto _L___16;
        } else {
          goto _L___17;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11837\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___17: /* CIL Label */ 
        if ((int )*(language_scanner_globals.yy_cursor) == 95) {
          goto _L___16;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11836\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(language_scanner_globals.yy_cursor) >= 0x7F) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11834\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___16: /* CIL Label */ 
            if (compiler_globals.heredoc_len < language_scanner_globals.yy_limit - language_scanner_globals.yy_cursor) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "11831\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___76 = memcmp((void const   *)language_scanner_globals.yy_cursor,
                                (void const   *)compiler_globals.heredoc,
                                (unsigned int )compiler_globals.heredoc_len);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "11832\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___76) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11816\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11828\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                end___2 = language_scanner_globals.yy_cursor + compiler_globals.heredoc_len;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11829\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*end___2 == 59) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "11817\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  end___2 ++;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "11818\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11830\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*end___2 == 10) {
                  goto _L___15;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "11827\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*end___2 == 13) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "11823\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L___15: /* CIL Label */ 
                    if ((int )*(language_scanner_globals.yy_cursor + -2) == 13) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "11821\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int )*(language_scanner_globals.yy_cursor + -1) == 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "11819\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        newline___0 = 2;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "11820\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        newline___0 = 1;
                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "11822\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      newline___0 = 1;
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "11824\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    compiler_globals.increment_lineno = (unsigned char)1;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "11825\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    language_scanner_globals.yy_state = 8;
                    goto nowdoc_scan_done;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "11826\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "11833\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11835\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    continue;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  nowdoc_scan_done: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11845\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___14 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11846\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___14,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                language_scanner_globals.yy_leng - (unsigned int )newline___0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11847\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___14;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11848\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___77 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng - (unsigned int )newline___0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11849\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___77;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11850\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - (unsigned int )newline___0);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11861\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p___4 = (char *)language_scanner_globals.yy_text;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11862\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    boundary___4 = p___4 + (language_scanner_globals.yy_leng - (unsigned int )newline___0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11863\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )p___4 < (unsigned int )boundary___4) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11851\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11859\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*p___4 == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11852\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.zend_lineno) ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11857\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*p___4 == 13) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11855\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*(p___4 + 1) != 10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11853\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.zend_lineno) ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11854\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11856\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11860\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      p___4 ++;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )314);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yyc_ST_VAR_OFFSET: 
  if ((unsigned int )(language_scanner_globals.yy_cursor + 3) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11864\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11865\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11882\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11874\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 12) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11868\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 0x08) {
          goto yy832;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11866\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11869\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 10) {
          goto yy828;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11867\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy832;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11872\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 13) {
          goto yy828;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11870\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11873\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 0x1F) {
          goto yy832;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11871\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy828;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11881\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 36) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11877\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 34) {
          goto yy827;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11875\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11878\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 35) {
          goto yy828;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11876\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy823;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11880\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych == 39) {
          goto yy828;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11879\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy827;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11903\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 92) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11891\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11885\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 48) {
          goto yy820;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11883\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11886\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 57) {
          goto yy822;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11884\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy827;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11889\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 90) {
          goto yy830;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11887\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11890\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 91) {
          goto yy827;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11888\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy828;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11902\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 95) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11894\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 93) {
          goto yy825;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11892\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11895\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 94) {
          goto yy827;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11893\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy830;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11899\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 96) {
          goto yy827;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11896\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11900\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 122) {
          goto yy830;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11897\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11901\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )yych <= 126) {
          goto yy827;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11898\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        goto yy830;
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy820: 
  yyaccept = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_marker = language_scanner_globals.yy_cursor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_marker);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 87) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11908\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11905\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy844;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11904\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11907\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 66) {
        goto yy841;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11906\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11915\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 98) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11911\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 88) {
        goto yy843;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11909\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11912\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 98) {
        goto yy841;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11910\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11914\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych == 120) {
        goto yy843;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11913\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy821: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.yy_leng < 10U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11916\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.lval = strtol((char const   */* __restrict  */)((char *)language_scanner_globals.yy_text),
                                  (char **/* __restrict  */)((void *)0), 10);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11917\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->type = (unsigned char)1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11947\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (language_scanner_globals.yy_leng == 10U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11941\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11929\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s1_len___0 = strlen((char const   *)((char *)language_scanner_globals.yy_text));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11930\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s2_len___0 = strlen(long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11931\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! ((unsigned int )((void const   *)((char *)language_scanner_globals.yy_text + 1)) - (unsigned int )((void const   *)((char *)language_scanner_globals.yy_text)) == 1U)) {
          goto _L___20;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11924\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s1_len___0 >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11922\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___20: /* CIL Label */ 
            if (! ((unsigned int )((void const   *)(long_min_digits + 1)) - (unsigned int )((void const   *)(long_min_digits)) == 1U)) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "11918\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___85 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "11921\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (__s2_len___0 >= 4U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11919\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___85 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "11920\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___85 = 0;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "11923\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___85 = 0;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11932\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___85) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11925\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___80 = __builtin_strcmp((char const   *)((char *)language_scanner_globals.yy_text),
                                      long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11926\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___84 = tmp___80;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11927\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___83 = __builtin_strcmp((char const   *)((char *)language_scanner_globals.yy_text),
                                      long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "11928\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___84 = tmp___83;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11933\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___83 = __builtin_strcmp((char const   *)((char *)language_scanner_globals.yy_text),
                                    long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11934\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___84 = tmp___83;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___84 < 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11935\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.lval = strtol((char const   */* __restrict  */)((char *)language_scanner_globals.yy_text),
                                      (char **/* __restrict  */)((void *)0), 10);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11936\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->type = (unsigned char)1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11937\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___78 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                             language_scanner_globals.yy_leng);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11938\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.str.val = (char *)tmp___78;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11939\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11940\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zendlval->type = (unsigned char)6;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___78 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                           language_scanner_globals.yy_leng);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11944\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->value.str.val = (char *)tmp___78;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11945\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11946\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zendlval->type = (unsigned char)6;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )310);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14701\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy822: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14702\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy840;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14703\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy823: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14704\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14705\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 95) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 64) {
      goto yy824;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11948\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11952\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 90) {
      goto yy836;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11949\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11953\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 95) {
      goto yy836;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11950\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      goto yy824;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11958\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 122) {
      goto yy836;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11955\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11959\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych >= 0x7F) {
      goto yy836;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11956\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14706\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy824: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14707\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )*((char *)language_scanner_globals.yy_text + 0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14708\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy825: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14709\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )']');
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14712\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy827: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14713\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy824;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14714\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy828: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14715\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14716\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11960\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_cursor = (unsigned char *)((char *)language_scanner_globals.yy_text) + 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11961\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    language_scanner_globals.yy_leng = 0U;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14717\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy_pop_state();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14718\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )314);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14719\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy830: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14720\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  goto yy835;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14721\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy831: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14722\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11962\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___15 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11963\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___15,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text)),
                                                language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11964\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___15;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11965\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___86 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                         language_scanner_globals.yy_leng);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11966\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___86;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11967\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14723\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )307);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy832: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )language_scanner_globals.yy_cursor > (unsigned int )language_scanner_globals.yy_limit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11968\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11969\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_error(1 << 7L,
             "Unexpected character in input:  \'%c\' (ASCII=%d) state=%d",
             *((char *)language_scanner_globals.yy_text + 0),
             *((char *)language_scanner_globals.yy_text + 0),
             language_scanner_globals.yy_state);
  goto restart;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy834: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11970\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11971\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy835: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 16) {
    goto yy834;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11972\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy831;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14734\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy836: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14735\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11973\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14736\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14737\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 94) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11981\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11976\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 48) {
        goto yy836;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11975\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11979\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 64) {
        goto yy838;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11977\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11980\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 90) {
        goto yy836;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11978\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )yych <= 96) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11983\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 95) {
        goto yy836;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych <= 122) {
        goto yy836;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11984\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "11987\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )yych >= 0x7F) {
        goto yy836;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "11985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy838: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (language_scanner_globals.output_filter) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    sz___16 = (size_t )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11990\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(language_scanner_globals.output_filter))((unsigned char **)(& zendlval->value.str.val),
                                                & sz___16,
                                                (unsigned char const   *)((unsigned char *)((char *)language_scanner_globals.yy_text + 1)),
                                                language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11991\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )sz___16;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11992\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___87 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text + 1),
                         language_scanner_globals.yy_leng - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.val = (char *)tmp___87;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11994\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zendlval->value.str.len = (int )(language_scanner_globals.yy_leng - 1U);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )309);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy839: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11995\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy840: ;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 32) {
    goto yy839;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11997\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy821;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy841: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 128) {
    goto yy849;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11998\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy842: 
  language_scanner_globals.yy_cursor = language_scanner_globals.yy_marker;
  goto yy821;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy843: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 64) {
    goto yy847;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "11999\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy842;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy844: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12000\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12001\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14757\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 47) {
    goto yy846;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12002\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14758\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )yych <= 57) {
    goto yy844;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12003\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14759\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy846: 
  language_scanner_globals.yy_leng = (unsigned int )(language_scanner_globals.yy_cursor - language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14760\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___88 = _estrndup((char const   *)((char *)language_scanner_globals.yy_text),
                       language_scanner_globals.yy_leng);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14761\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.val = (char *)tmp___88;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->value.str.len = (int )language_scanner_globals.yy_leng;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14763\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14764\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )310);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14765\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy847: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14766\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12004\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12005\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14767\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14768\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 64) {
    goto yy847;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12006\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy846;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yy849: 
  (language_scanner_globals.yy_cursor) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )(language_scanner_globals.yy_cursor + 1) >= (unsigned int )(language_scanner_globals.yy_limit + 32)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12007\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12008\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  yych = *(language_scanner_globals.yy_cursor);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "14772\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )yybm___6[(int )yych] & 128) {
    goto yy849;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-02-730b54a374-1953161b8c/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "12009\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  goto yy846;
}
}
